@@ -2,28 +2,11 @@
 #
 # Author: Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along
-# with this program; if not, see <http://www.gnu.org/licenses>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # Overrides are only supported when and as declared in the comments for
 # each constant.
 
-# Retry at most 5 times
-MAX_RETRY = 5
-
 # size of u-boot header to be removed from ramdisks, in bytes.
 UBOOT_DEFAULT_HEADER_LENGTH = 64
 
@@ -61,9 +44,6 @@ ACTION_TIMEOUT = 30
 # Max cleanup timeout
 CLEANUP_TIMEOUT = 5 * 60
 
-# Android tmp directory
-ANDROID_TMP_DIR = "/data/local/tmp"
-
 # LXC protocol name
 LXC_PROTOCOL = "lava-lxc"
 
@@ -118,9 +98,6 @@ UEFI_LINE_SEPARATOR = "\r\n"
 # excludes whitespace and punctuation (except hyphen and underscore)
 DEFAULT_TESTDEF_NAME_CLASS = r"^[\w\d\_\-]+$"
 
-# Limit repetitive messages
-METADATA_MESSAGE_LIMIT = 8192
-
 # Versatile Express autorun interrupt character
 VEXPRESS_AUTORUN_INTERRUPT_CHARACTER = " "
 
@@ -153,3 +130,8 @@ VALID_DISPATCHER_IP_PROTOCOLS = ["http", "nfs", "tftp"]
 # worker daemon data directory
 WORKER_DIR = "/var/lib/lava/dispatcher/worker"
 DOCKER_WORKER_DIR = "/var/lib/lava/dispatcher/docker-worker"
+
+# RequestDataTooBig error msg
+REQUEST_DATA_TOO_BIG_MSG = (
+    "The dataset provided is too large. Please reduce size and try again."
+)
