@@ -18,12 +18,15 @@ def schema():
         Optional("feedbacks"): [str],
         Required("image"): str,
         Optional("license_variable"): str,
+        Optional("environment"): [str],
         Optional("ubl_license"): str,
         Optional("version_string"): str,
         Required("arguments"): [str],
-        Required("prompts"): boot.prompts(),
+        Optional("prompts"): boot.prompts(),
         Required("docker"): docker("name"),
         Optional("transfer_overlay"): boot.transfer_overlay(),
         Optional("auto_login"): boot.auto_login(),
+        Optional("erpc_app"): str,
+        Optional("commands"): [str],
     }
     return {**boot.schema(), **base}
