@@ -5,11 +5,15 @@
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
-from voluptuous import Required
+from voluptuous import Optional, Required
 
 from lava_common.schemas import deploy
 
 
 def schema():
-    base = {Required("to"): "flasher", Required("images"): {str: deploy.url()}}
+    base = {
+        Required("to"): "flasher",
+        Required("images"): {str: deploy.url()},
+        Optional("uniquify"): bool,
+    }
     return {**deploy.schema(), **base}
