@@ -8,6 +8,7 @@ from __future__ import annotations
 import contextlib
 import os
 import re
+import time
 from typing import TYPE_CHECKING
 
 from lava_common.constants import (
@@ -28,7 +29,7 @@ from lava_common.timeout import Timeout
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
 from lava_dispatcher.connections.ssh import SShSession
-from lava_dispatcher.logical import Boot, RetryAction
+from lava_dispatcher.logical import RetryAction
 from lava_dispatcher.shell import ExpectShellSession
 from lava_dispatcher.utils.compression import untar_file
 from lava_dispatcher.utils.filesystem import write_bootscript
@@ -94,17 +95,11 @@ class LoginAction(Action):
             fail_msg=fail_msg,
             auto_login=auto_login,
         )
-        if len(parsed) and "success" in parsed[0]:
-            self.results = {"success": parsed[0]["success"]}
-            if len(parsed) > 1:
-                # errors detected.
-                self.logger.warning("Kernel warnings or errors detected.")
-                self.results = {"extra": parsed}
-        elif not parsed:
-            self.results = {"success": "No kernel warnings or errors detected."}
-        else:
-            self.results = {"fail": parsed}
+        if parsed:
             self.logger.warning("Kernel warnings or errors detected.")
+            self.results = {"extra": parsed}
+        else:
+            self.results = {"success": "No kernel warnings or errors detected."}
 
     def _check_prompt_characters(self, chk_prompt: str | type) -> None:
         if not isinstance(chk_prompt, str):
@@ -134,7 +129,7 @@ class LoginAction(Action):
             connection.prompt_str.append(failure)
 
         # linesep should come from deployment_data as from now on it is OS dependent
-        linesep = self.get_namespace_data(
+        linesep: str | None = self.get_namespace_data(
             action="deploy-device-env", label="environment", key="line_separator"
         )
         connection.raw_connection.linesep = linesep if linesep else LINE_SEPARATOR
@@ -146,18 +141,8 @@ class LoginAction(Action):
         params = self.parameters.get("auto_login")
         if not params:
             self.logger.debug("No login prompt set.")
-            # If auto_login is not enabled, login will time out if login
-            # details are requested.
-            connection.prompt_str.append(LOGIN_TIMED_OUT_MSG)
-            connection.prompt_str.append(LOGIN_INCORRECT_MSG)
             # wait for a prompt or kernel messages
             self.check_kernel_messages(connection, max_end_time, failure)
-            if "success" in self.results:
-                check = self.results["success"]
-                if LOGIN_TIMED_OUT_MSG in check or LOGIN_INCORRECT_MSG in check:
-                    raise JobError(
-                        "auto_login not enabled but image requested login details."
-                    )
             # clear kernel message prompt patterns
             connection.prompt_str = list(self.parameters.get("prompts", []))
             # already matched one of the prompts
@@ -170,12 +155,6 @@ class LoginAction(Action):
             self.check_kernel_messages(
                 connection, max_end_time, failure, auto_login=True
             )
-            if "success" in self.results:
-                if LOGIN_INCORRECT_MSG in self.results["success"]:
-                    self.logger.warning(
-                        "Login incorrect message matched before the login prompt. "
-                        "Please check that the login prompt is correct. Retrying login..."
-                    )
             self.logger.debug("Sending username %s", params["username"])
             connection.sendline(params["username"], delay=self.character_delay)
             # clear the kernel_messages patterns
@@ -369,7 +348,7 @@ class BootloaderCommandOverlay(Action):
         if self.commands is not None:
             return
         if self.parameters["method"] == "bootloader":
-            self.commands = self.parameters["commands"]
+            self.commands = self.parameters.get("commands", [])
         elif isinstance(self.parameters["commands"], list):
             self.commands = self.parameters["commands"]
             self.logger.warning(
@@ -421,7 +400,7 @@ class BootloaderCommandOverlay(Action):
         connection = super().run(connection, max_end_time)
         ip_addr = dispatcher_ip(self.job.parameters["dispatcher"])
 
-        self.ram_disk = self.get_namespace_data(
+        self.ram_disk: str | None = self.get_namespace_data(
             action="compress-ramdisk", label="file", key="ramdisk"
         )
         # most jobs substitute RAMDISK, so also use this for the initrd
@@ -430,7 +409,7 @@ class BootloaderCommandOverlay(Action):
                 action="download-action", label="file", key="initrd"
             )
 
-        substitutions = {
+        substitutions: dict[str, str | None] = {
             "{SERVER_IP}": ip_addr,
             "{PRESEED_CONFIG}": self.get_namespace_data(
                 action="download-action", label="file", key="preseed"
@@ -453,13 +432,18 @@ class BootloaderCommandOverlay(Action):
             "{DTB_BASE_ADDR}": None,
             "{DTB_BASE_RESIZE}": None,
             "{APPLY_DTBO_COMMANDS}": None,
+            "{LAVA_DISPATCHER_IP}": ip_addr,
+            "{LAVA_JOB_ID}": str(self.job.job_id),
         }
-        self.bootcommand = self.get_namespace_data(
-            action="uboot-prepare-kernel", label="bootcommand", key="bootcommand"
+        self.bootcommand = (
+            self.get_namespace_data(
+                action="uboot-prepare-kernel", label="bootcommand", key="bootcommand"
+            )
+            or ""
         )
         if not self.bootcommand and "type" in self.parameters:
             raise JobError("Kernel image type can't be determined")
-        prepared_kernel = self.get_namespace_data(
+        prepared_kernel: str | None = self.get_namespace_data(
             action="prepare-kernel", label="file", key="kernel"
         )
         if prepared_kernel:
@@ -481,7 +465,7 @@ class BootloaderCommandOverlay(Action):
                 dtbo_pos = self.commands.index("{APPLY_DTBO_COMMANDS}")
                 index = 0
                 while True:
-                    file = self.get_namespace_data(
+                    file: str | None = self.get_namespace_data(
                         action="download-action", label="file", key=f"dtbo{index}"
                     )
                     if file:
@@ -517,12 +501,19 @@ class BootloaderCommandOverlay(Action):
             add_header = self.job.device["actions"]["deploy"]["parameters"].get(
                 "add_header"
             )
-            if self.method == "u-boot" and not add_header == "u-boot":
+            if (
+                self.method == "u-boot"
+                and not add_header == "u-boot"
+                and not add_header == "raw"
+            ):
                 self.logger.debug("No u-boot header, not passing ramdisk to bootX cmd")
                 ramdisk_addr = "-"
 
-            if self.get_namespace_data(
-                action="download-action", label="file", key="initrd"
+            if (
+                self.get_namespace_data(
+                    action="download-action", label="file", key="initrd"
+                )
+                or add_header == "raw"
             ):
                 # no u-boot header, thus no embedded size, so we have to add it to the
                 # boot cmd with colon after the ramdisk
@@ -573,10 +564,10 @@ class BootloaderCommandOverlay(Action):
                 "tee_addr": tee_addr,
             }
 
-        nfs_address = self.get_namespace_data(
-            action="persistent-nfs-overlay", label="nfs_address", key="nfsroot"
+        nfs_address: str | None = self.get_namespace_data(
+            action="parse-persistent-nfs", label="nfs_address", key="nfsroot"
         )
-        nfs_root = self.get_namespace_data(
+        nfs_root: str | None = self.get_namespace_data(
             action="download-action", label="file", key="nfsrootfs"
         )
         if nfs_root:
@@ -587,7 +578,7 @@ class BootloaderCommandOverlay(Action):
         elif nfs_address:
             substitutions["{NFSROOTFS}"] = nfs_address
             substitutions["{NFS_SERVER_IP}"] = self.get_namespace_data(
-                action="persistent-nfs-overlay", label="nfs_address", key="serverip"
+                action="parse-persistent-nfs", label="nfs_address", key="serverip"
             )
 
         if "lava-xnbd" in self.parameters:
@@ -619,12 +610,14 @@ class BootloaderCommandOverlay(Action):
 
         if self.use_bootscript:
             script = "/script.ipxe"
-            bootscript = (
-                self.get_namespace_data(
-                    action="tftp-deploy", label="tftp", key="tftp_dir"
-                )
-                + script
+            tftp_dir: str | None = self.get_namespace_data(
+                action="tftp-deploy", label="tftp", key="tftp_dir"
             )
+            if tftp_dir is None:
+                raise JobError(
+                    "TFTP deployment not defined. Unable to create bootscript file."
+                )
+            bootscript = tftp_dir + script
             bootscripturi = "tftp://%s/%s" % (
                 ip_addr,
                 os.path.dirname(substitutions["{KERNEL}"]) + script,
@@ -644,8 +637,8 @@ class BootloaderCommandOverlay(Action):
             action="bootloader-overlay", label=self.method, key="commands", value=subs
         )
         self.logger.debug("substitutions:")
-        for k in sorted(substitutions.keys()):
-            self.logger.debug("- %s: %s", k, substitutions[k])
+        for k, v in substitutions.items():
+            self.logger.debug("- %s: %s", k, v)
         self.logger.info("Parsed boot commands:")
         for sub in subs:
             self.logger.info("- %s", sub)
@@ -751,7 +744,7 @@ class OverlayUnpack(Action):
         )
 
         if transfer_method == "http":
-            overlay_full_path = self.get_namespace_data(
+            overlay_full_path: str | None = self.get_namespace_data(
                 action="compress-overlay", label="output", key="file"
             )
             if not overlay_full_path:
@@ -771,16 +764,25 @@ class OverlayUnpack(Action):
             connection.sendline(
                 "%s http://%s/tmp/%s" % (cmd, ip_addr, overlay_path),
                 delay=self.character_delay,
+                check=True,
+                timeout=max_end_time - time.monotonic(),
             )
             connection.wait()
 
             unpack = self.parameters["transfer_overlay"]["unpack_command"]
-            connection.sendline(unpack + " " + overlay, delay=self.character_delay)
+            connection.sendline(
+                unpack + " " + overlay,
+                delay=self.character_delay,
+                check=True,
+                timeout=max_end_time - time.monotonic(),
+            )
             connection.wait()
         elif transfer_method == "nfs":
-            location = self.get_namespace_data(
+            location: str | None = self.get_namespace_data(
                 action="test", label="shared", key="location"
             )
+            if location is None:
+                raise JobError("Test location not defined.")
 
             cmd = self.parameters["transfer_overlay"]["download_command"]
             ip_addr = dispatcher_ip(self.job.parameters["dispatcher"], "nfs")
@@ -789,6 +791,8 @@ class OverlayUnpack(Action):
                 "mkdir -p %s; %s %s:%s %s"
                 % (mount_dir, cmd, ip_addr, location, mount_dir),
                 delay=self.character_delay,
+                check=True,
+                timeout=max_end_time - time.monotonic(),
             )
             connection.wait()
 
@@ -797,6 +801,8 @@ class OverlayUnpack(Action):
                 "%s %s/* /; umount %s; rm -fr %s"
                 % (unpack, mount_dir, mount_dir, mount_dir),
                 delay=self.character_delay,
+                check=True,
+                timeout=max_end_time - time.monotonic(),
             )
             connection.wait()
 
@@ -943,7 +949,7 @@ class BootloaderCommandsActionAltBank(Action):
         connection = super().run(connection, max_end_time)
         connection.raw_connection.linesep = self.line_separator()
         connection.prompt_str = [self.params["bootloader_prompt"]]
-        at_bootloader_prompt = self.get_namespace_data(
+        at_bootloader_prompt: bool | None = self.get_namespace_data(
             action="interrupt", label="interrupt", key="at_bootloader_prompt"
         )
         if not at_bootloader_prompt:
@@ -954,6 +960,10 @@ class BootloaderCommandsActionAltBank(Action):
         final_message = self.job.device.get_constant(
             "final-message", prefix=self.method, missing_ok=True
         )
+        if job_final_message := self.parameters.get("parameters", {}).get(
+            "bootloader-final-message"
+        ):
+            final_message = job_final_message
         if error_messages:
             if isinstance(connection.prompt_str, str):
                 connection.prompt_str = [connection.prompt_str]
@@ -1009,20 +1019,28 @@ class BootloaderCommandsAction(Action):
         connection = super().run(connection, max_end_time)
         connection.raw_connection.linesep = self.line_separator()
         connection.prompt_str = [self.params["bootloader_prompt"]]
-        at_bootloader_prompt = self.get_namespace_data(
+        at_bootloader_prompt: str | None = self.get_namespace_data(
             action="interrupt", label="interrupt", key="at_bootloader_prompt"
         )
         if not at_bootloader_prompt:
             self.wait(connection, max_end_time)
-        commands = self.get_namespace_data(
+        commands: list[str] | None = self.get_namespace_data(
             action="bootloader-overlay", label=self.method, key="commands"
         )
+        if commands is None:
+            raise JobError(
+                f"Bootloader commands for {self.method!r} method are not defined"
+            )
         error_messages = self.job.device.get_constant(
             "error-messages", prefix=self.method, missing_ok=True
         )
         final_message = self.job.device.get_constant(
             "final-message", prefix=self.method, missing_ok=True
         )
+        if job_final_message := self.parameters.get("parameters", {}).get(
+            "bootloader-final-message"
+        ):
+            final_message = job_final_message
         if error_messages:
             if isinstance(connection.prompt_str, str):
                 connection.prompt_str = [connection.prompt_str]
@@ -1068,7 +1086,7 @@ class AdbOverlayUnpack(Action):
         connection = super().run(connection, max_end_time)
         if not connection:
             raise LAVABug("Cannot transfer overlay, no connection available.")
-        overlay_file = self.get_namespace_data(
+        overlay_file: str | None = self.get_namespace_data(
             action="compress-overlay", label="output", key="file"
         )
         if not overlay_file:
