@@ -14,7 +14,7 @@ from lava_common.exceptions import JobError
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.boot import BootHasMixin
 from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
-from lava_dispatcher.logical import Boot, RetryAction
+from lava_dispatcher.logical import RetryAction
 from lava_dispatcher.shell import ExpectShellSession, ShellCommand, ShellSession
 from lava_dispatcher.utils.network import dispatcher_ip
 
@@ -22,22 +22,6 @@ if TYPE_CHECKING:
     from lava_dispatcher.job import Job
 
 
-class BootDocker(Boot):
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return BootDockerAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if "docker" not in device["actions"]["boot"]["methods"]:
-            return False, '"docker" was not in the device configuration boot methods'
-        if parameters["method"] != "docker":
-            return False, '"method" was not "docker"'
-        if "command" not in parameters:
-            return False, '"command" was not in boot parameters'
-        return True, "accepted"
-
-
 class BootDockerAction(BootHasMixin, RetryAction):
     name = "boot-docker"
     description = "boot docker image"
@@ -90,7 +74,7 @@ class CallDockerAction(Action):
         for cap in options["capabilities"]:
             self.extra_options += " --cap-add %s" % cap
         for device in options["devices"]:
-            self.extra_options += " --device %s" % device
+            self.extra_options += " --device %s" % os.path.realpath(device)
         for network in options["networks"]:
             self.extra_options += " --network %s" % network
         for volume in options["volumes"]:
@@ -147,7 +131,7 @@ class CallDockerAction(Action):
         shell = ShellCommand(cmd, self.timeout, logger=self.logger)
         self.cleanup_required = True
 
-        shell_connection = ShellSession(self.job, shell)
+        shell_connection = ShellSession(shell)
         shell_connection = super().run(shell_connection, max_end_time)
 
         self.set_namespace_data(
