@@ -15,7 +15,7 @@ from typing import TYPE_CHECKING
 from lava_common.exceptions import JobError
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.boot import AutoLoginAction, BootHasMixin, OverlayUnpack
-from lava_dispatcher.logical import Boot, RetryAction
+from lava_dispatcher.logical import RetryAction
 from lava_dispatcher.power import ReadFeedback
 from lava_dispatcher.shell import ExpectShellSession, ShellCommand, ShellSession
 
@@ -23,22 +23,6 @@ if TYPE_CHECKING:
     from lava_dispatcher.job import Job
 
 
-class BootFVP(Boot):
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return BootFVPAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if "fvp" not in device["actions"]["boot"]["methods"]:
-            return False, '"fvp" was not in the device configuration boot methods'
-        if parameters["method"] != "fvp":
-            return False, '"method" was not "fvp"'
-        if "image" not in parameters:
-            return False, '"image" was not in boot parameters'
-        return True, "accepted"
-
-
 class BootFVPAction(BootHasMixin, RetryAction):
     name = "boot-fvp"
     description = "boot fvp"
@@ -54,6 +38,10 @@ class BootFVPAction(BootHasMixin, RetryAction):
                 self.pipeline.add_action(ExpectShellSession(self.job))
                 if "transfer_overlay" in parameters:
                     self.pipeline.add_action(OverlayUnpack(self.job))
+        if parameters.get("erpc_app", ""):
+            self.pipeline.add_action(RunFVPeRPCApp(self.job))
+        if parameters.get("commands"):
+            self.pipeline.add_action(RunFVPShellCommands(self.job))
 
 
 class BootFVPMain(Action):
@@ -79,6 +67,7 @@ class BaseFVPAction(Action):
         super().__init__(job)
         self.extra_options = ""
         self.container = ""
+        self.environment = []
         self.fvp_license = None
         self.ubl_license = None
         self.docker_image = None
@@ -122,6 +111,10 @@ class BaseFVPAction(Action):
             self.extra_options += " --volume %s" % volume
         if "license_variable" in self.parameters:
             self.fvp_license = shlex.quote(self.parameters["license_variable"])
+        if "environment" in self.parameters and isinstance(
+            self.parameters["environment"], list
+        ):
+            self.environment = [shlex.quote(e) for e in self.parameters["environment"]]
         if options.get("ubl_license"):
             self.ubl_license = shlex.quote(options["ubl_license"])
         if "ubl_license" in self.parameters:
@@ -170,6 +163,8 @@ class BaseFVPAction(Action):
                 )
         else:
             cmd += " -e %s" % self.fvp_license
+        for env in self.environment:
+            cmd += f" -e {env}"
         fvp_image = self.parameters.get("image")
         cmd += self.extra_options
 
@@ -301,8 +296,11 @@ class CheckFVPVersionAction(BaseFVPAction):
         fvp_image = self.parameters.get("image")
         fvp_arguments = self.parameters.get("version_args", "--version")
 
+        cmd = ["docker", "run", "--rm"]
+        for env in self.environment:
+            cmd.extend(["-e", env])
         output = self.parsed_command(
-            ["docker", "run", "--rm", self.docker_image, fvp_image, fvp_arguments]
+            cmd + [self.docker_image, fvp_image, fvp_arguments]
         )
         m = re.match(self.fvp_version_string, output)
         matched_version_string = m and m.group(0) or output
@@ -359,7 +357,7 @@ class StartFVPAction(BaseFVPAction):
         self.logger.debug("Boot command: %s", cmd)
         shell = ShellCommand(cmd, self.timeout, logger=self.logger)
 
-        shell_connection = ShellSession(self.job, shell)
+        shell_connection = ShellSession(shell)
         shell_connection = super().run(shell_connection, max_end_time)
 
         # Wait for the console string
@@ -458,16 +456,13 @@ class GetFVPSerialAction(Action):
         for feedback_dict in feedback_ports:
             cmd = (
                 "lava-outerr docker exec --interactive --tty %s telnet localhost %s"
-                % (
-                    container,
-                    feedback_dict["port"],
-                )
+                % (container, feedback_dict["port"])
             )
 
             self.logger.debug("Feedback command: %s", cmd)
             shell = ShellCommand(cmd, self.timeout, logger=self.logger)
 
-            shell_connection = ShellSession(self.job, shell)
+            shell_connection = ShellSession(shell)
             shell_connection = super().run(shell_connection, max_end_time)
             shell_connection.raw_connection.logfile_read.is_feedback = True
 
@@ -490,10 +485,91 @@ class GetFVPSerialAction(Action):
         self.logger.debug("Connect command: %s", cmd)
         shell = ShellCommand(cmd, self.timeout, logger=self.logger)
 
-        shell_connection = ShellSession(self.job, shell)
+        shell_connection = ShellSession(shell)
         shell_connection = super().run(shell_connection, max_end_time)
 
         self.set_namespace_data(
             action="shared", label="shared", key="connection", value=shell_connection
         )
         return shell_connection
+
+
+class RunFVPeRPCApp(Action):
+    name = "run-fvp-erpc-app"
+    description = "run FVP eRPC app"
+    summary = "run FVP eRPC app"
+
+    def __init__(self, job: Job):
+        super().__init__(job)
+        self.app: str = ""
+
+    def validate(self):
+        super().validate()
+        self.app = self.parameters.get("erpc_app", "")
+        if not self.app:
+            raise JobError("'erpc_app' cannot be empty")
+
+    def run(self, connection, max_end_time):
+        container = self.get_namespace_data(
+            action=StartFVPAction.name, label="fvp", key="container"
+        )
+
+        app_path = self.get_namespace_data(
+            action="download-action", label="file", key=self.app
+        )
+        if app_path is None:
+            raise JobError(f"eRPC app '{self.app}' not deployed")
+        app_path = f"/{container}/{os.path.basename(app_path)}"
+
+        cmd = f"docker exec --tty {container} sh -c 'chmod +x {app_path} && {app_path}'"
+
+        self.logger.debug("Connect command: %s", cmd)
+        shell = ShellCommand(cmd, self.timeout, logger=self.logger)
+
+        connection = ShellSession(shell)
+        connection = super().run(connection, max_end_time)
+
+        self.set_namespace_data(
+            action="shared", label="shared", key="connection", value=connection
+        )
+
+        return connection
+
+
+class RunFVPShellCommands(Action):
+    name = "run-fvp-shell-commands"
+    description = "run shell commands in FVP docker container"
+    summary = "run shell commands in FVP docker container"
+
+    def __init__(self, job: Job):
+        super().__init__(job)
+        self.commands: list[str] = []
+
+    def validate(self):
+        super().validate()
+        self.commands = self.parameters.get("commands", [])
+        if not self.commands:
+            self.errors = "'commands' cannot be empty"
+            return
+        if not isinstance(self.commands, list):
+            self.errors = "'commands' must be a list"
+
+    def run(self, connection, max_end_time):
+        container = self.get_namespace_data(
+            action=StartFVPAction.name, label="fvp", key="container"
+        )
+
+        cmds = " && ".join(self.commands)
+        cmd = f"docker exec --tty {container} sh -c {shlex.quote(cmds)}"
+
+        self.logger.debug("Command: %s", cmd)
+        shell = ShellCommand(cmd, self.timeout, logger=self.logger)
+
+        connection = ShellSession(shell)
+        connection = super().run(connection, max_end_time)
+
+        self.set_namespace_data(
+            action="shared", label="shared", key="connection", value=connection
+        )
+
+        return connection
