@@ -24,7 +24,7 @@ from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
 from lava_dispatcher.actions.boot.fastboot import WaitFastBootInterrupt
 from lava_dispatcher.actions.boot.uefi_menu import UEFIMenuInterrupt, UefiMenuSelector
 from lava_dispatcher.connections.serial import ConnectDevice
-from lava_dispatcher.logical import Boot, RetryAction
+from lava_dispatcher.logical import RetryAction
 from lava_dispatcher.power import PowerOff, ResetDevice
 from lava_dispatcher.shell import ExpectShellSession
 
@@ -34,51 +34,7 @@ if TYPE_CHECKING:
     from lava_dispatcher.job import Job
 
 
-class GrubSequence(Boot):
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return GrubSequenceAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if parameters["method"] not in ["grub", "grub-efi"]:
-            return False, '"method" was not "grub" or "grub-efi"'
-
-        params = device["actions"]["boot"]["methods"]
-        if "grub" not in params:
-            return False, '"grub" was not in the device configuration boot methods'
-        if "grub-efi" in params:
-            return False, '"grub-efi" was not in the device configuration boot methods'
-        if "sequence" in params["grub"]:
-            return True, "accepted"
-        return False, '"sequence" not in device configuration boot methods'
-
-
-class Grub(Boot):
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return GrubMainAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if parameters["method"] not in ["grub", "grub-efi"]:
-            return False, '"method" was not "grub" or "grub-efi"'
-
-        params = device["actions"]["boot"]["methods"]
-        if "grub" in params and "sequence" in params["grub"]:
-            return False, '"sequence" was in "grub" parameters'
-        if "grub" in params or "grub-efi" in params:
-            return True, "accepted"
-        else:
-            return (
-                False,
-                '"grub" or "grub-efi" was not in the device configuration boot methods',
-            )
-
-
-def _grub_sequence_map(
-    sequence: str,
-) -> tuple[Optional[type[Action]], Optional[str]]:
+def _grub_sequence_map(sequence: str) -> tuple[Optional[type[Action]], Optional[str]]:
     """Maps grub sequence with corresponding class."""
     sequence_map: dict[str, tuple[type[Action], Optional[str]]] = {
         "wait-fastboot-interrupt": (WaitFastBootInterrupt, "grub"),
@@ -159,11 +115,12 @@ class GrubMainAction(BootHasMixin, RetryAction):
             .get("grub-efi", {})
             .get("reset_device", True)
         )
-        if parameters["method"] == "grub-efi" and reset_device:
-            # added unless the device specifies not to reset the device in grub.
-            self.pipeline.add_action(ResetDevice(self.job))
-        elif parameters["method"] == "grub":
-            self.pipeline.add_action(ResetDevice(self.job))
+        if parameters.get("reset", True):
+            if parameters["method"] == "grub-efi" and reset_device:
+                # added unless the device specifies not to reset the device in grub.
+                self.pipeline.add_action(ResetDevice(self.job))
+            elif parameters["method"] == "grub":
+                self.pipeline.add_action(ResetDevice(self.job))
         if parameters["method"] == "grub-efi":
             self.pipeline.add_action(UEFIMenuInterrupt(self.job))
             self.pipeline.add_action(GrubMenuSelector(self.job))
