@@ -1,4 +1,4 @@
-# Copyright 2019-2024 NXP
+# Copyright 2019-2025 NXP
 #
 # Author: Thomas Mahe <thomas.mahe@nxp.com>
 #         Franck Lenormand <franck.lenormand@nxp.com>
@@ -13,21 +13,17 @@ from __future__ import annotations
 
 import re
 import time
-from typing import TYPE_CHECKING
 
-from lava_common.exceptions import ConfigurationError, InfrastructureError, JobError
+from lava_common.exceptions import InfrastructureError, JobError
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.boot import BootloaderCommandOverlay
 from lava_dispatcher.actions.boot.bootloader import BootBootloaderAction
 from lava_dispatcher.connections.serial import ConnectDevice, DisconnectDevice
-from lava_dispatcher.logical import Boot, RetryAction
+from lava_dispatcher.logical import RetryAction
 from lava_dispatcher.power import PowerOff, ResetDevice
 from lava_dispatcher.utils.strings import safe_dict_format
 from lava_dispatcher.utils.uuu import OptionalContainerUuuAction
 
-if TYPE_CHECKING:
-    from lava_dispatcher.job import Job
-
 
 class CheckSerialDownloadMode(OptionalContainerUuuAction):
     name = "check-serial-availability"
@@ -94,42 +90,6 @@ class CheckSerialDownloadMode(OptionalContainerUuuAction):
         return connection
 
 
-class UUUBoot(Boot):
-    """
-    The UUUBoot method allow to always boot on USB serial download mode
-    used by uuu to flash on boot media a fresh bootimage.
-
-    If the board is not available for USB serial download, action 'boot-corrupt-boot-media'
-    will run commands in u-boot to corrupt boot media. After this action, board must be available in USB serial download
-    mode.
-    if the board is available without the previous action, 'boot-corrupt-boot-media' won't add anything to pipeline.
-
-    """
-
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return UUUBootRetryAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if parameters["method"] != "uuu":
-            return False, '"method" was not "uuu"'
-        if "commands" not in parameters:
-            raise ConfigurationError("commands not specified in boot parameters")
-        params = device["actions"]["boot"]["methods"]["uuu"]["options"]
-        if not params["usb_otg_path"] and not params["usb_otg_path_command"]:
-            raise ConfigurationError(
-                "'uuu_usb_otg_path' or 'uuu_usb_otg_path_command' not defined in device definition"
-            )
-        if params["corrupt_boot_media_command"] is None:
-            raise ConfigurationError(
-                "uuu_corrupt_boot_media_command not defined in device definition"
-            )
-        if "u-boot" in device["actions"]["boot"]["methods"]:
-            return True, "accepted"
-        return False, '"uuu" was not in the device configuration boot methods'
-
-
 class BootBootloaderCorruptBootMediaAction(Action):
     name = "boot-corrupt-boot-media"
     description = "boot using 'bootloader' method and corrupt boot media"
@@ -150,7 +110,11 @@ class BootBootloaderCorruptBootMediaAction(Action):
             "method": "bootloader",
             "bootloader": "u-boot",
             "commands": SD_ERASE_CMDS,
-            "prompts": ["=>"],
+            "prompts": [
+                self.job.device["actions"]["boot"]["methods"]["u-boot"]["parameters"][
+                    "bootloader_prompt"
+                ]
+            ],
         }
         self.pipeline = Pipeline(
             parent=self, job=self.job, parameters={**parameters, **u_boot_params}
@@ -319,7 +283,7 @@ Following actions will be skipped :
                 otg_paths = raw_option
 
             # All path are matching with uuu path format
-            if None not in map(lambda p: otg_path_fsm.fullmatch(p), otg_paths):
+            if None not in map(otg_path_fsm.fullmatch, otg_paths):
                 return otg_paths
 
         if uuu_options.get("usb_otg_path_command") is None:
@@ -339,7 +303,7 @@ Following actions will be skipped :
             uuu_otg_paths = list(map(lambda p: p.strip(), command_output.splitlines()))
 
             # All path are matching with uuu path format
-            if None not in map(lambda p: otg_path_fsm.fullmatch(p), uuu_otg_paths):
+            if None not in map(otg_path_fsm.fullmatch, uuu_otg_paths):
                 self.logger.info("uuu_otg_path matched : %s", uuu_otg_paths)
                 return uuu_otg_paths
 
@@ -370,7 +334,8 @@ Following actions will be skipped :
 
     def first_command_is_bcu_reset_usb(self, parameters):
         """
-        Return: True if "bcu: reset usb" is the first command listed in commands block
+        Return: True if first command listed in commands block starts with "bcu: reset usb"
+                e.g. "bcu: reset usb", "bcu: reset usb -keep"
                 False otherwise
         """
         # First command
@@ -378,7 +343,8 @@ Following actions will be skipped :
         if proto != "bcu":
             return False
 
-        return ["reset", "usb"] == list(map(str.strip, command.split(" ")))
+        pattern = ["reset", "usb"]
+        return pattern == list(map(str.strip, command.split(" ")))[: len(pattern)]
 
     def eval_bcu_board_id(self):
         uuu_options = self.job.device["actions"]["boot"]["methods"]["uuu"]["options"]
@@ -451,7 +417,7 @@ class UUUBootAction(OptionalContainerUuuAction):
         )
 
         # Use to replace {boot} identifier in cmd by correct 'boot' image path
-        templates = dict()
+        templates = {}
         for image in images_name:
             templates[image] = self.get_namespace_data(
                 "download-action", label=image, key="file"
@@ -516,3 +482,19 @@ class UUUBootAction(OptionalContainerUuuAction):
                 error_msg="Fail UUUBootAction on cmd : deinit",
             )
             self.cleanup_required = False
+
+            additional_bcu_cleanup_commands = self.job.device["actions"]["boot"][
+                "methods"
+            ]["uuu"]["options"].get("additional_bcu_cleanup_commands", [])
+            for additional_bcu_cleanup_command in additional_bcu_cleanup_commands:
+                exec_cmd = "{} {} -board={} -id={}".format(
+                    self.bcu,
+                    additional_bcu_cleanup_command,
+                    self.bcu_board_name,
+                    self.bcu_board_id,
+                )
+                self.run_bcu(
+                    exec_cmd.split(" "),
+                    allow_fail=False,
+                    error_msg="Fail UUUBootAction on cmd : additional_bcu_cleanup_command",
+                )
