@@ -7,16 +7,14 @@ from __future__ import annotations
 
 import os
 import shutil
+import zipfile
 from functools import partial
 from typing import TYPE_CHECKING
 
-import guestfs
-
 from lava_common.constants import RAMDISK_FNAME, UBOOT_DEFAULT_HEADER_LENGTH
 from lava_common.exceptions import InfrastructureError, JobError, LAVABug
 from lava_common.utils import debian_filename_version
 from lava_dispatcher.action import Action, Pipeline
-from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.actions.deploy.prepare import PrepareKernelAction
 from lava_dispatcher.utils.compression import (
     compress_file,
@@ -35,9 +33,12 @@ from lava_dispatcher.utils.filesystem import (
     prepare_guestfs,
 )
 from lava_dispatcher.utils.installers import add_late_command, add_to_kickstart
-from lava_dispatcher.utils.network import dispatcher_ip
+from lava_dispatcher.utils.network import dispatcher_ip, rpcinfo_nfs
 from lava_dispatcher.utils.shell import which
-from lava_dispatcher.utils.strings import substitute
+from lava_dispatcher.utils.strings import (
+    substitute,
+    substitute_address_with_static_info,
+)
 
 if TYPE_CHECKING:
     from lava_dispatcher.job import Job
@@ -70,7 +71,7 @@ class ApplyOverlayGuest(Action):
 
     def run(self, connection, max_end_time):
         applied = self.get_namespace_data(
-            action="append-overlays", label="guest", key="applied"
+            action="append-overlays", label="result", key="applied"
         )
         if applied:
             self.logger.debug("Overlay already applied")
@@ -92,6 +93,7 @@ class ApplyOverlayGuest(Action):
         )
 
         blkid = prepare_guestfs(
+            self,
             guest_file,
             overlay_file,
             mountpoint,
@@ -131,8 +133,8 @@ class ApplyOverlayImage(Action):
 
             if self.use_root_partition:
                 if (
-                    self.image_key not in self.parameters.keys()
-                    and "images" in self.parameters.keys()
+                    self.image_key not in self.parameters
+                    and "images" in self.parameters
                 ):
                     if self.image_key in self.parameters["images"]:
                         root_partition = self.parameters["images"][self.image_key].get(
@@ -227,15 +229,16 @@ class PrepareOverlayTftp(Action):
             ExtractNfsRootfs(self.job)
         )  # idempotent, checks for nfsrootfs parameter
         self.pipeline.add_action(
-            OverlayAction(self.job)
-        )  # idempotent, includes testdef
+            ParsePersistentNFS(self.job)
+        )  # idempotent, parse persistent nfs
         self.pipeline.add_action(
             ExtractRamdisk(self.job)
         )  # idempotent, checks for a ramdisk parameter
         self.pipeline.add_action(
             ExtractModules(self.job)
         )  # idempotent, checks for a modules parameter
-        self.pipeline.add_action(ApplyOverlayTftp(self.job))
+        if self.test_needs_overlay(parameters):
+            self.pipeline.add_action(ApplyOverlayTftp(self.job))
         if "kernel" in parameters and "type" in parameters["kernel"]:
             self.pipeline.add_action(PrepareKernelAction(self.job))
         self.pipeline.add_action(
@@ -284,48 +287,49 @@ class ApplyOverlayTftp(Action):
         connection = super().run(connection, max_end_time)
         directory = None
         nfs_address = None
-        overlay_file = None
+
+        if self.get_namespace_data(
+            action="append-overlays", label="result", key="applied"
+        ):
+            self.logger.debug("Overlay already applied")
+            return connection
+
+        overlay_file = self.get_namespace_data(
+            action="compress-overlay", label="output", key="file"
+        )
+        if not overlay_file:
+            self.logger.warning("No overlay to apply")
+            return connection
+
         namespace = self.parameters.get("namespace")
         if self.parameters.get("nfsrootfs") is not None:
             if not self.parameters["nfsrootfs"].get("install_overlay", True):
                 self.logger.info("[%s] Skipping applying overlay to NFS", namespace)
                 return connection
-            overlay_file = self.get_namespace_data(
-                action="compress-overlay", label="output", key="file"
-            )
             directory = self.get_namespace_data(
                 action="extract-rootfs", label="file", key="nfsroot"
             )
-            if overlay_file:
-                self.logger.info("[%s] Applying overlay to NFS", namespace)
+            self.logger.info("[%s] Applying overlay to NFS", namespace)
         elif self.parameters.get("images", {}).get("nfsrootfs") is not None:
             if not self.parameters["images"]["nfsrootfs"].get("install_overlay", True):
                 self.logger.info("[%s] Skipping applying overlay to NFS", namespace)
                 return connection
-            overlay_file = self.get_namespace_data(
-                action="compress-overlay", label="output", key="file"
-            )
             directory = self.get_namespace_data(
                 action="extract-rootfs", label="file", key="nfsroot"
             )
-            if overlay_file:
-                self.logger.info("[%s] Applying overlay to NFS", namespace)
+            self.logger.info("[%s] Applying overlay to NFS", namespace)
         elif self.parameters.get("persistent_nfs") is not None:
             if not self.parameters["persistent_nfs"].get("install_overlay", True):
                 self.logger.info(
                     "[%s] Skipping applying overlay to persistent NFS", namespace
                 )
                 return connection
-            overlay_file = self.get_namespace_data(
-                action="compress-overlay", label="output", key="file"
-            )
             nfs_address = self.parameters["persistent_nfs"].get("address")
-            if overlay_file:
-                self.logger.info(
-                    "[%s] Applying overlay to persistent NFS address %s",
-                    namespace,
-                    nfs_address,
-                )
+            self.logger.info(
+                "[%s] Applying overlay to persistent NFS address %s",
+                namespace,
+                nfs_address,
+            )
             # need to mount the persistent NFS here.
             # We can't use self.mkdtemp() here because this directory should
             # not be removed if umount fails.
@@ -335,27 +339,21 @@ class ApplyOverlayTftp(Action):
             if not self.parameters["ramdisk"].get("install_overlay", True):
                 self.logger.info("[%s] Skipping applying overlay to ramdisk", namespace)
                 return connection
-            overlay_file = self.get_namespace_data(
-                action="compress-overlay", label="output", key="file"
-            )
             directory = self.get_namespace_data(
                 action="extract-overlay-ramdisk",
                 label="extracted_ramdisk",
                 key="directory",
             )
-            if overlay_file:
-                self.logger.info(
-                    "[%s] Applying overlay %s to ramdisk", namespace, overlay_file
-                )
-        elif self.parameters.get("rootfs") is not None:
-            overlay_file = self.get_namespace_data(
-                action="compress-overlay", label="output", key="file"
+            self.logger.info(
+                "[%s] Applying overlay %s to ramdisk", namespace, overlay_file
             )
+        elif self.parameters.get("rootfs") is not None:
             directory = self.get_namespace_data(
                 action="apply-overlay", label="file", key="root"
             )
         else:
             self.logger.debug("[%s] No overlay directory", namespace)
+            return connection
         if self.parameters.get("os") == "centos_installer":
             # centos installer ramdisk doesn't like having anything other
             # than the kickstart config being inserted. Instead, make the
@@ -377,17 +375,21 @@ class ApplyOverlayTftp(Action):
                 key="overlay",
                 value=os.path.join(suffix, "ramdisk", os.path.basename(overlay_file)),
             )
-        if overlay_file:
-            self.logger.debug(
-                "[%s] Applying overlay %s to directory %s",
-                namespace,
-                overlay_file,
-                directory,
-            )
-            untar_file(overlay_file, directory)
-            if nfs_address:
-                self.run_cmd(["umount", directory])
-                os.rmdir(directory)  # fails if the umount fails
+
+        if not directory:
+            return connection
+
+        self.logger.debug(
+            "[%s] Applying overlay %s to directory %s",
+            namespace,
+            overlay_file,
+            directory,
+        )
+        untar_file(overlay_file, directory)
+        if nfs_address:
+            self.run_cmd(["umount", directory])
+            os.rmdir(directory)  # fails if the umount fails
+
         return connection
 
 
@@ -657,7 +659,7 @@ class CompressRamdisk(Action):
                     self.mkimage_arch = self.job.device["actions"]["deploy"][
                         "parameters"
                     ]["mkimage_arch"]
-                else:
+                elif self.add_header != "raw":
                     self.errors = "ramdisk: add_header: unknown header type"
 
     def run(self, connection, max_end_time):
@@ -865,9 +867,6 @@ class AppendOverlays(Action):
             raise JobError("'overlays' is not a dictionary")
         for overlay, params in self.params["overlays"].items():
             if overlay == "lava":
-                self.set_namespace_data(
-                    action=self.name, label="guest", key="applied", value=True
-                )
                 continue
             if params.get("format") not in self.OVERLAY_FORMATS:
                 raise JobError(
@@ -940,20 +939,31 @@ class AppendOverlays(Action):
                     action="download-action", label=label, key="file"
                 )
                 path = self.params["overlays"][overlay]["path"]
-            # Take off initial "/" from path, extract relative to this directory
-            extract_path = os.path.join(tempdir, path[1:])
-            if overlay == "lava" or self.params["overlays"][overlay]["format"] == "tar":
-                self.logger.debug(
-                    "- %s: untar %r to %r", label, overlay_image, extract_path
-                )
-                # In the "validate" function, we check that path startswith '/'
-                # and does not contains '..'
-                untar_file(overlay_image, extract_path)
+
+            if overlay_image:
+                # Take off initial "/" from path, extract relative to this directory
+                extract_path = os.path.join(tempdir, path[1:])
+                if (
+                    overlay == "lava"
+                    or self.params["overlays"][overlay]["format"] == "tar"
+                ):
+                    self.logger.debug(
+                        "- %s: untar %r to %r", label, overlay_image, extract_path
+                    )
+                    # In the "validate" function, we check that path startswith '/'
+                    # and does not contains '..'
+                    untar_file(overlay_image, extract_path)
+                    if overlay == "lava":
+                        self.set_namespace_data(
+                            action=self.name, label="result", key="applied", value=True
+                        )
+                else:
+                    self.logger.debug(
+                        "- %s: cp %r to %r", label, overlay_image, extract_path
+                    )
+                    shutil.copy(overlay_image, extract_path)
             else:
-                self.logger.debug(
-                    "- %s: cp %r to %r", label, overlay_image, extract_path
-                )
-                shutil.copy(overlay_image, extract_path)
+                self.logger.warning("- %s: <MISSING> to %r", label, path)
 
         # Recreating the archive
         self.logger.debug("* archiving %r", image)
@@ -981,6 +991,8 @@ class AppendOverlays(Action):
             self.run_cmd(command_list, error_msg="simg2img failed for %s" % image)
             os.replace(f"{image}.non-sparse", image)
 
+        import guestfs
+
         guest = guestfs.GuestFS(python_return_dict=True)
         guest.add_drive(image)
         try:
@@ -1027,6 +1039,10 @@ class AppendOverlays(Action):
                 else:
                     guest.mkdir_p(os.path.dirname(path))
                     guest.upload(overlay_image, path)
+                if overlay == "lava":
+                    self.set_namespace_data(
+                        action=self.name, label="result", key="applied", value=True
+                    )
             else:
                 self.logger.warning("- %s: <MISSING> to %r", label, path)
         guest.umount(device)
@@ -1037,3 +1053,101 @@ class AppendOverlays(Action):
             command_list = ["/usr/bin/img2simg", image, f"{image}.sparse"]
             self.run_cmd(command_list, error_msg="img2simg failed for %s" % image)
             os.replace(f"{image}.sparse", image)
+
+
+class ParsePersistentNFS(Action):
+    """
+    Instead of extracting, just populate the location of the persistent NFS
+    so that it can be mounted later when the overlay is applied.
+    """
+
+    name = "parse-persistent-nfs"
+    description = "parse persistent nfs"
+    summary = "parse persistent nfs"
+
+    def validate(self):
+        super().validate()
+        persist = self.parameters.get("persistent_nfs")
+        if not persist:
+            return
+        if "address" not in persist:
+            self.errors = "Missing address for persistent NFS"
+            return
+        if ":" not in persist["address"]:
+            self.errors = (
+                "Unrecognised NFS URL: '%s'"
+                % self.parameters["persistent_nfs"]["address"]
+            )
+            return
+
+        persist["address"] = substitute_address_with_static_info(
+            persist["address"], self.job.device.get("static_info", [])
+        )
+
+        nfs_server, dirname = persist["address"].split(":")
+        self.errors = rpcinfo_nfs(nfs_server)
+        self.set_namespace_data(
+            action=self.name, label="nfs_address", key="nfsroot", value=dirname
+        )
+        self.set_namespace_data(
+            action=self.name, label="nfs_address", key="serverip", value=nfs_server
+        )
+
+        self.job.device["dynamic_data"]["NFS_ROOTFS"] = dirname
+        self.job.device["dynamic_data"]["NFS_SERVER_IP"] = nfs_server
+
+
+class ApplyOverlayAvh(Action):
+    """
+    Apply lava overlay to the specific partition of avh storage file.
+    """
+
+    name = "apply-overlay-avh"
+    description = "apply overlay via guestfs to avh storage file"
+    summary = "apply overlay to avh storage file"
+    timeout_exception = InfrastructureError
+
+    def __init__(self, job: Job):
+        super().__init__(job)
+        self.storage_file = None
+        self.root_partition = None
+
+    def validate(self):
+        super().validate()
+
+        self.storage_file = self.parameters["fw_package"].get("storage_file")
+        if self.storage_file is None:
+            raise JobError("Unable to apply overlay without 'fw_package.storage_file'")
+
+        self.root_partition = self.parameters["fw_package"].get("root_partition")
+        if self.root_partition is None:
+            raise JobError(
+                "Unable to apply overlay without 'fw_package.root_partition'"
+            )
+
+    def run(self, connection, max_end_time):
+        connection = super().run(connection, max_end_time)
+
+        overlay_file = self.get_namespace_data(
+            action="compress-overlay", label="output", key="file"
+        )
+        if overlay_file is None:
+            self.logger.warning("No overlay to apply")
+            return connection
+
+        fw_package_path = self.get_namespace_data(
+            action="download-action", label="fw_package", key="file"
+        )
+        self.logger.info(
+            f"applying overlay to {self.storage_file} in {fw_package_path}"
+        )
+        tempdir = self.mkdtemp()
+        with zipfile.ZipFile(fw_package_path, "r") as zf:
+            zf.extractall(tempdir)
+
+        storage_file_path = f"{tempdir}/{self.storage_file}"
+        copy_in_overlay(storage_file_path, self.root_partition, overlay_file)
+
+        shutil.make_archive(fw_package_path[:-4], "zip", tempdir)
+
+        return connection
