@@ -5,9 +5,6 @@
 #         Antonio Terceiro <antonio.terceiro@linaro.org>
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
-
-# This class is used for all downloads, including images and individual files for tftp.
-# python2 only
 from __future__ import annotations
 
 import contextlib
@@ -15,6 +12,7 @@ import hashlib
 import math
 import os
 import pathlib
+import re
 import subprocess  # nosec - verified.
 import time
 from typing import TYPE_CHECKING
@@ -33,9 +31,9 @@ from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.boot.fastboot import EnterFastbootAction
 from lava_dispatcher.actions.boot.u_boot import UBootEnterFastbootAction
 from lava_dispatcher.actions.deploy.apply_overlay import AppendOverlays
-from lava_dispatcher.actions.deploy.overlay import OverlayAction
+from lava_dispatcher.actions.deploy.overlay import CreateOverlay, OverlayAction
 from lava_dispatcher.connections.serial import ConnectDevice
-from lava_dispatcher.logical import Deployment, RetryAction
+from lava_dispatcher.logical import RetryAction
 from lava_dispatcher.power import ResetDevice
 from lava_dispatcher.protocols.lxc import LxcProtocol
 from lava_dispatcher.utils.compression import untar_file
@@ -51,6 +49,9 @@ if TYPE_CHECKING:
     from lava_dispatcher.job import Job
 
 
+HTTP_CODE_OK: int = requests.codes["OK"]
+
+
 class DownloaderAction(RetryAction):
     """
     The retry pipeline for downloads.
@@ -96,7 +97,7 @@ class DownloaderAction(RetryAction):
             action = LxcDownloadAction(self.job, self.key, self.path, url)
         elif url.scheme == "downloads":
             action = PreDownloadedAction(
-                self.job, self.key, url, self.path, params=self.params
+                self.job, self.key, url, self.path, self.uniquify, params=self.params
             )
         else:
             raise JobError("Unsupported url protocol scheme: %s" % url.scheme)
@@ -116,6 +117,8 @@ class DownloaderAction(RetryAction):
                 )
             )
         if overlays:
+            if "lava" in overlays and not self.test_needs_overlay(self.parameters):
+                self.pipeline.add_action(CreateOverlay(self.job))
             self.pipeline.add_action(
                 AppendOverlays(self.job, self.key, params=self.params)
             )
@@ -153,16 +156,17 @@ class DownloadHandler(Action):
 
         self.path = path
         # Store the files in a sub-directory to keep the path unique.
-        if uniquify:
+        self.uniquify = uniquify
+        if self.uniquify:
             self.path = os.path.join(path, key)
-        self.fname = None
+        self.fname: str = ""
         self.params = params
 
     def reader(self):
         raise LAVABug("'reader' function unimplemented")
 
     def cleanup(self, connection):
-        if self.fname is not None and os.path.exists(self.fname):
+        if self.fname and os.path.exists(self.fname):
             self.logger.debug("Cleaning up downloaded image: %s", self.fname)
             try:
                 os.remove(self.fname)
@@ -178,7 +182,7 @@ class DownloadHandler(Action):
             return False
         return self.params.get("compression", False)
 
-    def _url_to_fname(self):
+    def _url_to_fname(self) -> str:
         compression = self._compression()
         filename = os.path.basename(self.url.path)
 
@@ -295,9 +299,6 @@ class DownloadHandler(Action):
 
         connection = super().run(connection, max_end_time)
         # self.cookies = self.job.context.config.lava_cookies  # FIXME: work out how to restore
-        md5 = hashlib.md5()  # nosec - not being used for cryptography.
-        sha256 = hashlib.sha256()
-        sha512 = hashlib.sha512()
 
         # Create a fresh directory if the old one has been removed by a previous cleanup
         # (when retrying inside a RetryAction)
@@ -321,6 +322,11 @@ class DownloadHandler(Action):
         sha256sum = self.params.get("sha256sum")
         sha512sum = self.params.get("sha512sum")
 
+        sha256 = hashlib.sha256()
+        md5 = hashlib.md5() if md5sum is not None else None
+        sha512 = hashlib.sha512() if sha512sum is not None else None
+        hash_constructors = tuple(h for h in (md5, sha256, sha512) if h is not None)
+
         if os.path.isdir(self.fname):
             raise JobError("Download '%s' is a directory, not a file" % self.fname)
         if os.path.exists(self.fname):
@@ -361,7 +367,7 @@ class DownloadHandler(Action):
         last_update = time.monotonic()  # time for rate limiting the progress output
 
         def update_progress(buff):
-            nonlocal downloaded_size, last_update, last_value, md5, sha256, sha512
+            nonlocal downloaded_size, last_update, last_value
             downloaded_size += len(buff)
             (printing, new_value, msg) = progress(
                 downloaded_size, last_value, last_update
@@ -370,9 +376,9 @@ class DownloadHandler(Action):
                 last_update = time.monotonic()
                 last_value = new_value
                 self.logger.debug(msg)
-            md5.update(buff)
-            sha256.update(buff)
-            sha512.update(buff)
+
+            for hash_constructor in hash_constructors:
+                hash_constructor.update(buff)
 
         if compression and decompress_command:
             try:
@@ -417,21 +423,12 @@ class DownloadHandler(Action):
         self.set_namespace_data(
             action="download-action", label="file", key=self.key, value=self.fname
         )
-        self.set_namespace_data(
-            action="download-action", label=self.key, key="md5", value=md5.hexdigest()
-        )
         self.set_namespace_data(
             action="download-action",
             label=self.key,
             key="sha256",
             value=sha256.hexdigest(),
         )
-        self.set_namespace_data(
-            action="download-action",
-            label=self.key,
-            key="sha512",
-            value=sha512.hexdigest(),
-        )
 
         # handle archive files
         archive = self.params.get("archive")
@@ -455,31 +452,39 @@ class DownloadHandler(Action):
                 value=target_fname_path,
             )
 
-        self._check_checksum("md5", md5.hexdigest(), md5sum)
-        self._check_checksum("sha256", sha256.hexdigest(), sha256sum)
-        self._check_checksum("sha512", sha512.hexdigest(), sha512sum)
+        if md5sum is not None:
+            self._check_checksum("md5", md5.hexdigest(), md5sum)
+        if sha256sum is not None:
+            self._check_checksum("sha256", sha256.hexdigest(), sha256sum)
+        if sha512sum is not None:
+            self._check_checksum("sha512", sha512.hexdigest(), sha512sum)
 
         # certain deployments need prefixes set
-        if self.parameters["to"] == "tftp" or self.parameters["to"] == "nbd":
-            suffix = self.get_namespace_data(
-                action="tftp-deploy", label="tftp", key="suffix"
-            )
-            self.set_namespace_data(
-                action="download-action",
-                label="file",
-                key=self.key,
-                value=os.path.join(suffix, self.key, os.path.basename(self.fname)),
-            )
-        elif self.parameters["to"] == "iso-installer":
-            suffix = self.get_namespace_data(
-                action="deploy-iso-installer", label="iso", key="suffix"
-            )
-            self.set_namespace_data(
-                action="download-action",
-                label="file",
-                key=self.key,
-                value=os.path.join(suffix, self.key, os.path.basename(self.fname)),
-            )
+        # TFTP deploy base dir suffix (e.g., "job_id/tftp-deploy-uid") will be used as
+        # the TFTP file loading path prefix.
+        if self.parameters.get("to") in ["tftp", "nbd", "iso-installer"]:
+            if self.parameters["to"] in ["tftp", "nbd"]:
+                prefix: str | None = self.get_namespace_data(
+                    action="tftp-deploy", label="tftp", key="suffix"
+                )
+            elif self.parameters["to"] == "iso-installer":
+                prefix = self.get_namespace_data(
+                    action="deploy-iso-installer", label="iso", key="suffix"
+                )
+
+            if prefix:
+                _path_dirs = [prefix]
+                if self.uniquify:
+                    _path_dirs.append(self.key)
+                _path_dirs.append(os.path.basename(self.fname))
+                self.set_namespace_data(
+                    action="download-action",
+                    label="file",
+                    key=self.key,
+                    value=os.path.join(*_path_dirs),
+                )
+            else:
+                raise JobError(f"'{self.key}' path prefix is required but not found")
 
         # xnbd protocol needs to know the location
         nbdroot = self.get_namespace_data(
@@ -491,21 +496,11 @@ class DownloadHandler(Action):
         self.results = {
             "label": self.key,
             "size": downloaded_size,
-            "md5sum": str(
-                self.get_namespace_data(
-                    action="download-action", label=self.key, key="md5"
-                )
-            ),
             "sha256sum": str(
                 self.get_namespace_data(
                     action="download-action", label=self.key, key="sha256"
                 )
             ),
-            "sha512sum": str(
-                self.get_namespace_data(
-                    action="download-action", label=self.key, key="sha512"
-                )
-            ),
         }
         return connection
 
@@ -588,66 +583,137 @@ class HttpDownloadAction(DownloadHandler):
 
     def validate(self):
         super().validate()
+        http_cache = self.job.parameters["dispatcher"].get("http_url_format_string", "")
+        fallback_origin_url = self.job.parameters["dispatcher"].get(
+            "http_cache_fallback_origin_url", False
+        )
+
+        use_cache = True
+        if self.params:
+            use_cache = self.params.get("use_cache", True)
+
+        if http_cache and use_cache:
+            # cache rules
+            http_cache_include_rules = self.job.parameters["dispatcher"].get(
+                "http_cache_include_rules", []
+            )
+            http_cache_exclude_rules = self.job.parameters["dispatcher"].get(
+                "http_cache_exclude_rules", []
+            )
+            url_str = self.url.geturl()
+
+            # 1. If not match any include rule, disable cache
+            if http_cache_include_rules:
+                if not any(
+                    re.search(rule, url_str) for rule in http_cache_include_rules
+                ):
+                    self.logger.info(
+                        "Skipping caching service: '%s' does not match any http_cache_include_rules: %s",
+                        url_str,
+                        http_cache_include_rules,
+                    )
+                    use_cache = False
+
+            # 2. If still True, check exclude rules
+            if use_cache and http_cache_exclude_rules:
+                if any(re.search(rule, url_str) for rule in http_cache_exclude_rules):
+                    self.logger.info(
+                        "Skipping caching service: '%s' matches http_cache_exclude_rules: %s",
+                        url_str,
+                        http_cache_exclude_rules,
+                    )
+                    use_cache = False
+
+        # Force the non-use of Accept-Encoding: gzip, this will permit to know the final size
+        headers = {"Accept-Encoding": ""}
+        if self.params and "headers" in self.params:
+            headers.update(self.params["headers"])
+
+        if http_cache and use_cache:
+            self.logger.info("Using caching service: '%s'", http_cache)
+            try:
+                original_url = self.url
+                self.url = urlparse(http_cache % quote_plus(self.url.geturl()))
+                # call validate_cache_source
+                if self.validate_cache_source(headers):
+                    # Cache server available
+                    return
+                elif not fallback_origin_url:
+                    self.errors = f"Unable to get '{self.url.geturl()}'"
+                    return
+                self.url = original_url
+                self.logger.info("Fallback to original URL : %s", self.url.geturl())
+            except TypeError as exc:
+                self.logger.error("Invalid http_url_format_string: '%s'", exc)
+                self.errors = "Invalid http_url_format_string: '%s'" % str(exc)
+                return
+
+        # validate url if cache not available or disabled
+        self.validate_url(headers)
+
+    def validate_cache_source(self, headers):
+        try:
+            res = self._head_or_get(self.url.geturl(), headers)
+            if res.status_code != HTTP_CODE_OK:
+                self.logger.error(
+                    "Resource unavailable at '%s' (%d)",
+                    self.url.geturl(),
+                    res.status_code,
+                )
+                return False
+            self.size = int(res.headers.get("content-length", -1))
+            res.close()
+            return True
+        except (requests.Timeout, requests.RequestException) as exc:
+            self.logger.error("Unable to use cache %s", exc)
+            return False
+
+    def validate_url(self, headers):
         res = None
         try:
-            http_cache = self.job.parameters["dispatcher"].get(
-                "http_url_format_string", ""
-            )
-
-            use_cache = True
-            if self.params:
-                use_cache = self.params.get("use_cache", True)
-
-            if http_cache and use_cache:
-                self.logger.info("Using caching service: '%s'", http_cache)
-                try:
-                    self.url = urlparse(http_cache % quote_plus(self.url.geturl()))
-                except TypeError as exc:
-                    self.logger.error("Invalid http_url_format_string: '%s'", exc)
-                    self.errors = "Invalid http_url_format_string: '%s'" % str(exc)
-                    return
-
-            headers = {"Accept-Encoding": ""}
-            if self.params and "headers" in self.params:
-                headers.update(self.params["headers"])
-            self.logger.debug("Validating that %s exists", self.url.geturl())
-            # Force the non-use of Accept-Encoding: gzip, this will permit to know the final size
-            res = requests_retry().head(
-                self.url.geturl(),
-                allow_redirects=True,
-                headers=headers,
-                timeout=HTTP_DOWNLOAD_TIMEOUT,
-            )
-            if res.status_code != requests.codes.OK:
-                # try using (the slower) get for services with broken redirect support
-                self.logger.debug("Using GET because HEAD is not supported properly")
-                res.close()
-                # Like for HEAD, we need get a size, so disable gzip
-                res = requests_retry().get(
+            res = self._head_or_get(self.url.geturl(), headers)
+            if res.status_code != HTTP_CODE_OK:
+                self.errors = "Resource unavailable at '%s' (%d)" % (
                     self.url.geturl(),
-                    allow_redirects=True,
-                    stream=True,
-                    headers=headers,
-                    timeout=HTTP_DOWNLOAD_TIMEOUT,
+                    res.status_code,
                 )
-                if res.status_code != requests.codes.OK:
-                    self.errors = "Resource unavailable at '%s' (%d)" % (
-                        self.url.geturl(),
-                        res.status_code,
-                    )
-                    return
-
+                return
             self.size = int(res.headers.get("content-length", -1))
+            res.close()
+            return
         except requests.Timeout:
             self.logger.error("Request timed out")
             self.errors = "'%s' timed out" % (self.url.geturl())
         except requests.RequestException as exc:
-            self.logger.error("Resource not available")
-            self.errors = "Unable to get '%s': %s" % (self.url.geturl(), str(exc))
+            self.logger.error("Resource not available: %s", exc)
+            self.errors = f"Unable to get '{self.url.geturl()}': {exc}"
         finally:
             if res is not None:
                 res.close()
 
+    def _head_or_get(self, url, headers):
+        """
+        Try HEAD first, fallback to GET if needed.
+        """
+        self.logger.debug("Validating that %s exists", self.url.geturl())
+        res = requests_retry(retries=9).head(
+            url,
+            allow_redirects=True,
+            headers=headers,
+            timeout=HTTP_DOWNLOAD_TIMEOUT,
+        )
+        if res.status_code != HTTP_CODE_OK:
+            res.close()
+            self.logger.debug("Using GET because HEAD is not supported properly")
+            res = requests_retry(retries=9).get(
+                url,
+                allow_redirects=True,
+                stream=True,
+                headers=headers,
+                timeout=HTTP_DOWNLOAD_TIMEOUT,
+            )
+        return res
+
     def reader(self):
         res = None
         try:
@@ -663,7 +729,7 @@ class HttpDownloadAction(DownloadHandler):
                 headers=headers,
                 timeout=HTTP_DOWNLOAD_TIMEOUT,
             )
-            if res.status_code != requests.codes.OK:
+            if res.status_code != HTTP_CODE_OK:
                 # This is an Infrastructure error because the validate function
                 # checked that the file does exist.
                 raise InfrastructureError(
@@ -781,11 +847,14 @@ class PreDownloadedAction(Action):
     description = "Map to the correct downloaded path"
     summary = "pre downloaded"
 
-    def __init__(self, job: Job, key, url, path, params=None):
+    def __init__(self, job: Job, key, url, path, uniquify=True, params=None):
         super().__init__(job)
         self.key = key
         self.url = url
         self.path = path
+        self.uniquify = uniquify
+        if self.uniquify:
+            self.path = os.path.join(path, key)
         self.params = params
 
     def validate(self):
@@ -804,6 +873,14 @@ class PreDownloadedAction(Action):
                 value=image_arg,
             )
 
+        if self.key == "kernel" and ("kernel" in self.parameters):
+            self.set_namespace_data(
+                action="download-action",
+                label="type",
+                key=self.key,
+                value=self.parameters[self.key].get("type"),
+            )
+
     def run(self, connection, max_end_time):
         # In downloads://foo/bar.ext, "foo" is the "netloc", "/bar.ext" is
         # the path. But in downloads://foo.ext, "foo.ext" is the netloc, and
@@ -842,16 +919,22 @@ class PreDownloadedAction(Action):
         self.set_namespace_data(
             action="download-action", label="file", key=self.key, value=str(dest)
         )
-        if self.parameters["to"] == "tftp" or self.parameters["to"] == "nbd":
-            suffix = self.get_namespace_data(
+        if self.parameters.get("to") in ["tftp", "nbd"]:
+            if prefix := self.get_namespace_data(
                 action="tftp-deploy", label="tftp", key="suffix"
-            )
-            self.set_namespace_data(
-                action="download-action",
-                label="file",
-                key=self.key,
-                value=os.path.join(suffix, dest.name),
-            )
+            ):
+                _path_dirs = [prefix]
+                if self.uniquify:
+                    _path_dirs.append(self.key)
+                _path_dirs.append(os.path.basename(dest.name))
+                self.set_namespace_data(
+                    action="download-action",
+                    label="file",
+                    key=self.key,
+                    value=os.path.join(*_path_dirs),
+                )
+            else:
+                raise JobError(f"'{self.key}' path prefix is required but not found")
 
         if "lava-xnbd" in self.parameters and str(self.key) == "nbdroot":
             self.parameters["lava-xnbd"]["nbdroot"] = str(dest)
@@ -859,72 +942,6 @@ class PreDownloadedAction(Action):
         return connection
 
 
-class QCowConversionAction(Action):
-    """
-    explicit action for qcow conversion to avoid reliance
-    on filename suffix
-    """
-
-    name = "qcow2-convert"
-    description = "convert qcow image using qemu-img"
-    summary = "qcow conversion"
-
-    def __init__(self, job: Job, key):
-        super().__init__(job)
-        self.key = key
-
-    def run(self, connection, max_end_time):
-        connection = super().run(connection, max_end_time)
-        fname = self.get_namespace_data(
-            action="download-action", label=self.key, key="file"
-        )
-        origin = fname
-        # Remove the '.qcow2' extension and add '.img'
-        if fname.endswith(".qcow2"):
-            fname = fname[:-6]
-        fname += ".img"
-
-        self.logger.debug("Converting downloaded image from qcow2 to raw")
-        try:
-            subprocess.check_output(  # nosec - checked.
-                ["qemu-img", "convert", "-f", "qcow2", "-O", "raw", origin, fname],
-                stderr=subprocess.STDOUT,
-            )
-        except subprocess.CalledProcessError as exc:
-            self.logger.error("Unable to convert the qcow2 image")
-            self.logger.error(exc.output)
-            raise JobError(exc.output)
-
-        self.set_namespace_data(
-            action=self.name, label=self.key, key="file", value=fname
-        )
-        self.set_namespace_data(
-            action=self.name, label="file", key=self.key, value=fname
-        )
-        return connection
-
-
-class Download(Deployment):
-    """
-    Strategy class for a download deployment.
-    Downloads the relevant parts, copies to LXC if available.
-    """
-
-    name = "download"
-
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return DownloadAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if "to" not in parameters:
-            return False, '"to" is not in deploy parameters'
-        if parameters["to"] != "download":
-            return False, '"to" parameter is not "download"'
-        return True, "accepted"
-
-
 class DownloadAction(Action):
     name = "download-deploy"
     description = "download files and copy to LXC if available"
@@ -961,13 +978,13 @@ class DownloadAction(Action):
             self.pipeline.add_action(EnterFastbootAction(self.job))
 
         self.download_dir = self.mkdtemp()
-        for image in sorted(parameters["images"].keys()):
+        for image_key, image_params in parameters["images"].items():
             self.pipeline.add_action(
                 DownloaderAction(
                     self.job,
-                    image,
+                    image_key,
                     self.download_dir,
-                    params=parameters["images"][image],
+                    params=image_params,
                 )
             )
         if self.test_needs_overlay(parameters):
