@@ -20,48 +20,18 @@ from lava_dispatcher.actions.deploy.download import DownloaderAction
 from lava_dispatcher.actions.deploy.environment import DeployDeviceEnvironment
 from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.connections.serial import ConnectDevice
-from lava_dispatcher.logical import Deployment
 from lava_dispatcher.power import PDUReboot, PrePower, ReadFeedback, ResetDevice
-from lava_dispatcher.utils.fastboot import OptionalContainerFastbootAction
+from lava_dispatcher.utils.fastboot import (
+    DetectFastbootDevice,
+    OptionalContainerFastbootAction,
+)
 from lava_dispatcher.utils.lxc import is_lxc_requested
 from lava_dispatcher.utils.udev import WaitDeviceBoardID
 
 if TYPE_CHECKING:
-    from lava_dispatcher.action import Action
     from lava_dispatcher.job import Job
 
 
-class Fastboot(Deployment):
-    """
-    Strategy class for a fastboot deployment.
-    Downloads the relevant parts, copies to the locations using fastboot.
-    """
-
-    name = "fastboot"
-
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return FastbootAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if "to" not in parameters:
-            return False, '"to" is not in deploy parameters'
-        if parameters["to"] != "fastboot":
-            return False, '"to" parameter is not "fastboot"'
-        if "deploy" not in device["actions"]:
-            return False, '"deploy" is not in the device configuration actions'
-        if "adb_serial_number" not in device:
-            return False, '"adb_serial_number" is not in the device configuration'
-        if "fastboot_serial_number" not in device:
-            return False, '"fastboot_serial_number" is not in the device configuration'
-        if "fastboot_options" not in device:
-            return False, '"fastboot_options" is not in the device configuration'
-        if "fastboot" in device["actions"]["deploy"]["methods"]:
-            return True, "accepted"
-        return False, '"fastboot" was not in the device configuration deploy methods"'
-
-
 class FastbootAction(
     OptionalContainerFastbootAction
 ):  # pylint:disable=too-many-instance-attributes
@@ -82,40 +52,39 @@ class FastbootAction(
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         if self.test_needs_overlay(parameters):
             self.pipeline.add_action(OverlayAction(self.job))
+        board_id = self.job.device["fastboot_serial_number"]
         # Check if the device has a power command such as HiKey, Dragonboard,
         # etc. against device that doesn't like Nexus, etc.
         if self.job.device.get("fastboot_via_uboot", False):
             self.pipeline.add_action(ConnectDevice(self.job))
             self.pipeline.add_action(UBootEnterFastbootAction(self.job))
+            DetectFastbootDevice.add_if_needed(self)
         elif self.job.device.hard_reset_command:
             self.force_prompt = True
             self.pipeline.add_action(ConnectDevice(self.job))
             if not is_lxc_requested(self.job):
                 self.pipeline.add_action(PrePower(self.job))
             self.pipeline.add_action(ResetDevice(self.job))
+            DetectFastbootDevice.add_if_needed(self)
         else:
             self.pipeline.add_action(EnterFastbootAction(self.job))
 
         fastboot_dir = self.mkdtemp()
-        for image in sorted(parameters["images"].keys()):
+        for image_key, image_params in parameters["images"].items():
             self.pipeline.add_action(
-                DownloaderAction(
-                    self.job, image, fastboot_dir, params=parameters["images"][image]
-                )
+                DownloaderAction(self.job, image_key, fastboot_dir, params=image_params)
             )
-            if parameters["images"][image].get("apply-overlay", False):
+            if image_params.get("apply-overlay", False):
                 if self.test_needs_overlay(parameters):
-                    if parameters["images"][image].get("sparse", True):
+                    if image_params.get("sparse", True):
                         self.pipeline.add_action(
-                            ApplyOverlaySparseImage(self.job, image)
+                            ApplyOverlaySparseImage(self.job, image_key)
                         )
                     else:
-                        use_root_part = parameters["images"][image].get(
-                            "root_partition", False
-                        )
+                        use_root_part = image_params.get("root_partition", False)
                         self.pipeline.add_action(
                             ApplyOverlayImage(
-                                self.job, image, use_root_partition=use_root_part
+                                self.job, image_key, use_root_partition=use_root_part
                             )
                         )
 
@@ -182,10 +151,11 @@ class FastbootFlashOrderAction(OptionalContainerFastbootAction):
             self.errors = (
                 "Device not configured to support fastboot deployment connections."
             )
-        if "fastboot_serial_number" not in self.job.device:
-            self.errors = "device fastboot serial number missing"
-        elif self.job.device["fastboot_serial_number"] == "0000000000":
-            self.errors = "device fastboot serial number unset"
+        if not self.job.device.get("fastboot_auto_detection", False):
+            if "fastboot_serial_number" not in self.job.device:
+                self.errors = "device fastboot serial number missing"
+            elif self.job.device["fastboot_serial_number"] == "0000000000":
+                self.errors = "device fastboot serial number unset"
         if "flash_cmds_order" not in self.job.device:
             self.errors = "device flash commands order missing"
         if "fastboot_options" not in self.job.device:
@@ -195,7 +165,6 @@ class FastbootFlashOrderAction(OptionalContainerFastbootAction):
 
 
 class FastbootFlashAction(OptionalContainerFastbootAction):
-
     """
     Fastboot flash image.
     """
@@ -269,10 +238,11 @@ class FastbootReboot(OptionalContainerFastbootAction):
             self.errors = (
                 "Device not configured to support fastboot deployment connections."
             )
-        if "fastboot_serial_number" not in self.job.device:
-            self.errors = "device fastboot serial number missing"
-        elif self.job.device["fastboot_serial_number"] == "0000000000":
-            self.errors = "device fastboot serial number unset"
+        if not self.job.device.get("fastboot_auto_detection", False):
+            if "fastboot_serial_number" not in self.job.device:
+                self.errors = "device fastboot serial number missing"
+            elif self.job.device["fastboot_serial_number"] == "0000000000":
+                self.errors = "device fastboot serial number unset"
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
@@ -298,10 +268,11 @@ class FastbootRebootBootloader(OptionalContainerFastbootAction):
             self.errors = (
                 "Device not configured to support fastboot deployment connections."
             )
-        if "fastboot_serial_number" not in self.job.device:
-            self.errors = "device fastboot serial number missing"
-        elif self.job.device["fastboot_serial_number"] == "0000000000":
-            self.errors = "device fastboot serial number unset"
+        if not self.job.device.get("fastboot_auto_detection", False):
+            if "fastboot_serial_number" not in self.job.device:
+                self.errors = "device fastboot serial number missing"
+            elif self.job.device["fastboot_serial_number"] == "0000000000":
+                self.errors = "device fastboot serial number unset"
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
@@ -327,10 +298,11 @@ class FastbootRebootFastboot(OptionalContainerFastbootAction):
             self.errors = (
                 "Device not configured to support fastboot deployment connections."
             )
-        if "fastboot_serial_number" not in self.job.device:
-            self.errors = "device fastboot serial number missing"
-        elif self.job.device["fastboot_serial_number"] == "0000000000":
-            self.errors = "device fastboot serial number unset"
+        if not self.job.device.get("fastboot_auto_detection", False):
+            if "fastboot_serial_number" not in self.job.device:
+                self.errors = "device fastboot serial number missing"
+            elif self.job.device["fastboot_serial_number"] == "0000000000":
+                self.errors = "device fastboot serial number unset"
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
