@@ -11,27 +11,11 @@ from lava_common.exceptions import JobError
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.deploy.download import DownloaderAction
 from lava_dispatcher.actions.deploy.overlay import OverlayAction
-from lava_dispatcher.logical import Deployment
 
 if TYPE_CHECKING:
     from lava_dispatcher.job import Job
 
 
-class FVP(Deployment):
-    name = "fvp"
-
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return FVPDeploy(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        to = parameters.get("to")
-        if to != "fvp":
-            return False, "'to' was not fvp"
-        return True, "accepted"
-
-
 class FVPDeploy(Action):
     name = "fvp-deploy"
     description = "Download images for use with fvp"
@@ -44,12 +28,12 @@ class FVPDeploy(Action):
 
     def validate(self):
         super().validate()
-        if "images" not in self.parameters.keys():
+        if "images" not in self.parameters:
             raise JobError("No 'images' specified on FVP deploy")
-        for image in self.parameters["images"]:
-            if "overlays" in self.parameters["images"][image]:
+        for image_key, image_params in self.parameters["images"].items():
+            if "overlays" in image_params:
                 if self.parameters.get("format", None) == "disk":
-                    if "partition" not in self.parameters["images"][image]:
+                    if "partition" not in image_params:
                         self.errors = "Missing partition value for 'overlays' value for FVPDeploy."
 
     def populate(self, parameters):
@@ -61,7 +45,7 @@ class FVPDeploy(Action):
         if "images" in parameters:
             if not isinstance(parameters["images"], dict):
                 raise JobError("'deploy.images' should be a dictionary")
-            for k in sorted(parameters["images"].keys()):
+            for k in parameters["images"].keys():
                 self.pipeline.add_action(
                     DownloaderAction(
                         self.job,
