@@ -22,7 +22,6 @@ from lava_dispatcher.actions.deploy.vemsd import (
     UnmountVExpressMassStorageDevice,
 )
 from lava_dispatcher.connections.serial import DisconnectDevice
-from lava_dispatcher.logical import Deployment
 from lava_dispatcher.power import PowerOff, ResetDevice
 from lava_dispatcher.utils.udev import WaitUSBMassStorageDeviceAction
 
@@ -30,31 +29,6 @@ if TYPE_CHECKING:
     from lava_dispatcher.job import Job
 
 
-class Mps(Deployment):
-    """
-    Strategy class for a booting Arm MPS devices.
-    Downloads board recovery image and deploys to target
-    """
-
-    name = "mps"
-
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return MpsAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if "mps" not in device["actions"]["deploy"]["methods"]:
-            return False, '"mps" was not in the device configuration deploy methods'
-        if "to" not in parameters:
-            return False, '"to" was not in parameters'
-        if parameters["to"] != "mps":
-            return False, '"to" was not "mps"'
-        if "usb_filesystem_label" not in device:
-            return False, '"usb_filesystem_label" is not in the device configuration'
-        return True, "accepted"
-
-
 class MpsAction(Action):
     """
     Action for deploying firmware to a MPS board in the form
@@ -91,23 +65,23 @@ class MpsAction(Action):
         self.pipeline.add_action(DisconnectDevice(self.job))
         self.pipeline.add_action(ResetDevice(self.job))
         self.pipeline.add_action(WaitUSBMassStorageDeviceAction(self.job))
-        for image in parameters["images"].keys():
+        for image_key, image_params in parameters["images"].items():
             self.pipeline.add_action(
                 DownloaderAction(
                     self.job,
-                    image,
+                    image_key,
                     path=download_dir,
-                    params=parameters["images"][image],
+                    params=image_params,
                 )
             )
         self.pipeline.add_action(MountVExpressMassStorageDevice(self.job))
         # Sort the keys so recovery_image will be first
-        for image in sorted(parameters["images"].keys()):
-            if image == "recovery_image":
+        for image_key in sorted(parameters["images"].keys()):
+            if image_key == "recovery_image":
                 self.pipeline.add_action(ExtractVExpressRecoveryImage(self.job))
                 self.pipeline.add_action(DeployVExpressRecoveryImage(self.job))
             else:
-                self.pipeline.add_action(DeployMPSTestBinary(self.job, image))
+                self.pipeline.add_action(DeployMPSTestBinary(self.job, image_key))
 
         # Should we hard reboot the board after flash?
         params = self.job.device["actions"]["deploy"]["methods"]["mps"]["parameters"]
