@@ -16,35 +16,15 @@ from typing import TYPE_CHECKING
 from lava_common.exceptions import InfrastructureError, JobError, LAVABug
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.deploy.testdef import TestDefinitionAction
-from lava_dispatcher.logical import Deployment
 from lava_dispatcher.protocols.multinode import MultinodeProtocol
 from lava_dispatcher.protocols.vland import VlandProtocol
-from lava_dispatcher.utils.contextmanager import chdir
 from lava_dispatcher.utils.filesystem import check_ssh_identity_file
-from lava_dispatcher.utils.network import dispatcher_ip, rpcinfo_nfs
-from lava_dispatcher.utils.shell import which
-from lava_dispatcher.utils.strings import substitute_address_with_static_info
+from lava_dispatcher.utils.network import dispatcher_ip
 
 if TYPE_CHECKING:
     from lava_dispatcher.job import Job
 
 
-class Overlay(Deployment):
-    name = "overlay"
-
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return OverlayAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if "overlay" not in device["actions"]["deploy"]["methods"]:
-            return False, "'overlay' not in the device configuration deploy methods"
-        if parameters["to"] != "overlay":
-            return False, '"to" parameter is not "overlay"'
-        return True, "accepted"
-
-
 class CreateOverlay(Action):
     """
     Creates a temporary location into which the lava test shell scripts are installed.
@@ -159,7 +139,6 @@ class CreateOverlay(Action):
             skip_overlay_compression = True
         if not skip_overlay_compression:
             self.pipeline.add_action(CompressOverlay(self.job))
-        self.pipeline.add_action(PersistentNFSOverlay(self.job))  # idempotent
 
     def _export_data(self, fout, data, prefix):
         if isinstance(data, dict):
@@ -176,7 +155,7 @@ class CreateOverlay(Action):
             if isinstance(data, bool):
                 data = "1" if data else "0"
             elif isinstance(data, int):
-                data = data
+                data = str(data)
             else:
                 data = shlex.quote(data)
             self.logger.debug("- %s=%s", prefix, data)
@@ -266,6 +245,16 @@ class CreateOverlay(Action):
             self.logger.debug("LAVA metadata")
             self._export_data(fout, self.job.job_id, "LAVA_JOB_ID")
             self._export_data(fout, self.dispatcher_ip, "LAVA_DISPATCHER_IP")
+            self._export_data(
+                fout,
+                self.job.parameters.get("dispatcher", {}).get("prefix", ""),
+                "LAVA_DISPATCHER_PREFIX",
+            )
+
+            if http_cache := self.job.parameters["dispatcher"].get(
+                "http_url_format_string", ""
+            ):
+                self._export_data(fout, http_cache, "HTTP_CACHE")
 
         # Generate the file containing the secrets
         if "secrets" in self.job.parameters:
@@ -603,25 +592,20 @@ class CompressOverlay(Action):
             self.logger.error(self.errors)
             return connection
         connection = super().run(connection, max_end_time)
-        # This will split the path into two parts. Second part is only the
-        # bottom level directory. The first part is the rest.
-        # Example: /run/mount/lava/ will be split to /run/mount/ and lava/
-        # Used to create a tarball with only the bottom level dir included and
-        # not with the whole lava_test_results_dir property.
-        results_dir_list = os.path.split(os.path.normpath(lava_test_results_dir))
-        with chdir(
-            os.path.join(location, os.path.relpath(results_dir_list[0], os.sep))
-        ):
-            try:
-                with tarfile.open(output, "w:gz") as tar:
-                    tar.add("%s" % results_dir_list[1])
-                    # ssh authorization support
-                    if os.path.exists("./root/"):
-                        tar.add(".%s" % "/root/")
-            except tarfile.TarError as exc:
-                raise InfrastructureError(
-                    "Unable to create lava overlay tarball: %s" % exc
-                )
+
+        # For example, location="/foo/bar" and lava_test_results_dir="/lava-12345"
+        # The overlay working directory will be "/foo/bar/lava-12345".
+        # The tar archive will contain this directory as "lava-12345"
+        lava_path = os.path.abspath("%s/%s" % (location, lava_test_results_dir))
+        try:
+            with tarfile.open(output, "w:gz") as tar:
+                tar.add(lava_path, os.path.basename(lava_path))
+                # ssh authorization support
+                root_dir = os.path.join(location, "root")
+                if os.path.exists(root_dir):
+                    tar.add(root_dir, "./root")
+        except tarfile.TarError as exc:
+            raise InfrastructureError("Unable to create lava overlay tarball") from exc
 
         self.set_namespace_data(
             action=self.name, label="output", key="file", value=output
@@ -718,46 +702,3 @@ class SshAuthorize(Action):
         shutil.copyfile("%s.pub" % self.identity_file, authorize)
         os.chmod(authorize, 0o600)
         return connection
-
-
-class PersistentNFSOverlay(Action):
-    """
-    Instead of extracting, just populate the location of the persistent NFS
-    so that it can be mounted later when the overlay is applied.
-    """
-
-    name = "persistent-nfs-overlay"
-    description = "unpack overlay into persistent NFS"
-    summary = "add test overlay to NFS"
-
-    def validate(self):
-        super().validate()
-        persist = self.parameters.get("persistent_nfs")
-        if not persist:
-            return
-        if "address" not in persist:
-            self.errors = "Missing address for persistent NFS"
-            return
-        if ":" not in persist["address"]:
-            self.errors = (
-                "Unrecognised NFS URL: '%s'"
-                % self.parameters["persistent_nfs"]["address"]
-            )
-            return
-
-        persist["address"] = substitute_address_with_static_info(
-            persist["address"], self.job.device.get("static_info", [])
-        )
-
-        nfs_server, dirname = persist["address"].split(":")
-        which("rpcinfo")
-        self.errors = rpcinfo_nfs(nfs_server)
-        self.set_namespace_data(
-            action=self.name, label="nfs_address", key="nfsroot", value=dirname
-        )
-        self.set_namespace_data(
-            action=self.name, label="nfs_address", key="serverip", value=nfs_server
-        )
-
-        self.job.device["dynamic_data"]["NFS_ROOTFS"] = dirname
-        self.job.device["dynamic_data"]["NFS_SERVER_IP"] = nfs_server
