@@ -5,15 +5,9 @@
 # SPDX-License-Identifier: GPL-2.0-or-later
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
-
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.deploy.download import CopyToLxcAction, DownloaderAction
 from lava_dispatcher.connections.serial import ConnectDevice
-from lava_dispatcher.logical import Deployment
-
-if TYPE_CHECKING:
-    from lava_dispatcher.job import Job
 
 
 class RecoveryModeAction(Action):
@@ -26,11 +20,9 @@ class RecoveryModeAction(Action):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         recovery = self.job.device["actions"]["deploy"]["methods"]["recovery"]
         recovery_dir = self.mkdtemp()
-        for image in sorted(parameters["images"].keys()):
+        for image_key, image_params in parameters["images"].items():
             self.pipeline.add_action(
-                DownloaderAction(
-                    self.job, image, recovery_dir, params=parameters["images"][image]
-                )
+                DownloaderAction(self.job, image_key, recovery_dir, params=image_params)
             )
         self.pipeline.add_action(CopyToLxcAction(self.job))
 
@@ -41,21 +33,3 @@ class RecoveryModeAction(Action):
             # might not be a usable shell here, just power on.
             # FIXME: if used, FastbootAction must not try to reconnect
             self.pipeline.add_action(ConnectDevice(self.job))
-
-
-class RecoveryMode(Deployment):
-    name = "recovery-mode"
-
-    @classmethod
-    def action(cls, job: Job) -> Action:
-        return RecoveryModeAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if "recovery" not in device["actions"]["deploy"]["methods"]:
-            return False, "'recovery' not in the device configuration deploy methods"
-        if parameters["to"] != "recovery":
-            return False, '"to" parameter is not "recovery"'
-        if "images" not in parameters:
-            return False, '"images" is not in the deployment parameters'
-        return True, "accepted"
