@@ -11,6 +11,8 @@ import re
 import shutil
 from typing import TYPE_CHECKING
 
+import yaml
+
 from lava_common.constants import DEFAULT_TESTDEF_NAME_CLASS, DISPATCHER_DOWNLOAD_DIR
 from lava_common.decorators import nottest
 from lava_common.exceptions import InfrastructureError, JobError, LAVABug, TestError
@@ -33,7 +35,11 @@ def identify_test_definitions(test_info, namespace):
     test_list = []
     for test in test_info.get(namespace, []):
         if "definitions" in test["parameters"]:
-            test_list.append(test["parameters"]["definitions"])
+            testdefs = test["parameters"]["definitions"]
+            needs_delay = test["class"].needs_character_delay(test["parameters"])
+            for testdef in testdefs:
+                testdef["needs_character_delay"] = needs_delay
+            test_list.append(testdefs)
     return test_list
 
 
@@ -72,7 +78,8 @@ class RepoAction(Action):
         super().__init__(job)
         self.vcs = None
         self.runner = None
-        self.uuid = None
+        self.uuid: str | None = None
+        self.stage = 0
 
     @classmethod
     def select(cls, repo_type):
@@ -93,9 +100,7 @@ class RepoAction(Action):
         if "test_name" not in self.parameters:
             self.errors = "Unable to determine test_name"
             return
-        if not isinstance(self, InlineRepoAction) and not isinstance(
-            self, UrlRepoAction
-        ):
+        if not isinstance(self, (InlineRepoAction, UrlRepoAction)):
             if self.vcs is None:
                 raise LAVABug(
                     "RepoAction validate called super without setting the vcs"
@@ -211,6 +216,18 @@ class RepoAction(Action):
         self.set_namespace_data(
             action="test", label=self.uuid, key="testdef_metadata", value=val
         )
+        testdef_expected: list[str] = []
+        if "expected" in testdef:
+            testdef_expected = testdef["expected"]
+        if "expected" in self.parameters:
+            testdef_expected = self.parameters["expected"]
+        if testdef_expected:
+            self.set_namespace_data(
+                action="test",
+                label=self.uuid,
+                key="testdef_expected",
+                value=testdef_expected,
+            )
         if "parse" in testdef:
             pattern = testdef["parse"].get("pattern", "")
             fixup = testdef["parse"].get("fixupdict", "")
@@ -285,12 +302,16 @@ class GitRepoAction(RepoAction):
         if not revision:
             shallow = self.parameters.get("shallow", True)
 
+        # clone submodules if recursive is set
+        recursive = self.parameters.get("recursive", False)
+
         commit_id = self.vcs.clone(
             runner_path,
             shallow=shallow,
             revision=revision,
             branch=branch,
             history=self.parameters.get("history", True),
+            recursive=recursive,
         )
         if commit_id is None:
             raise InfrastructureError(
@@ -309,7 +330,7 @@ class GitRepoAction(RepoAction):
         try:
             with open(yaml_file) as test_file:
                 testdef = yaml_safe_load(test_file)
-        except OSError as exc:
+        except (OSError, yaml.YAMLError) as exc:
             raise JobError(
                 "Unable to open test definition '%s': %s"
                 % (self.parameters["path"], str(exc))
@@ -399,7 +420,7 @@ class UrlRepoAction(RepoAction):
 
         # Add 'url' as an alias to 'repository'. DownloaderAction requires an
         # 'url' key.
-        params = dict(**self.parameters, url=self.parameters["repository"])
+        params = {**self.parameters, "url": self.parameters["repository"]}
 
         self.download_dir = self.mkdtemp()
         self.action_key = "url_repo"
@@ -578,6 +599,9 @@ class TestDefinitionAction(Action):
                             "Invalid characters found in test definition name: %s"
                             % testdef["name"]
                         )
+                if "expected" in testdef:
+                    if not isinstance(testdef["expected"], list):
+                        self.errors = "'expected' should be a test case list"
         super().validate()
         for testdefs in self.test_list:
             for testdef in testdefs:
@@ -1029,8 +1053,17 @@ class TestRunnerAction(TestOverlayAction):
                     action="uuid", label="runner_path", key=self.parameters["test_name"]
                 )
             )
-            runsh.write("UUID=`cat uuid`\n")
+            runsh.write('UUID="$(cat uuid)"\n')
             runsh.write("set +x\n")
+            needs_delay = self.parameters.get("needs_character_delay")
+            delay = self.job.device.get("character_delays", {}).get("test", 0)
+            if needs_delay and delay:
+                self.logger.debug(
+                    f"A delay of {delay} milliseconds will be used for sending result signals"
+                )
+                delay = float(delay) / 1000
+                runsh.write(f"export CHARACTER_DELAY={delay}\n")
+                runsh.write(f"sleep {delay}\n")
             if lava_signal == "kmsg":
                 runsh.write("export KMSG=true\n")
                 runsh.write(
@@ -1040,11 +1073,14 @@ class TestRunnerAction(TestOverlayAction):
                 runsh.write('echo "<LAVA_SIGNAL_STARTRUN $TESTRUN_ID $UUID>"\n')
             runsh.write("set -x\n")
             steps = testdef.get("run", {}).get("steps", [])
-            for cmd in [step for step in steps if step is not None]:
-                if "--cmd" in cmd or "--shell" in cmd:
-                    cmd = re.sub(r"\$(\d+)\b", r"\\$\1", cmd)
-                runsh.write("%s\n" % cmd)
+            if steps is not None:
+                for cmd in [step for step in steps if step is not None]:
+                    if "--cmd" in cmd or "--shell" in cmd:
+                        cmd = re.sub(r"\$(\d+)\b", r"\\$\1", cmd)
+                    runsh.write("%s\n" % cmd)
             runsh.write("set +x\n")
+            if needs_delay and delay:
+                runsh.write(f"sleep {delay}\n")
             if lava_signal == "kmsg":
                 runsh.write("unset KMSG\n")
                 runsh.write(
