@@ -8,55 +8,21 @@ from __future__ import annotations
 import os
 import pathlib
 import shlex
-from typing import TYPE_CHECKING
 
 from lava_common.constants import LAVA_DOWNLOADS
+from lava_common.device_mappings import remove_device_container_mappings
 from lava_common.exceptions import LAVATimeoutError
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.deploy.overlay import CreateOverlay
 from lava_dispatcher.actions.test.multinode import MultinodeMixin
 from lava_dispatcher.actions.test.shell import TestShellAction
-from lava_dispatcher.logical import LavaTest
+from lava_dispatcher.connections.serial import DisconnectDevice
 from lava_dispatcher.power import ReadFeedback
 from lava_dispatcher.shell import ShellCommand, ShellSession
 from lava_dispatcher.utils.containers import DeviceContainerMappingMixin
 from lava_dispatcher.utils.docker import DockerRun
 from lava_dispatcher.utils.udev import get_udev_devices
 
-if TYPE_CHECKING:
-    from lava_dispatcher.job import Job
-
-
-class DockerTest(LavaTest):
-    """
-    DockerTest Strategy object
-    """
-
-    priority = 10
-
-    @classmethod
-    def action(cls, job: Job, parameters) -> Action:
-        return DockerTestAction(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if "definition" in parameters or "definitions" in parameters:
-            if "docker" in parameters:
-                return True, "accepted"
-        return False, "docker or definition(s) not in parameters"
-
-    @classmethod
-    def needs_deployment_data(cls, parameters):
-        return False
-
-    @classmethod
-    def needs_overlay(cls, parameters):
-        return True
-
-    @classmethod
-    def has_shell(cls, parameters):
-        return True
-
 
 class GetBoardId(Action):
     @property
@@ -80,6 +46,8 @@ class DockerTestAction(GetBoardId):
         self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         self.pipeline.add_action(DockerTestSetEnvironment(self.job))
         self.pipeline.add_action(CreateOverlay(self.job))
+        if parameters.get("disconnect_connection", False):
+            self.pipeline.add_action(DisconnectDevice(self.job))
         if "role" in parameters:
             self.pipeline.add_action(MultinodeDockerTestShell(self.job))
         else:
@@ -108,8 +76,13 @@ class DockerTestSetEnvironment(GetBoardId):
                 if "primary" in tags:
                     environment["LAVA_CONNECTION_COMMAND"] = connect
 
-        power_commands = ["hard_reset", "power_on", "power_off"]
-        for c in power_commands:
+        if boot_delay := self.job.device.get("character_delays", {}).get("boot"):
+            environment["LAVA_BOOT_CHARACTER_DELAYS"] = boot_delay
+        if test_delay := self.job.device.get("character_delays", {}).get("test"):
+            environment["LAVA_TEST_CHARACTER_DELAYS"] = test_delay
+
+        commands = ["connect", "hard_reset", "power_on", "power_off"]
+        for c in commands:
             cmd = self.job.device.get("commands", {}).get(c)
             if cmd:
                 if not isinstance(cmd, list):
@@ -172,37 +145,13 @@ class DockerTestShell(TestShellAction, GetBoardId, DeviceContainerMappingMixin):
         docker.prepare(action=self)
         docker.bind_mount(os.path.join(location, overlay), "/" + overlay)
 
-        docker_method_conf = (
+        docker_test_method_conf = (
             self.job.device["actions"]
             .get("test", {})
             .get("methods", {})
             .get("docker", {})
         )
-
-        # Preprocess docker option list, to better support partial
-        # overriding of them via device dict:
-        # 1. Filter out None, to make it easier to template
-        # YAML syntactic lists with Jinja2:
-        # '- {{ some_opt_from_device_dict }}'
-        # (if not default, will be set to None).
-        # 2. Flatten sublists, `- ['--opt1', '--opt2']`.
-        def preproc_opts(opts):
-            res = []
-            for o in opts:
-                if o is None:
-                    continue
-                elif isinstance(o, list):
-                    res += o
-                else:
-                    res.append(o)
-            return res
-
-        if "global_options" in docker_method_conf:
-            docker.add_docker_options(
-                *preproc_opts(docker_method_conf["global_options"])
-            )
-        if "options" in docker_method_conf:
-            docker.add_docker_run_options(*preproc_opts(docker_method_conf["options"]))
+        docker.add_device_docker_method_options(docker_test_method_conf)
 
         namespace = self.parameters.get(
             "downloads-namespace", self.parameters.get("namespace")
@@ -213,7 +162,7 @@ class DockerTestShell(TestShellAction, GetBoardId, DeviceContainerMappingMixin):
                 docker.bind_mount(downloads_dir, LAVA_DOWNLOADS)
 
         for bind_mount in self.test_docker_bind_mounts:
-            read_only = True if len(bind_mount) == 2 else False
+            read_only = len(bind_mount) == 2
             docker.bind_mount(bind_mount[0], bind_mount[1], read_only)
 
         docker.interactive()
@@ -227,7 +176,7 @@ class DockerTestShell(TestShellAction, GetBoardId, DeviceContainerMappingMixin):
         self.logger.debug("Starting docker test shell container: %s" % cmd)
         shell = ShellCommand(cmd, self.timeout, logger=self.logger)
 
-        shell_connection = ShellSession(self.job, shell)
+        shell_connection = ShellSession(shell)
         shell_connection.prompt_str = "docker-test-shell:"
         self.parameters["connection-namespace"] = "docker-test-shell"
         self.set_namespace_data(
@@ -248,16 +197,27 @@ class DockerTestShell(TestShellAction, GetBoardId, DeviceContainerMappingMixin):
 
         # share all the devices as there isn't a 1:1 relationship between
         # the trigger and actual sharing of the devices
+        shared_devices: list[str] = []
         for dev in devices:
             if not os.path.islink(dev):
                 self.trigger_share_device_with_container(dev)
+                shared_devices.append(dev)
+            else:
+                self.logger.debug(
+                    f"Device {dev} is a symlink, skipping sharing with container"
+                )
 
-        for dev in devices:
+        for dev in shared_devices:
             docker.wait_file(dev)
+            self.logger.info(
+                f"Shared device {dev} to docker container {docker.__name__}"
+            )
 
         try:
             super().run(shell_connection, max_end_time)
         finally:
+            remove_device_container_mappings(prefix + self.job.job_id)
+            self.logger.debug("Removed device container mappings")
             # finish the container
             shell_connection.finalise()
             docker.destroy()
