@@ -20,49 +20,14 @@ from lava_common.exceptions import (
     TestError,
 )
 from lava_dispatcher.action import Action, Pipeline
-from lava_dispatcher.logical import LavaTest, RetryAction
+from lava_dispatcher.actions.test.mixins import ReportMixin
+from lava_dispatcher.logical import RetryAction
 from lava_dispatcher.protocols.multinode import MultinodeProtocol
 from lava_dispatcher.utils.network import dispatcher_ip
 from lava_dispatcher.utils.strings import substitute
 
 if TYPE_CHECKING:
-    from lava_dispatcher.job import Job
-
-
-@nottest
-class TestInteractive(LavaTest):
-    """
-    TestInteractive Strategy object
-    """
-
-    @classmethod
-    def action(cls, job: Job, parameters) -> Action:
-        return TestInteractiveRetry(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        required_parms = ["name", "prompts", "script"]
-        if "interactive" in parameters:
-            for script in parameters["interactive"]:
-                if not all([x for x in required_parms if x in script]):
-                    return (
-                        False,
-                        "missing a required parameter from %s" % required_parms,
-                    )
-            return True, "accepted"
-        return False, '"interactive" not in parameters'
-
-    @classmethod
-    def needs_deployment_data(cls, parameters):
-        return False
-
-    @classmethod
-    def needs_overlay(cls, parameters):
-        return False
-
-    @classmethod
-    def has_shell(cls, parameters):
-        return False
+    from typing import Any
 
 
 @nottest
@@ -77,12 +42,17 @@ class TestInteractiveRetry(RetryAction):
 
 
 @nottest
-class TestInteractiveAction(Action):
+class TestInteractiveAction(ReportMixin, Action):
     name = "lava-test-interactive"
     description = "Executing lava-test-interactive"
     summary = "Lava Test Interactive"
     timeout_exception = LAVATimeoutError
 
+    def __init__(self, job):
+        super().__init__(job)
+        self.reports: dict[str, dict[str, Any]] = {}
+        self.report: dict[str, Any] = {}
+
     def populate(self, parameters):
         super().populate(parameters)
         proto = [
@@ -92,6 +62,14 @@ class TestInteractiveAction(Action):
         ]
         self.multinode_proto = proto[0] if proto else None
 
+    @staticmethod
+    def _get_test_suite_name(stage, script_name: str) -> str:
+        # NOTE: Test 'stage' is only incremented for shell tests that need
+        # overlay, so using the test stage as the test suite prefix here cannot
+        # really guarantee unique test suite ID. The document is updated to ask
+        # users use unique script name across all interactive test actions.
+        return "%s_%s" % (stage, script_name)
+
     def run(self, connection, max_end_time):
         super().run(connection, max_end_time)
 
@@ -158,9 +136,15 @@ class TestInteractiveAction(Action):
             # Set the connection prompts
             connection.prompt_str = script["prompts"]
 
+            test_suite_name = self._get_test_suite_name(
+                self.parameters["stage"], script["name"]
+            )
+            self.report = {}
+            self.reports[test_suite_name] = {"results": self.report, "ran": False}
+
             result = {
                 "definition": "lava",
-                "case": "%s_%s" % (self.parameters["stage"], script["name"]),
+                "case": test_suite_name,
                 "result": "fail",
             }
             try:
@@ -174,6 +158,12 @@ class TestInteractiveAction(Action):
                     )
                     self.run_script(test_connection, script, substitutions)
                 result["result"] = "pass"
+
+                if expected := script.get("expected"):
+                    self.handle_expected(expected, test_suite_name)
+
+                self.reports[test_suite_name]["ran"] = True
+                self.handle_summary(test_suite_name)
             finally:
                 # Log the current script result (even in case of error)
                 result["duration"] = "%.02f" % (time.monotonic() - start)
@@ -249,7 +239,9 @@ class TestInteractiveAction(Action):
                 command = substitute([cmd["command"]], substitutions)[0]
             start = time.monotonic()
             result = {
-                "definition": "%s_%s" % (self.parameters["stage"], script["name"]),
+                "definition": self._get_test_suite_name(
+                    self.parameters["stage"], script["name"]
+                ),
                 "case": cmd.get("name", ""),
                 "result": "fail",
             }
@@ -322,6 +314,7 @@ class TestInteractiveAction(Action):
                 if "name" in cmd:
                     result["duration"] = "%.02f" % (time.monotonic() - start)
                     self.logger.results(result)
+                    self.report[cmd["name"]] = result["result"]
 
     def listen_feedback(self, connection):
         if time.monotonic() - self.last_check <= connection.timeout:
@@ -350,6 +343,25 @@ class TestInteractiveAction(Action):
                 continue
             return ret
 
+    def cleanup(self, connection, max_end_time=None):
+        super().cleanup(connection, max_end_time=None)
+
+        scripts = self.parameters.get("interactive", [])
+        for script in scripts:
+            expected = script.get("expected")
+            if not expected:
+                continue
+
+            test_suite_name = self._get_test_suite_name(
+                self.parameters["stage"], script["name"]
+            )
+
+            if self.reports.get(test_suite_name, {}).get("ran"):
+                continue
+
+            self.report = self.reports.get(test_suite_name, {}).get("results", {})
+            self.handle_expected(expected, test_suite_name)
+
     def raise_exception(self, exc_name, exc_message):
         if exc_name == "InfrastructureError":
             raise InfrastructureError(exc_message)
