@@ -14,47 +14,15 @@ import pexpect
 from lava_common.decorators import nottest
 from lava_common.exceptions import ConnectionClosedError, LAVATimeoutError
 from lava_dispatcher.action import Action, Pipeline
-from lava_dispatcher.logical import LavaTest, RetryAction
+from lava_dispatcher.actions.test.mixins import ReportMixin
+from lava_dispatcher.logical import RetryAction
 
 if TYPE_CHECKING:
+    from typing import Any
+
     from lava_dispatcher.job import Job
 
 
-@nottest
-class TestMonitor(LavaTest):
-    """
-    LavaTestMonitor Strategy object
-    """
-
-    @classmethod
-    def action(cls, job: Job, parameters) -> Action:
-        return TestMonitorRetry(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        # TODO: Add configurable timeouts
-        required_parms = ["name", "start", "end", "pattern"]
-        if "monitors" in parameters:
-            for monitor in parameters["monitors"]:
-                for param in required_parms:
-                    if param not in monitor:
-                        return (False, "missing required parameter '%s'" % param)
-            return True, "accepted"
-        return False, '"monitors" not in parameters'
-
-    @classmethod
-    def needs_deployment_data(cls, parameters):
-        return False
-
-    @classmethod
-    def needs_overlay(cls, parameters):
-        return False
-
-    @classmethod
-    def has_shell(cls, parameters):
-        return False
-
-
 @nottest
 class TestMonitorRetry(RetryAction):
     name = "lava-test-monitor-retry"
@@ -67,7 +35,7 @@ class TestMonitorRetry(RetryAction):
 
 
 @nottest
-class TestMonitorAction(Action):
+class TestMonitorAction(ReportMixin, Action):
     """
     Watch the DUT output and match known results strings without any interaction.
     """
@@ -80,17 +48,24 @@ class TestMonitorAction(Action):
     def __init__(self, job: Job):
         super().__init__(job)
         self.test_suite_name = None
+        self.reports: dict[str, dict[str, Any]] = {}
         self.report = {}
         self.fixupdict = {}
         self.patterns = {}
 
+    @staticmethod
+    def _normalize_monitor_name(monitor_name: str) -> str:
+        return monitor_name.replace(" ", "-").lower()
+
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
 
         if not connection:
             raise ConnectionClosedError("Connection closed")
         for monitor in self.parameters["monitors"]:
-            self.test_suite_name = monitor["name"]
+            self.report = {}
+            self.test_suite_name = self._normalize_monitor_name(monitor["name"])
+            self.reports[self.test_suite_name] = {"results": self.report, "ran": False}
 
             self.fixupdict = monitor.get("fixupdict")
 
@@ -114,6 +89,11 @@ class TestMonitorAction(Action):
                 ):
                     pass
 
+            if expected := monitor.get("expected"):
+                self.handle_expected(expected, self.test_suite_name)
+
+            self.reports[self.test_suite_name]["ran"] = True
+            self.handle_summary(self.test_suite_name)
         return connection
 
     def _keep_running(self, test_connection, timeout=120):
@@ -153,9 +133,7 @@ class TestMonitorAction(Action):
                         case_id = re.sub(r"\W+", "_", case_id)
                         self.logger.debug("test_case_id: %s", case_id)
                         results = {
-                            "definition": self.test_suite_name.replace(
-                                " ", "-"
-                            ).lower(),
+                            "definition": self.test_suite_name,
                             "case": case_id,
                             "level": self.level,
                             "result": match["result"],
@@ -165,6 +143,7 @@ class TestMonitorAction(Action):
                         if "units" in match:
                             results.update({"units": match["units"]})
                         self.logger.results(results)
+                        self.report[case_id] = match["result"]
             else:
                 if all(x in match for x in ["test_case_id", "measurement"]):
                     if match["measurement"] and match["test_case_id"]:
@@ -173,9 +152,7 @@ class TestMonitorAction(Action):
                         case_id = re.sub(r"\W+", "_", case_id)
                         self.logger.debug("test_case_id: %s", case_id)
                         results = {
-                            "definition": self.test_suite_name.replace(
-                                " ", "-"
-                            ).lower(),
+                            "definition": self.test_suite_name,
                             "case": case_id,
                             "level": self.level,
                             "result": "pass",
@@ -184,5 +161,23 @@ class TestMonitorAction(Action):
                         if "units" in match:
                             results.update({"units": match["units"]})
                         self.logger.results(results)
+                        self.report[case_id] = "pass"
             ret_val = True
         return ret_val
+
+    def cleanup(self, connection, max_end_time=None):
+        super().cleanup(connection, max_end_time=None)
+
+        monitors = self.parameters.get("monitors", [])
+        for monitor in monitors:
+            expected = monitor.get("expected")
+            if not expected:
+                continue
+
+            test_suite_name = self._normalize_monitor_name(monitor["name"])
+
+            if self.reports.get(test_suite_name, {}).get("ran"):
+                continue
+
+            self.report = self.reports.get(test_suite_name, {}).get("results", {})
+            self.handle_expected(expected, test_suite_name)
