@@ -5,7 +5,6 @@
 # SPDX-License-Identifier: GPL-2.0-or-later
 from __future__ import annotations
 
-import decimal
 import logging
 import re
 import time
@@ -22,17 +21,20 @@ from lava_common.exceptions import (
 )
 from lava_common.yaml import yaml_safe_dump
 from lava_dispatcher.action import Action, Pipeline
+from lava_dispatcher.actions.test.mixins import ReportMixin
 from lava_dispatcher.connection import SignalMatch
-from lava_dispatcher.logical import LavaTest, RetryAction
+from lava_dispatcher.logical import RetryAction
 
 if TYPE_CHECKING:
+    from typing import Any
+
     from lava_dispatcher.job import Job
 
 
 def handle_testcase(params):
     data = {}
     for param in params:
-        parts = param.split("=")
+        parts = param.split("=", maxsplit=1)
         if len(parts) == 2:
             key, value = parts
             key = key.lower()
@@ -42,35 +44,6 @@ def handle_testcase(params):
     return data
 
 
-class TestShell(LavaTest):
-    """
-    LavaTestShell Strategy object
-    """
-
-    @classmethod
-    def action(cls, job: Job, parameters) -> Action:
-        return TestShellRetry(job)
-
-    @classmethod
-    def accepts(cls, device, parameters):
-        if "definitions" in parameters:
-            return True, "accepted"
-        return False, '"definitions" not in parameters'
-
-    @classmethod
-    def needs_deployment_data(cls, parameters):
-        """Some, not all, deployments will want deployment_data"""
-        return True
-
-    @classmethod
-    def needs_overlay(cls, parameters):
-        return True
-
-    @classmethod
-    def has_shell(cls, parameters):
-        return True
-
-
 @nottest
 class TestShellRetry(RetryAction):
     name = "lava-test-retry"
@@ -141,7 +114,7 @@ class PatternFixup:
 
 
 @nottest
-class TestShellAction(Action):
+class TestShellAction(ReportMixin, Action):
     """
     Sets up and runs the LAVA Test Shell Definition scripts.
     Supports a pre-command-list of operations necessary on the
@@ -160,6 +133,7 @@ class TestShellAction(Action):
         self.signal_match = SignalMatch()
         self.definition = None
         self.testset_name = None
+        self.reports: dict[str, dict[str, Any]] = {}
         self.report = {}
         self.start = None
         self.testdef_dict = {}
@@ -292,7 +266,9 @@ class TestShellAction(Action):
 
         # source the environment file containing device-specific shell variables
         connection.sendline(
-            ". %s/environment" % lava_test_results_dir, delay=self.character_delay
+            ". %s/environment" % lava_test_results_dir,
+            delay=self.character_delay,
+            check=True,
         )
         connection.wait()
 
@@ -361,9 +337,6 @@ class TestShellAction(Action):
                 self.logger.results(self.current_run)
                 self.current_run = None
 
-        # Only print if the report is not empty
-        if self.report:
-            self.logger.debug(yaml_safe_dump(self.report, default_flow_style=False))
         if self.errors:
             raise TestError(self.errors)
         return connection
@@ -391,6 +364,10 @@ class TestShellAction(Action):
             raise TestError("Invalid signal")
         uuid = params[1]
         self.start = time.monotonic()
+        self.report = {}
+        # For each test definition, save what results were reported and
+        # run start/end status, so action cleanup knows what to do.
+        self.reports[self.definition] = {"results": self.report, "ran": False}
         self.logger.info("Starting test lava.%s (%s)", self.definition, uuid)
         # set the pattern for this run from pattern_dict
         testdef_index = self.get_namespace_data(
@@ -433,6 +410,12 @@ class TestShellAction(Action):
             uuid = params[1]
         except IndexError:
             raise TestError("Invalid signal")
+
+        if expected := self.get_namespace_data(
+            action="test", label=uuid, key="testdef_expected"
+        ):
+            self.handle_expected(expected, self.definition)
+
         # remove the pattern for this run from pattern_dict
         self._reset_patterns()
         # catch error in ENDRUN being handled without STARTRUN
@@ -468,8 +451,12 @@ class TestShellAction(Action):
             res["commit_id"] = commit_id
 
         self.logger.results(res)
+        if self.definition in self.reports:
+            self.reports[self.definition]["ran"] = True
         self.start = None
 
+        self.handle_summary(self.definition)
+
     @nottest
     def signal_test_case(self, params):
         # If the STARTRUN signal was not received correctly, we cannot continue
@@ -498,10 +485,9 @@ class TestShellAction(Action):
         # check for measurements
         if "measurement" in res:
             try:
-                measurement = decimal.Decimal(res["measurement"])
-            except decimal.InvalidOperation:
+                res_data["measurement"] = float(res["measurement"])
+            except ValueError:
                 raise TestError("Invalid measurement %s" % res["measurement"])
-            res_data["measurement"] = float(measurement)
             if "units" in res:
                 res_data["units"] = res["units"]
 
@@ -598,10 +584,9 @@ class TestShellAction(Action):
             # check for measurements
             if "measurement" in res:
                 try:
-                    measurement = decimal.Decimal(res["measurement"])
-                except decimal.InvalidOperation:
+                    res_data["measurement"] = float(res["measurement"])
+                except ValueError:
                     raise TestError("Invalid measurement %s" % res["measurement"])
-                res_data["measurement"] = float(measurement)
                 if "units" in res:
                     res_data["units"] = res["units"]
 
@@ -682,6 +667,46 @@ class TestShellAction(Action):
         retval = test_connection.expect(list(self.patterns.values()), timeout=timeout)
         return self.check_patterns(list(self.patterns.keys())[retval], test_connection)
 
+    def cleanup(self, connection, max_end_time=None):
+        super().cleanup(connection, max_end_time=None)
+
+        test_list = self.get_namespace_data(
+            action="test-definition",
+            label="test-definition",
+            key="test_list",
+        )
+        # Test list is set when test definition deploy actions are populated. It
+        # is needed for calculating test suite index.
+        if not test_list:
+            return
+
+        testdefs = self.parameters.get("definitions", [])
+        for sub_index, testdef in enumerate(testdefs):
+            # Since test definitions may be unreachable or not yet deployed,
+            # the feature is limited to expected test case list from job
+            # definitions, ensuring the output is always consistent.
+            expected = testdef.get("expected")
+            if not expected:
+                continue
+
+            # The index equals the number of all test definitions in all test
+            # actions defined before the current action within the same
+            # namespace, plus the index of the current test definition within
+            # the current test action.
+            index = (
+                sum(len(s) for s in test_list[: self.parameters["stage"]]) + sub_index
+            )
+            test_suite = f"{index}_{testdef['name']}"
+
+            # At the end of each successful test run, the expected test case
+            # list is already checked.
+            if self.reports.get(test_suite, {}).get("ran"):
+                continue
+
+            # Reset test report for comparison.
+            self.report = self.reports.get(test_suite, {}).get("results", {})
+            self.handle_expected(expected, test_suite)
+
     class SignalDirector:
         # FIXME: create proxy handlers
         def __init__(self, protocol=None):
