@@ -14,7 +14,6 @@ import json
 import logging
 import logging.handlers
 import os
-import shutil
 import sqlite3
 import subprocess
 import sys
@@ -26,8 +25,9 @@ from dataclasses import dataclass
 from functools import partial
 from json import loads as json_loads
 from pathlib import Path
+from shutil import rmtree
 from signal import Signals
-from typing import Any
+from typing import TYPE_CHECKING
 
 import aiohttp
 import sentry_sdk
@@ -40,6 +40,10 @@ from lava_common.version import __version__
 from lava_common.worker import get_parser, init_sentry_sdk
 from lava_common.yaml import yaml_safe_load
 
+if TYPE_CHECKING:
+    from argparse import Namespace as argparse_ns
+    from typing import Any
+
 ###########
 # Constants
 ###########
@@ -53,7 +57,7 @@ WORKER_DIR = Path(_WORKER_DIR_STR)
 # Globals
 #########
 # Create the logger that will be configured later
-logging.Formatter.convert = time.gmtime
+logging.Formatter.converter = time.gmtime
 LOG = logging.getLogger("lava-worker")
 FORMAT = "%(asctime)-15s %(levelname)7s %(message)s"
 
@@ -124,12 +128,13 @@ async def aiohttp_get(
         headers["LAVA-Token"] = token
 
     try:
-        async with session.get(
-            url, params=params, headers=headers, timeout=TIMEOUT
-        ) as request:
+        async with session.get(url, params=params, headers=headers) as request:
             return Response(request.status, await request.text())
     except aiohttp.ClientError as exc:
         return Response(503, str(exc))
+    except asyncio.TimeoutError as exc:
+        LOG.error(f"Timeout during get request to {url}: {exc}")
+        return Response(504, f"Timeout error: {str(exc)}")
 
 
 async def aiohttp_post(
@@ -140,12 +145,13 @@ async def aiohttp_post(
         headers["LAVA-Token"] = token
 
     try:
-        async with session.post(
-            url, data=data, headers=headers, timeout=TIMEOUT
-        ) as request:
+        async with session.post(url, data=data, headers=headers) as request:
             return Response(request.status, await request.text())
     except aiohttp.ClientError as exc:
         return Response(503, str(exc))
+    except asyncio.TimeoutError as exc:
+        LOG.error(f"Timeout during post request to {url}: {exc}")
+        return Response(504, f"Timeout error: {str(exc)}")
 
 
 ###############
@@ -222,6 +228,10 @@ def start_job(
         return None
 
 
+def rmtree_job_dir(dir_path: str) -> None:
+    rmtree(dir_path, ignore_errors=True)
+
+
 async def cleanup_job(prefix: str, job_id: int) -> None:
     loop = asyncio.get_running_loop()
     for directory, pattern in STALE_CONFIG.items():
@@ -230,10 +240,7 @@ async def cleanup_job(prefix: str, job_id: int) -> None:
         if not dir_path.exists():
             continue
         LOG.debug("[%d] Removing %s", job_id, dir_path)
-        await loop.run_in_executor(
-            THREAD_EXECUTOR,
-            partial(shutil.rmtree, str(dir_path), ignore_errors=True),
-        )
+        await loop.run_in_executor(THREAD_EXECUTOR, rmtree_job_dir, str(dir_path))
 
     LOG.debug("[%d] Finished cleanup", job_id)
 
@@ -261,18 +268,42 @@ class Job:
 
     def errors(self) -> str:
         with contextlib.suppress(OSError, UnicodeDecodeError):
-            return (self.base_dir / "stderr").read_text(
-                encoding="utf-8", errors="replace"
+            return (
+                (self.base_dir / "stderr")
+                .read_text(encoding="utf-8", errors="backslashreplace")
+                .replace("\x00", "\ufffd")
             )
         return ""
 
     def description(self) -> str:
         with contextlib.suppress(OSError):
-            return (self.base_dir / "description.yaml").read_text(
-                encoding="utf-8", errors="backslashreplace"
+            return (
+                (self.base_dir / "description.yaml")
+                .read_text(encoding="utf-8", errors="backslashreplace")
+                .replace("\x00", "\ufffd")
             )
         return ""
 
+    def finalize_timed_out(self) -> bool:
+        timeout = None
+        if desc_str := self.description():
+            with contextlib.suppress(yaml.YAMLError):
+                desc = yaml_safe_load(desc_str)
+                with contextlib.suppress(AttributeError):
+                    if pipeline := desc.get("pipeline", []):
+                        # Assume finalize always is the last action.
+                        action = pipeline[-1]
+                        if action.get("name") == "finalize":
+                            timeout = action.get("timeout", 0)
+
+        if timeout and time.monotonic() - self.last_update < timeout:
+            LOG.debug(
+                f"[{self.job_id}] Waiting for job to finalize (timeout {timeout}s)"
+            )
+            return False
+
+        return True
+
     def result(self) -> dict[str, Any]:
         with contextlib.suppress(OSError, yaml.YAMLError):
             data = yaml_safe_load((self.base_dir / "result.yaml").read_bytes())
@@ -284,7 +315,7 @@ class Job:
         # If the pid is 0, just skip because lava-run was not started
         if self.pid == 0:
             return
-        os.kill(self.pid, Signals.SIGKILL)
+        os.killpg(self.pid, Signals.SIGKILL)
 
     def terminate(self) -> None:
         # If the pid is 0, just skip because lava-run was not started
@@ -470,6 +501,12 @@ async def finish_job(
         LOG.debug("[%d] --> %s", job.job_id, ret.text)
         if ret.status_code != 404:  # If the job is not present on lava-server delete it
             return
+        # Check if the '404' is returned by 'URL_JOBS' api.
+        json_data = {}
+        with contextlib.suppress(aiohttp.ContentTypeError):
+            json_data = ret.json()
+        if json_data.get("error") != f"Unknown job '{job.job_id}'":
+            return
 
     # Remove stale resources
     prefix = "" if job is None else job.prefix
@@ -488,7 +525,7 @@ async def check(session: aiohttp.ClientSession, url: str, jobs: JobsDB) -> None:
         if not job.is_running():
             # wait for the job
             try:
-                os.waitpid(job.pid, 0)
+                os.waitpid(job.pid, os.WNOHANG)
             except OSError as exc:
                 LOG.debug(
                     "[%d] unable to wait for the process: %s", job.job_id, str(exc)
@@ -501,18 +538,23 @@ async def check(session: aiohttp.ClientSession, url: str, jobs: JobsDB) -> None:
         if not job.is_running():
             # wait for the job
             try:
-                os.waitpid(job.pid, 0)
+                os.waitpid(job.pid, os.WNOHANG)
             except OSError as exc:
                 LOG.debug(
                     "[%d] unable to wait for the process: %s", job.job_id, str(exc)
                 )
             LOG.info("[%d] canceling -> finished", job.job_id)
             jobs.update(job.job_id, Job.FINISHED)
-
-        elif time.monotonic() - job.last_update > FINISH_MAX_DURATION:
+        elif (
+            time.monotonic() - job.last_update > FINISH_MAX_DURATION
+            and job.finalize_timed_out()
+        ):
             LOG.info("[%d] not finishing => killing", job.job_id)
             job.kill()
-        elif time.monotonic() - job.last_update > FINISH_MAX_DURATION / 2:
+        elif (
+            time.monotonic() - job.last_update > FINISH_MAX_DURATION / 2
+            and job.finalize_timed_out()
+        ):
             LOG.info("[%d] not finishing => second signal", job.job_id)
             job.terminate()
 
@@ -659,23 +701,30 @@ async def start(
         LOG.debug("[%d] --> %s", job_id, ret.text)
 
 
+async def get_job_data(
+    session: aiohttp.ClientSession, options: argparse_ns
+) -> dict[str, Any]:
+    try:
+        data = await ping(session, options.url, options.token, options.name)
+        return data
+    except ServerUnavailable:
+        LOG.error("-> server unavailable")
+        return {}
+    except VersionMismatch as exc:
+        if options.exit_on_version_mismatch:
+            raise exc
+        return {}
+
+
 ###############
 # Entrypoints #
 ###############
 async def handle(options, session: aiohttp.ClientSession, jobs: JobsDB) -> None:
-    name: str = options.name
-    token: str = options.token
     url: str = options.url
     job_log_interval: int = options.job_log_interval
 
-    try:
-        data = await ping(session, url, token, name)
-    except ServerUnavailable:
-        LOG.error("-> server unavailable")
-        return
-    except VersionMismatch as exc:
-        if options.exit_on_version_mismatch:
-            raise exc
+    data = await get_job_data(session, options)
+    if not data:
         return
 
     # running jobs
@@ -834,14 +883,17 @@ async def main() -> int:
     async with aiohttp.ClientSession(
         headers={
             "User-Agent": f"lava-worker {__version__}",
-        }
+            "Connection": "keep-alive",
+        },
+        connector=aiohttp.TCPConnector(limit=10, keepalive_timeout=TIMEOUT),
+        timeout=aiohttp.ClientTimeout(total=ping_interval),
     ) as session:
         try:
             if options.username is not None:
                 LOG.info("[INIT] Token  : '<auto register with %s>'", options.username)
                 password = getpass.getpass()
                 options.token = await register(
-                    options.url, options.name, options.username, password
+                    session, options.url, options.name, options.username, password
                 )
                 options.token_file.write_text(options.token, encoding="utf-8")
                 options.token_file.chmod(0o600)
@@ -870,10 +922,7 @@ async def main() -> int:
 
             LOG.debug(f"LAVA worker pid is {os.getpid()}")
             for sig in (Signals.SIGINT, Signals.SIGTERM):
-                loop.add_signal_handler(
-                    sig,
-                    partial(ask_exit, sig.name, group),
-                )
+                loop.add_signal_handler(sig, partial(ask_exit, sig.name, group))
 
             loop.add_signal_handler(
                 Signals.SIGCHLD, partial(sigchld_handler, session, options.url, jobs)
@@ -886,7 +935,14 @@ async def main() -> int:
             if options.wait_jobs:
                 LOG.info("[EXIT] Wait for jobs to finish")
                 while True:
-                    await check(session, options.url, jobs)
+                    data = await get_job_data(session, options)
+                    async with jobs.lock:
+                        if data:
+                            for job in data.get("cancel", []):
+                                cancel(options.url, jobs, job["id"], job["token"])
+
+                        await check(session, options.url, jobs)
+
                     all_ids = jobs.all_ids()
                     LOG.info(
                         "[EXIT] => %d jobs ([%s])",
