@@ -31,6 +31,7 @@ from rest_framework.permissions import (
     BasePermission,
     DjangoModelPermissions,
     DjangoModelPermissionsOrAnonReadOnly,
+    IsAuthenticated,
     IsAuthenticatedOrReadOnly,
 )
 from rest_framework.response import Response
@@ -58,6 +59,7 @@ from lava_scheduler_app.models import (
     DeviceType,
     GroupDevicePermission,
     GroupDeviceTypePermission,
+    RemoteArtifactsAuth,
     Tag,
     TestJob,
     Worker,
@@ -409,6 +411,10 @@ class TestJobViewSet(viewsets.ModelViewSet):
         strict = request.data.get("strict", False)
         if not definition:
             raise ValidationError({"definition": "Test job definition is required."})
+        if not isinstance(definition, str):
+            raise ValidationError(
+                {"definition": "Test job definition should be a string."}
+            )
 
         data = yaml_safe_load(definition)
         try:
@@ -420,7 +426,8 @@ class TestJobViewSet(viewsets.ModelViewSet):
             return Response({"message": "Job valid."}, status=status.HTTP_200_OK)
         except voluptuous.Invalid as exc:
             return Response(
-                {"message": "Job invalid: %s" % exc.msg}, status=status.HTTP_200_OK
+                {"message": "Job invalid: %s @ %s" % (exc.msg, exc.path)},
+                status=status.HTTP_200_OK,
             )
 
     @action(
@@ -442,7 +449,7 @@ class TestJobViewSet(viewsets.ModelViewSet):
             )
         except voluptuous.MultipleInvalid as exc:
             return Response(
-                {"message": "Test defnition invalid: %s" % str(exc)},
+                {"message": "Test definition invalid: %s" % str(exc)},
                 status=status.HTTP_200_OK,
             )
 
@@ -494,7 +501,10 @@ class TestJobViewSet(viewsets.ModelViewSet):
         # this method would report as you successfully cancelled the job
         # even if you don't have required permissions.
         with transaction.atomic():
-            job = TestJob.objects.select_for_update().get(pk=kwargs["pk"])
+            try:
+                job = TestJob.objects.select_for_update().get(pk=kwargs["pk"])
+            except TestJob.DoesNotExist:
+                raise NotFound()
             job.cancel(request.user)
         return Response(
             {"message": "Job cancel signal sent."}, status=status.HTTP_200_OK
@@ -543,7 +553,10 @@ class TestSuiteViewSet(NestedViewSetMixin, viewsets.ReadOnlyModelViewSet):
     def get_queryset(self):
         job_id = self.kwargs["parent_lookup_job_id"]
 
-        job = TestJob.objects.select_related("submitter").get(id=job_id)
+        try:
+            job = TestJob.objects.select_related("submitter").get(id=job_id)
+        except TestJob.DoesNotExist:
+            raise NotFound
 
         if not job.can_view(self.request.user):
             raise PermissionDenied
@@ -598,7 +611,10 @@ class TestCaseViewSet(NestedViewSetMixin, viewsets.ReadOnlyModelViewSet):
     def get_queryset(self):
         job_id = self.kwargs["parent_lookup_suite__job_id"]
 
-        job = TestJob.objects.select_related("submitter").get(id=job_id)
+        try:
+            job = TestJob.objects.select_related("submitter").get(id=job_id)
+        except TestJob.DoesNotExist:
+            raise NotFound
 
         if not job.can_view(self.request.user):
             raise PermissionDenied
@@ -1137,3 +1153,15 @@ class SystemViewSet(viewsets.ViewSet):
         ```
         """
         return Response(data={"user": request.user.username})
+
+
+class RemoteArtifactTokenViewSet(viewsets.ModelViewSet):
+    serializer_class = serializers.RemoteArtifactTokenSerializer
+    permission_classes = [IsAuthenticated]
+    lookup_field = "name"
+
+    def get_queryset(self):
+        return RemoteArtifactsAuth.objects.filter(user=self.request.user)
+
+    def perform_create(self, serializer):
+        serializer.save(user=self.request.user)
