@@ -81,14 +81,12 @@ class ResultsAPI(ExposedAPI):
             content_type = Query.get_content_type(entity)
         except InvalidContentTypeError:
             raise xmlrpc.client.Fault(
-                400,
-                "Wrong table name in entity parameter. Please refer to query docs.",
+                400, "Wrong table name in entity parameter. Please refer to query docs."
             )
 
         if content_type.model_class() not in QueryCondition.RELATION_MAP:
             raise xmlrpc.client.Fault(
-                400,
-                "Wrong table name in entity parameter. Please refer to query docs.",
+                400, "Wrong table name in entity parameter. Please refer to query docs."
             )
 
         conditions = Query.parse_conditions(content_type, conditions)
