@@ -14,6 +14,7 @@ TestSuite is based on the test definition
 TestSet can be enabled within a test definition run step
 TestCase is a single lava-test-case record or Action result.
 """
+from __future__ import annotations
 
 import contextlib
 import logging
@@ -132,9 +133,7 @@ class QueryMaterializedView(MaterializedView):
             # will be quoted with single quotes making
             # it incompatible with table_name identifier
             query_id_str_quoted = quote_ident(query_id_str, cursor.cursor)
-            cursor.execute(
-                f"REFRESH MATERIALIZED VIEW {query_id_str_quoted}",
-            )
+            cursor.execute(f"REFRESH MATERIALIZED VIEW {query_id_str_quoted}")
 
     @classmethod
     def drop(cls, query_id):
@@ -144,9 +143,7 @@ class QueryMaterializedView(MaterializedView):
             # will be quoted with single quotes making
             # it incompatible with table_name identifier
             query_id_str_quoted = quote_ident(query_id_str, cursor.cursor)
-            cursor.execute(
-                f"DROP MATERIALIZED VIEW IF EXISTS {query_id_str_quoted}",
-            )
+            cursor.execute(f"DROP MATERIALIZED VIEW IF EXISTS {query_id_str_quoted}")
 
     @classmethod
     def view_exists(cls, query_id):
@@ -192,27 +189,15 @@ class TestSuite(models.Model, Queryable):
         if not hasattr(self, "_testcase_count"):
             res = self.testcase_set.values("result")
             res = res.aggregate(
-                PASS=Count(
-                    "pk",
-                    filter=Q(result=TestCase.RESULT_PASS),
-                ),
-                FAIL=Count(
-                    "pk",
-                    filter=Q(result=TestCase.RESULT_FAIL),
-                ),
-                SKIP=Count(
-                    "pk",
-                    filter=Q(result=TestCase.RESULT_SKIP),
-                ),
-                UNKNOWN=Count(
-                    "pk",
-                    filter=Q(result=TestCase.RESULT_UNKNOWN),
-                ),
+                PASS=Count("pk", filter=Q(result=TestCase.RESULT_PASS)),
+                FAIL=Count("pk", filter=Q(result=TestCase.RESULT_FAIL)),
+                SKIP=Count("pk", filter=Q(result=TestCase.RESULT_SKIP)),
+                UNKNOWN=Count("pk", filter=Q(result=TestCase.RESULT_UNKNOWN)),
             )
             self._testcase_count = {k.lower(): (v or 0) for (k, v) in res.items()}
 
         if value is None:
-            return sum([v for (k, v) in self._testcase_count.items()])
+            return sum(self._testcase_count.values())
         return self._testcase_count[value]
 
     def get_passfail_results(self):
@@ -322,6 +307,19 @@ class TestCase(models.Model, Queryable):
     """
 
     class Meta:
+        indexes = (
+            models.Index(
+                fields=("-suite",),
+                condition=(
+                    Q(name="job")
+                    & Q(result=1)  # HACK: Refers to RESULT_FAIL
+                    & Q(
+                        metadata__regex="error_type: (Configuration|Infrastructure|Bug)"
+                    )
+                ),
+                name="testcases_with_job_errors_idx",
+            ),
+        )
         constraints = (
             models.UniqueConstraint(
                 fields=("suite",),
@@ -671,7 +669,7 @@ class Query(models.Model):
     def has_view(self):
         return QueryMaterializedView.view_exists(self.id)
 
-    def get_results(self, user, order_by=["-id"]):
+    def get_results(self, user, order_by=("-id",)):
         """Used to get query results for persistent queries."""
 
         omitted_list = QueryOmitResult.objects.filter(query=self).values_list(
@@ -693,6 +691,8 @@ class Query(models.Model):
                 view = TestCaseViewFactory(self)
             elif self.content_type.model_class() == TestSuite:
                 view = TestSuiteViewFactory(self)
+            else:
+                raise ValueError(f"Unknown model {self.content_type.model_class()!r}")
 
             return (
                 view.__class__.objects.all()
@@ -702,7 +702,7 @@ class Query(models.Model):
             )
 
     @classmethod
-    def get_queryset(cls, content_type, conditions, limit=None, order_by=["-id"]):
+    def get_queryset(cls, content_type, conditions, limit=None, order_by=("-id",)):
         """Return list of QuerySet objects for class 'content_type'.
 
         Be mindful when using this method directly as it does not apply the
@@ -1008,7 +1008,7 @@ class QueryCondition(models.Model):
     }
 
     # Allowed fields for condition entities.
-    FIELD_CHOICES = {
+    FIELD_CHOICES: dict[type[models.Model], list[str]] = {
         TestJob: [
             "submitter",
             "start_time",
@@ -1075,7 +1075,7 @@ class QueryCondition(models.Model):
         # If job is supplied, return available metadata field names as well.
 
         condition_choices = {}
-        for model in cls.FIELD_CHOICES:
+        for model, field_choices in cls.FIELD_CHOICES.items():
             condition_choice = {}
             condition_choice["fields"] = {}
             content_type = ContentType.objects.get_for_model(model)
@@ -1089,7 +1089,7 @@ class QueryCondition(models.Model):
                         condition_choice["fields"][attribute.name] = {}
 
             else:
-                for field_name in cls.FIELD_CHOICES[model]:
+                for field_name in field_choices:
                     field = {}
 
                     field_object = content_type.model_class()._meta.get_field(
@@ -1471,7 +1471,7 @@ class ChartQuery(models.Model):
                         "pass": measurement_results[result]["fail"] == 0,
                         "measurement": measurement_results[result]["measurement"],
                     }
-                data.append(chart_item)
+                    data.append(chart_item)
 
         return data
 
@@ -1490,12 +1490,12 @@ class ChartQuery(models.Model):
                         "pass": attribute_results[result]["fail"] == 0,
                         "attr_value": attribute_results[result]["value"],
                     }
-                data.append(chart_item)
+                    data.append(chart_item)
 
         return data
 
     def __str__(self):
-        return self.id
+        return str(self.id)
 
     def get_absolute_url(self):
         return reverse("lava.results.chart_query_edit", args=[self.chart.name, self.id])
