@@ -23,7 +23,7 @@ class UserChartTable(LavaTable):
 
     def render_description(self, value):
         value = " ".join(value.split(" ")[:15])
-        return value.split("\n")[0]
+        return value.split("\n", maxsplit=1)[0]
 
     owner = tables.TemplateColumn(
         """
@@ -73,7 +73,7 @@ class OtherChartTable(UserChartTable):
 
     def render_description(self, value):
         value = " ".join(value.split(" ")[:15])
-        return value.split("\n")[0]
+        return value.split("\n", maxsplit=1)[0]
 
     class Meta(UserChartTable.Meta):
         fields = ("name", "description", "owner")
@@ -84,7 +84,7 @@ class OtherChartTable(UserChartTable):
 class GroupChartTable(UserChartTable):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.base_columns["chart_group"].visible = False
+        self.columns["chart_group"].column.visible = False
 
     name = tables.TemplateColumn(
         """
@@ -96,7 +96,7 @@ class GroupChartTable(UserChartTable):
 
     def render_description(self, value):
         value = " ".join(value.split(" ")[:15])
-        return value.split("\n")[0]
+        return value.split("\n", maxsplit=1)[0]
 
     class Meta(UserChartTable.Meta):
         fields = ("name", "description", "owner")
