@@ -552,13 +552,9 @@ def query_include_result(request, username, name, id):
     query = get_object_or_404(Query, owner__username=username, name=name)
     result_object = get_object_or_404(query.content_type.model_class(), id=id)
 
-    try:
-        QueryOmitResult.objects.get(
-            query=query, object_id=result_object.id, content_type=query.content_type
-        ).delete()
-    except QueryOmitResult.DoesNotExist:
-        # Ignore does not exist violation.
-        raise
+    QueryOmitResult.objects.get(
+        query=query, object_id=result_object.id, content_type=query.content_type
+    ).delete()
 
     return HttpResponseRedirect(query.get_absolute_url())
 
@@ -635,9 +631,9 @@ def query_condition_form(request, query, bread_crumb_trail, instance=None):
     if form.is_valid():
         query_condition = form.save()
 
-        return HttpResponse(
-            serializers.serialize("json", [query_condition, query_condition.table]),
-            content_type="application/json",
+        return JsonResponse(
+            serializers.serialize("python", [query_condition, query_condition.table]),
+            safe=False,
         )
     else:
         return JsonResponse(["fail", form.errors], safe=False)
