@@ -1,22 +1,9 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2018 Linaro Limited
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #         Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License version 3
-# as published by the Free Software Foundation
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public License
-# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import warnings
 
@@ -68,9 +55,7 @@ class GroupObjectPermissionInline(admin.TabularInline):
                 content_type__model=self.parent_model._meta.object_name.lower(),
                 codename__in=supported_codenames,
             )
-        return super(GroupObjectPermissionInline, self).formfield_for_foreignkey(
-            db_field, request, **kwargs
-        )
+        return super().formfield_for_foreignkey(db_field, request, **kwargs)
 
 
 class GroupDeviceTypePermissionInline(GroupObjectPermissionInline):
@@ -142,14 +127,14 @@ class CustomGroupAdminForm(forms.ModelForm):
     )
 
     def __init__(self, *args, **kwargs):
-        super(CustomGroupAdminForm, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if self.instance.pk:
             initial_users = self.instance.user_set.values_list("pk", flat=True)
             self.initial["users"] = initial_users
 
     def save(self, *args, **kwargs):
         kwargs["commit"] = True
-        return super(CustomGroupAdminForm, self).save(*args, **kwargs)
+        return super().save(*args, **kwargs)
 
     def save_m2m(self):
         self.instance.user_set.clear()
@@ -166,7 +151,6 @@ class RemoteArtifactsAuthInline(admin.TabularInline):
 
 
 class CustomUserAdmin(UserAdmin):
-
     actions = [expire_user_action]
 
     def has_delete_permission(self, request, obj=None):
@@ -195,11 +179,11 @@ def cancel_action(modeladmin, request, queryset):
             if testjob.can_cancel(request.user):
                 if testjob.is_multinode:
                     for job in testjob.sub_jobs_list:
-                        job.go_state_canceling()
-                        job.save()
+                        fields = job.go_state_canceling()
+                        job.save(update_fields=fields)
                 else:
-                    testjob.go_state_canceling()
-                    testjob.save()
+                    fields = testjob.go_state_canceling()
+                    testjob.save(update_fields=fields)
 
 
 cancel_action.short_description = "cancel selected jobs"
@@ -216,8 +200,8 @@ def fail_action(modeladmin, request, queryset):
                     DeprecationWarning,
                 )
                 testjob = TestJob.objects.select_for_update().get(pk=testjob.pk)
-                testjob.go_state_finished(TestJob.HEALTH_INCOMPLETE)
-                testjob.save()
+                fields = testjob.go_state_finished(TestJob.HEALTH_INCOMPLETE)
+                testjob.save(update_fields=fields)
 
 
 fail_action.short_description = "fail selected jobs"
@@ -305,7 +289,7 @@ def _update_devices_health(request, queryset, health):
 
             old_health_display = device.get_health_display()
             device.health = health
-            device.save()
+            device.save(update_fields=["health"])
             device.log_admin_entry(
                 request.user,
                 "%s → %s" % (old_health_display, device.get_health_display()),
@@ -349,7 +333,7 @@ class DeviceAdmin(admin.ModelAdmin):
 
     def get_queryset(self, request):
         return (
-            super(DeviceAdmin, self)
+            super()
             .get_queryset(request)
             .select_related("worker_host", "device_type")
             .prefetch_related(
@@ -445,7 +429,7 @@ class DeviceAdmin(admin.ModelAdmin):
 class TestJobAdmin(admin.ModelAdmin):
     def get_queryset(self, request):
         return (
-            super(TestJobAdmin, self)
+            super()
             .get_queryset(request)
             .select_related("requested_device_type", "actual_device", "submitter")
         )
@@ -461,7 +445,7 @@ class TestJobAdmin(admin.ModelAdmin):
 
     requested_device_type_name.short_description = "Request device type"
     actions = [cancel_action, fail_action]
-    list_filter = ("state", RequestedDeviceTypeFilter, ActualDeviceFilter)
+    list_filter = ("state", RequestedDeviceTypeFilter, ActualDeviceFilter, "submitter")
     fieldsets = (
         ("Owner", {"fields": ("submitter", "viewing_groups", "is_public")}),
         ("Request", {"fields": ("requested_device_type", "priority", "health_check")}),
@@ -486,6 +470,7 @@ class TestJobAdmin(admin.ModelAdmin):
         "end_time",
     )
     ordering = ["-submit_time"]
+    show_full_result_count = False
 
 
 def disable_health_check_action(modeladmin, request, queryset):
@@ -498,7 +483,7 @@ disable_health_check_action.short_description = "disable health checks"
 class DeviceTypeAdmin(admin.ModelAdmin):
     def get_queryset(self, request):
         return (
-            super(DeviceTypeAdmin, self)
+            super()
             .get_queryset(request)
             .select_related("architecture", "bits", "processor")
         )
@@ -603,22 +588,22 @@ class DeviceTypeAdmin(admin.ModelAdmin):
 def worker_health_active(ModelAdmin, request, queryset):
     with transaction.atomic():
         for worker in queryset.select_for_update():
-            worker.go_health_active(request.user)
-            worker.save()
+            fields = worker.go_health_active(request.user)
+            worker.save(update_fields=fields)
 
 
 def worker_health_maintenance(ModelAdmin, request, queryset):
     with transaction.atomic():
         for worker in queryset.select_for_update():
-            worker.go_health_maintenance(request.user)
-            worker.save()
+            fields = worker.go_health_maintenance(request.user)
+            worker.save(update_fields=fields)
 
 
 def worker_health_retired(ModelAdmin, request, queryset):
     with transaction.atomic():
         for worker in queryset.select_for_update():
-            worker.go_health_retired(request.user)
-            worker.save()
+            fields = worker.go_health_retired(request.user)
+            worker.save(update_fields=fields)
 
 
 worker_health_active.short_description = "Update health of selected workers to Active"
