@@ -11,6 +11,7 @@ from django.contrib.auth.models import Group, User
 from django.core.exceptions import ValidationError
 from django.db import IntegrityError, transaction
 from django.db.models import Prefetch, Q
+from django.utils.html import escape
 
 from lava_common.yaml import yaml_safe_load
 from lava_scheduler_app.api import check_perm
@@ -361,6 +362,7 @@ class SchedulerDevicesAPI(ExposedV2API):
         health=None,
         description=None,
         device_type=None,
+        reason=None,
     ):
         """
         Name
@@ -368,7 +370,8 @@ class SchedulerDevicesAPI(ExposedV2API):
         `scheduler.devices.update` (`hostname`, `worker_hostname=None`,
                                     `user_name=None`, `group_name=None`,
                                     `public=True`,  `health=None`,
-                                    `description=None`, `device_type=None`)
+                                    `description=None`, `device_type=None`
+                                    `reason=None`)
 
         Description
         -----------
@@ -394,6 +397,8 @@ class SchedulerDevicesAPI(ExposedV2API):
           Device description
         `device_type`: string
           Device type
+        `reason`: string
+          Reason of health change
 
         Return value
         ------------
@@ -437,8 +442,12 @@ class SchedulerDevicesAPI(ExposedV2API):
                         fields.append("health")
                         device.log_admin_entry(
                             self.user,
-                            "%s → %s (xmlrpc api)"
-                            % (prev_health, device.get_health_display()),
+                            "%s → %s (%s)"
+                            % (
+                                prev_health,
+                                device.get_health_display(),
+                                escape(reason) or "xmlrpc api",
+                            ),
                         )
                 except KeyError:
                     raise xmlrpc.client.Fault(400, "Health '%s' is invalid" % health)
