@@ -173,7 +173,7 @@ class SchedulerWorkersAPI(ExposedV2API):
 
         Return value
         ------------
-        This function returns the worker environment
+        This function returns the worker DUT environment
         """
         # Sanitize hostname as we will use it in a path
         if len(pathlib.Path(hostname).parts) != 1:
@@ -466,7 +466,7 @@ class SchedulerWorkersAPI(ExposedV2API):
 
             if job_limit is not None:
                 if not isinstance(job_limit, int) or job_limit < 0:
-                    raise xmlrpc.client.fault(400, "Invalid job limit")
+                    raise xmlrpc.client.Fault(400, "Invalid job limit")
                 worker.job_limit = job_limit
                 fields.append("job_limit")
 
