@@ -136,6 +136,9 @@ class Architecture(models.Model):
 
 
 class ProcessorFamily(models.Model):
+    class Meta:
+        verbose_name_plural = "Processor families"
+
     name = models.CharField(
         primary_key=True,
         verbose_name="Processor Family",
@@ -1056,10 +1059,10 @@ def _check_tags(taglist, device_type=None, hostname=None):
         return []
     q = models.Q()
     if device_type:
-        q = q.__and__(models.Q(device_type=device_type))
+        q &= models.Q(device_type=device_type)
     if hostname:
-        q = q.__and__(models.Q(hostname=hostname))
-    q = q.__and__(~models.Q(health=Device.HEALTH_RETIRED))
+        q &= models.Q(hostname=hostname)
+    q &= ~models.Q(health=Device.HEALTH_RETIRED)
     tag_devices = set(Device.objects.filter(q))
     matched_devices = []
     for device in tag_devices:
@@ -1211,7 +1214,11 @@ def _create_pipeline_job(
                 )
 
     # handle queue timeout.
-    queue_timeout = None
+    queue_timeout = (
+        round(settings.QUEUE_TIMEOUT_HOURS * 3600)
+        if settings.QUEUE_TIMEOUT_HOURS
+        else None
+    )
     if "timeouts" in job_data and "queue" in job_data["timeouts"]:
         queue_timeout = Timeout.parse(job_data["timeouts"]["queue"])
 
@@ -1399,6 +1406,13 @@ class TestJob(models.Model):
                 fields=("requested_device_type", "-submit_time", "id", "health"),
                 condition=Q(health_check=True),
             ),
+            models.Index(
+                name="device_finished_jobs_index",
+                fields=("actual_device", "-end_time"),
+                condition=Q(
+                    health_check=False, state=5
+                ),  # HACK: refers to TestJob.STATE_FINISHED
+            ),
         )
         constraints = (
             models.UniqueConstraint(
@@ -1863,6 +1877,13 @@ class TestJob(models.Model):
     def get_absolute_url(self):
         return reverse("lava.scheduler.job.detail", args=[self.display_id])
 
+    def get_definition_url(self):
+        if self.is_multinode:
+            return reverse(
+                "lava.scheduler.job.multinode_definition", args=[self.display_id]
+            )
+        return reverse("lava.scheduler.job.definition", args=[self.display_id])
+
     @classmethod
     def from_yaml_and_user(cls, yaml_data, user, original_job=None):
         """
@@ -2021,12 +2042,14 @@ class TestJob(models.Model):
             "start_time": str(self.start_time),
             "end_time": str(self.end_time),
             "submitter_username": self.submitter.username,
+            "submitter_email": self.submitter.email,
             "failure_comment": self.failure_comment,
             "priority": self.priority,
             "description": self.description,
             "actual_device_id": self.actual_device_id,
             "definition": self.definition,
             "metadata": self.get_metadata_dict(),
+            "health_check": self.health_check,
         }
 
         # Logs.
@@ -2135,6 +2158,14 @@ class TestJob(models.Model):
         if "." in str(job_id):
             job = query.get(sub_id=job_id)
         else:
+            # Validate 'job_id' earlier. Raise 'TestJob.DoesNotExist' for
+            # invalid IDs. This exception is expected by job query APIs.
+            try:
+                int(job_id)
+            except (TypeError, ValueError):
+                raise TestJob.DoesNotExist(
+                    f"'job_id' expected a number but got {job_id}"
+                )
             job = query.get(pk=job_id)
         return job
 
@@ -2370,6 +2401,8 @@ class NotificationRecipient(models.Model):
     IRC = 1
     IRC_STR = "irc"
 
+    LAVA_DEVICE_OWNER_STR = "{LAVA_DEVICE_OWNER}"
+
     METHOD_CHOICES = ((EMAIL, EMAIL_STR), (IRC, IRC_STR))
     METHOD_MAP = {EMAIL_STR: EMAIL, IRC_STR: IRC}
 
