@@ -162,6 +162,7 @@ def _monitor_def_schema():
                 Required("end"): str,
                 Required("pattern"): str,
                 Optional("fixupdict"): dict,
+                Optional("expected"): [str],
             }
         ]
     )
@@ -184,8 +185,9 @@ def _interactive_def_schema():
                 Required("name"): Match(
                     r"^[a-zA-Z0-9-_]+$", msg=INVALID_CHARACTER_ERROR_MSG
                 ),
-                Required("prompts"): list,
+                Required("prompts"): [All(str, Length(min=1))],
                 Optional("echo"): "discard",
+                Optional("expected"): [All(str, Length(min=1))],
                 Required("script"): _interactive_script_schema(),
             }
         ]
@@ -490,6 +492,7 @@ _device_schema = Schema(
         "character_delays": dict,
         "commands": _device_commands_schema(),
         "constants": dict,
+        "fastboot_auto_detection": bool,
         "adb_serial_number": str,
         "fastboot_serial_number": str,
         "fastboot_options": [str],
