@@ -4,6 +4,7 @@
 #         Senthil Kumaran <senthil.kumaran@linaro.org>
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import contextlib
 import copy
@@ -11,6 +12,7 @@ import errno
 import ipaddress
 import os
 import subprocess  # nosec verified
+from typing import TYPE_CHECKING
 
 from django.conf import settings
 from django.contrib.auth.models import User
@@ -25,6 +27,10 @@ SERVICE_UNKNOWN_ERROR = "service not known"
 NO_SUCH_NICK_ERROR = "No such nick/channel"
 
 
+if TYPE_CHECKING:
+    from typing import Any
+
+
 class IRCSendError(Exception):
     """Global IRC error."""
 
@@ -87,37 +93,33 @@ def split_multinode_yaml(submission, target_group):
     ]  # top level values to be preserved
     maps = ["count"]  # elements to be matched but not listed at top level.
 
-    roles = {}
-    actions = {}
+    roles: dict[str, dict[str, Any]] = {}
+    actions: dict[str, dict[str, Any]] = {}
     subid = 0
 
     # FIXME: check structure using a schema
 
     role_data = submission["protocols"]["lava-multinode"]["roles"]
     group_size = sum(
-        [
-            role_data[count]["count"]
-            for count in role_data
-            if "count" in role_data[count]
-        ]
+        role_data[count]["count"] for count in role_data if "count" in role_data[count]
     )
 
     # populate the lava-multinode protocol metadata
-    for role, value in submission["protocols"]["lava-multinode"]["roles"].items():
-        roles[role] = {}
+    for role_name, value in submission["protocols"]["lava-multinode"]["roles"].items():
+        roles[role_name] = {}
         for item in copies:
             if item in submission:
-                roles[role][item] = submission[item]
+                roles[role_name][item] = submission[item]
         for name in maps:
             if name in value:
-                roles[role][name] = value[name]
+                roles[role_name][name] = value[name]
         for name in scheduling:
             if name in value:
-                roles[role][name] = value[name]
+                roles[role_name][name] = value[name]
         tags = set(value) - set(maps) - set(scheduling)
         params = {
             "target_group": target_group,
-            "role": role,
+            "role": role_name,
             "group_size": group_size,
             "sub_id": subid,
         }
@@ -125,11 +127,11 @@ def split_multinode_yaml(submission, target_group):
             params["essential"] = value
         for tag in tags:
             params[tag] = value[tag]
-        roles[role].update({"protocols": {"lava-multinode": params}})
+        roles[role_name].update({"protocols": {"lava-multinode": params}})
         subid += 1
 
     # split the submission based on the roles specified for the actions, retaining order.
-    for role in roles:
+    for role_name in roles:
         for action in submission["actions"]:
             for key, value in action.items():
                 try:
@@ -139,9 +141,9 @@ def split_multinode_yaml(submission, target_group):
                         "Invalid YAML - Did not find a role in action '%s', check for consistent use of whitespace indents."
                         % next(iter(action.keys()))
                     )
-                if role in value["role"]:
-                    actions.setdefault(role, {"actions": []})
-                    actions[role]["actions"].append(
+                if role_name in value["role"]:
+                    actions.setdefault(role_name, {"actions": []})
+                    actions[role_name]["actions"].append(
                         {copy.deepcopy(key): copy.deepcopy(value)}
                     )
 
@@ -149,35 +151,35 @@ def split_multinode_yaml(submission, target_group):
     for key, value in submission["protocols"]["lava-multinode"].items():
         if key in skip:
             continue
-        for role in roles:
-            roles[role]["protocols"]["lava-multinode"][key] = value
+        for role_dict in roles.values():
+            role_dict["protocols"]["lava-multinode"][key] = value
 
     # set the role for each action to the role of the job instead of the original list..
-    for role in actions:
-        for action in actions[role]["actions"]:
+    for role_name, action_dict in actions.items():
+        for action in action_dict["actions"]:
             for key, value in action.items():
-                value["role"] = role
+                value["role"] = role_name
 
     # jobs dictionary lists the jobs per role,
     jobs = {}
     # check the count of the host_roles
     check_count = None
-    for role in roles:
-        if "host_role" in roles[role]:
-            check_count = roles[role]["host_role"]
-    for role in roles:
-        if role == check_count:
-            if roles[role]["count"] != 1:
+    for role_dict in roles.values():
+        if "host_role" in role_dict:
+            check_count = role_dict["host_role"]
+    for role_name, role_dict in roles.items():
+        if role_name == check_count:
+            if role_dict["count"] != 1:
                 raise SubmissionException(
                     "The count for a role designated as a host_role must be 1."
                 )
     sub_id_count = 0
-    for role in roles:
-        jobs[role] = []
-        for sub in range(0, roles[role]["count"]):
+    for role_name, role_dict in roles.items():
+        jobs[role_name] = []
+        for sub in range(0, role_dict["count"]):
             job = {}
-            job.update(actions[role])
-            job.update(roles[role])
+            job.update(actions[role_name])
+            job.update(role_dict)
             # only here do multiple jobs for the same role differ
             params = job["protocols"]["lava-multinode"]
             params.update({"sub_id": sub_id_count})
@@ -186,7 +188,7 @@ def split_multinode_yaml(submission, target_group):
             for item in maps:
                 if item in job:
                     del job[item]
-            jobs[role].append(copy.deepcopy(job))
+            jobs[role_name].append(copy.deepcopy(job))
             sub_id_count += 1
 
     # populate the lava-vland protocol metadata
@@ -195,13 +197,13 @@ def split_multinode_yaml(submission, target_group):
 
     # populate the lava-lxc protocol data
     if "lava-lxc" in submission["protocols"]:
-        for role, _ in jobs.items():
-            if role not in submission["protocols"]["lava-lxc"]:
+        for role_name, role_dicts in jobs.items():
+            if role_name not in submission["protocols"]["lava-lxc"]:
                 continue
             # populate the lava-lxc protocol metadata
-            for job in jobs[role]:
+            for job in role_dicts:
                 job["protocols"].update(
-                    {"lava-lxc": submission["protocols"]["lava-lxc"][role]}
+                    {"lava-lxc": submission["protocols"]["lava-lxc"][role_name]}
                 )
 
     return jobs
@@ -284,6 +286,7 @@ def get_ldap_user_properties(ldap_user):
     attributes = ["uid", "givenName", "sn", "mail"]
     search_filter = "cn=*"
 
+    user_dn: str | None = None
     if user_dn_template:
         user_dn = user_dn_template % {"user": ldap_user}
     if user_search is not None:
@@ -297,39 +300,30 @@ def get_ldap_user_properties(ldap_user):
         conn = ldap.initialize(server_uri)
         if user_dn:
             conn.simple_bind_s(bind_dn, bind_password)
-            try:
-                result = conn.search_s(user_dn, search_scope, search_filter, attributes)
-                if len(result) == 1:
-                    result_type, result_data = result[0]
-                    user_properties["uid"] = result_data.get("uid", [None])[0]
-                    user_properties["mail"] = result_data.get("mail", [None])[0]
-                    user_properties["sn"] = result_data.get("sn", [None])[0]
-                    user_properties["given_name"] = result_data.get(
-                        "givenName", [None]
-                    )[0]
+            result = conn.search_s(user_dn, search_scope, search_filter, attributes)
+            if len(result) == 1:
+                result_type, result_data = result[0]
+                user_properties["uid"] = result_data.get("uid", [None])[0]
+                user_properties["mail"] = result_data.get("mail", [None])[0]
+                user_properties["sn"] = result_data.get("sn", [None])[0]
+                user_properties["given_name"] = result_data.get("givenName", [None])[0]
 
-                    user_properties = {
-                        k: v.decode() for (k, v) in user_properties.items()
-                    }
+                user_properties = {k: v.decode() for (k, v) in user_properties.items()}
 
-                    # Grab max_length and truncate first and last name.
-                    # For some users, first or last name is too long to create.
-                    first_name_max_length = User._meta.get_field(
-                        "first_name"
-                    ).max_length
-                    last_name_max_length = User._meta.get_field("last_name").max_length
-                    user_properties["sn"] = truncatechars(
-                        user_properties["sn"], last_name_max_length
-                    )
-                    user_properties["given_name"] = truncatechars(
-                        user_properties["given_name"], first_name_max_length
-                    )
+                # Grab max_length and truncate first and last name.
+                # For some users, first or last name is too long to create.
+                first_name_max_length = User._meta.get_field("first_name").max_length
+                last_name_max_length = User._meta.get_field("last_name").max_length
+                user_properties["sn"] = truncatechars(
+                    user_properties["sn"], last_name_max_length
+                )
+                user_properties["given_name"] = truncatechars(
+                    user_properties["given_name"], first_name_max_length
+                )
 
-                    return user_properties
-                elif len(result) == 0:
-                    raise ldap.NO_SUCH_OBJECT
-            except ldap.NO_SUCH_OBJECT:
-                raise
+                return user_properties
+            elif len(result) == 0:
+                raise ldap.NO_SUCH_OBJECT
     else:
         raise ldap.UNAVAILABLE
 
