@@ -5,6 +5,7 @@
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
+from __future__ import annotations
 
 import contextlib
 import datetime
@@ -15,6 +16,7 @@ import re
 import tarfile
 from json import dumps as json_dumps
 from pathlib import Path
+from typing import TYPE_CHECKING
 
 import voluptuous
 import yaml
@@ -22,6 +24,7 @@ from django import forms
 from django.conf import settings
 from django.contrib.admin.models import LogEntry
 from django.contrib.auth import authenticate
+from django.contrib.auth.decorators import login_required
 from django.contrib.auth.models import User
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.humanize.templatetags.humanize import naturaltime
@@ -34,6 +37,8 @@ from django.db import transaction
 from django.db.models import (
     Count,
     Exists,
+    ExpressionWrapper,
+    F,
     IntegerField,
     OuterRef,
     Prefetch,
@@ -56,7 +61,8 @@ from django.template.loader import render_to_string
 from django.urls import reverse
 from django.utils import timezone
 from django.utils.crypto import constant_time_compare
-from django.utils.html import escape
+from django.utils.html import escape, json_script
+from django.utils.safestring import mark_safe
 from django.utils.timesince import timeuntil
 from django.views.decorators.csrf import csrf_exempt
 from django.views.decorators.http import require_http_methods, require_POST
@@ -75,7 +81,7 @@ from lava_results_app.models import (
     TestCase,
     TestData,
 )
-from lava_results_app.utils import description_data, description_filename
+from lava_results_app.utils import description_filename
 from lava_scheduler_app.dbutils import (
     device_summary,
     device_type_summary,
@@ -127,6 +133,9 @@ from lava_server.files import File
 from lava_server.lavatable import LavaView
 from lava_server.views import index as lava_index
 
+if TYPE_CHECKING:
+    from typing import Any
+
 
 def request_config(request, paginate):
     return RequestConfig(request, paginate={**paginate, **djt2_paginator_class()})
@@ -315,11 +324,22 @@ class ActiveJobsTableView(JobTableView):
 
 class DeviceTableView(JobTableView):
     def get_queryset(self):
+        last_job_end_time_subquery = Subquery(
+            TestJob.objects.filter(
+                actual_device__hostname=OuterRef("hostname"),
+                health_check__exact=False,
+                state__exact=TestJob.STATE_FINISHED,
+            )
+            .order_by("-end_time")
+            .values("end_time")[:1]
+        )
+
         q = (
             Device.objects.select_related("device_type", "worker_host")
             .prefetch_related("tags")
             .visible_by_user(self.request.user)
             .order_by("hostname")
+            .annotate(last_job_end_time=last_job_end_time_subquery)
             .distinct()
         )
         return q.prefetch_related(
@@ -335,30 +355,22 @@ class DeviceTableView(JobTableView):
 
 class JobErrorsView(LavaView):
     def get_job_errors_queryset(self):
-        metadata_subquery = Subquery(
-            TestCase.objects.filter(
-                Q(metadata__contains="error_type: Configuration")
-                | Q(metadata__contains="error_type: Infrastructure")
-                | Q(metadata__contains="error_type: Bug"),
-                suite=OuterRef("testsuite"),
-                result=TestCase.RESULT_FAIL,
-                name="job",
-            ).values("metadata")[:1],
-            # HACK: Add LIMIT to fix edge case
-            # when job has multiple failure testcases
-            output_field=YamlField(),
-        )
-
         return (
             TestJob.objects.filter(
-                health__in=(TestJob.HEALTH_INCOMPLETE, TestJob.HEALTH_CANCELED),
                 testsuite__name="lava",
+                testsuite__testcase__name="job",
+                testsuite__testcase__result=TestCase.RESULT_FAIL,
+                testsuite__testcase__metadata__regex=(
+                    "error_type: (Configuration|Infrastructure|Bug)"
+                ),
             )
             .annotate(
-                failure_metadata=metadata_subquery,
+                failure_metadata=ExpressionWrapper(
+                    F("testsuite__testcase__metadata"),
+                    output_field=YamlField(),
+                ),
             )
-            .filter(failure_metadata__isnull=False)
-            .order_by("-end_time")
+            .order_by("-testsuite__id")
         )
 
     def get_queryset(self):
@@ -494,6 +506,7 @@ def job_report_data(start_day, end_day):
 
 
 @BreadCrumb("Reports", parent=index)
+@login_required
 def reports(request):
     health_day_report = []
     health_week_report = []
@@ -523,6 +536,7 @@ def reports(request):
 
 
 @BreadCrumb("Failure Report", parent=reports)
+@login_required
 def failure_report(request):
     data = FailedJobsTableView(request)
     ptable = FailedJobsTable(data.get_table_data())
@@ -626,6 +640,16 @@ def passing_health_checks(request):
 
 class MyDeviceView(DeviceTableView):
     def get_queryset(self):
+        last_job_end_time_subquery = Subquery(
+            TestJob.objects.filter(
+                actual_device__hostname=OuterRef("hostname"),
+                health_check__exact=False,
+                state__exact=TestJob.STATE_FINISHED,
+            )
+            .order_by("-end_time")
+            .values("end_time")[:1]
+        )
+
         return (
             Device.objects.accessible_by_user(
                 self.request.user, Device.CHANGE_PERMISSION
@@ -633,6 +657,7 @@ class MyDeviceView(DeviceTableView):
             .select_related("device_type", "worker_host")
             .prefetch_related("tags")
             .order_by("hostname")
+            .annotate(last_job_end_time=last_job_end_time_subquery)
         )
 
 
@@ -722,30 +747,15 @@ class DeviceTypeOverView(JobTableView):
                 .annotate(queued_jobs=Count("*"))
                 .values("queued_jobs"),
                 output_field=IntegerField(),
-            ),
-        )
-
-
-class NoDTDeviceView(DeviceTableView):
-    def get_queryset(self):
-        q = (
-            Device.objects.exclude(health=Device.HEALTH_RETIRED)
-            .visible_by_user(self.request.user)
-            .select_related("device_type", "worker_host")
-            .prefetch_related("tags")
-            .order_by("hostname")
-        )
-        return q.prefetch_related(
-            Prefetch(
-                "testjobs",
-                queryset=TestJob.objects.filter(
-                    ~Q(state=TestJob.STATE_FINISHED)
-                ).select_related("submitter"),
-                to_attr="running_jobs",
             )
         )
 
 
+class DTDeviceView(DeviceTableView):
+    def get_queryset(self):
+        return super().get_queryset().exclude(health=Device.HEALTH_RETIRED)
+
+
 @BreadCrumb("Maintenance", parent=device_list)
 def maintenance_devices(request):
     data = MaintenanceDeviceView(request, model=Device, table_class=DeviceTable)
@@ -906,24 +916,24 @@ def device_type_detail(request, pk):
         {"Duration": "Month", "Complete": monthly_complete, "Failed": monthly_failed},
     ]
 
-    prefix = "no_dt_"
-    no_dt_data = NoDTDeviceView(request, model=Device, table_class=DeviceTable)
-    no_dt_ptable = DeviceTable(
-        no_dt_data.get_table_data(prefix).filter(device_type=dt),
+    prefix = "devices_"
+    devices_data = DTDeviceView(request, model=Device, table_class=DeviceTable)
+    devices_ptable = DeviceTable(
+        devices_data.get_table_data(prefix).filter(device_type=dt),
         request=request,
         prefix=prefix,
     )
-    config = RequestConfig(request, paginate={"per_page": no_dt_ptable.length})
-    config.configure(no_dt_ptable)
+    config = RequestConfig(request, paginate={"per_page": devices_ptable.length})
+    config.configure(devices_ptable)
 
-    prefix = "dt_"
-    dt_jobs_data = AllJobsView(request, model=TestJob, table_class=DeviceTypeJobsTable)
-    dt_jobs_ptable = DeviceTypeJobsTable(
-        dt_jobs_data.get_table_data(prefix).filter(requested_device_type=pk),
+    prefix = "jobs_"
+    jobs_data = AllJobsView(request, model=TestJob, table_class=DeviceTypeJobsTable)
+    jobs_ptable = DeviceTypeJobsTable(
+        jobs_data.get_table_data(prefix).filter(requested_device_type=pk),
         prefix=prefix,
     )
-    config = request_config(request, {"per_page": dt_jobs_ptable.length})
-    config.configure(dt_jobs_ptable)
+    config = request_config(request, {"per_page": jobs_ptable.length})
+    config.configure(jobs_ptable)
 
     prefix = "health_"
     health_table = HealthJobSummaryTable(
@@ -956,10 +966,7 @@ def device_type_detail(request, pk):
             "pk",
             filter=Q(state__in=(Device.STATE_RUNNING, Device.STATE_RESERVED)),
         ),
-        retired_devices_count=Count(
-            "pk",
-            filter=Q(health=Device.HEALTH_RETIRED),
-        ),
+        retired_devices_count=Count("pk", filter=Q(health=Device.HEALTH_RETIRED)),
     )
 
     if device_statistics["available_devices_count"]:
@@ -994,8 +1001,8 @@ def device_type_detail(request, pk):
             "available_devices_label": available_devices_label,
             "queued_jobs_count": queued_jobs_count or 0,
             "health_job_summary_table": health_table,
-            "device_type_jobs_table": dt_jobs_ptable,
-            "devices_table_no_dt": no_dt_ptable,
+            "device_type_jobs_table": jobs_ptable,
+            "devices_table": devices_ptable,
             "bread_crumb_trail": BreadCrumbTrail.leading_to(device_type_detail, pk=pk),
             "context_help": BreadCrumbTrail.leading_to(device_type_detail, pk="help"),
             "health_freq": health_freq_str,
@@ -1076,6 +1083,8 @@ def device_type_reports(request, pk):
 
 def device_dictionary_plain(request, pk):
     device = get_object_or_404(Device, pk=pk)
+    if not device.can_view(request.user):
+        raise PermissionDenied()
     device_configuration = device.load_configuration(output_format="yaml")
     response = HttpResponse(device_configuration, content_type="text/yaml")
     response["Content-Disposition"] = "attachment; filename=%s.yaml" % pk
@@ -1101,6 +1110,8 @@ def lab_health(request):
 @BreadCrumb("All Health Jobs on Device {pk}", parent=index, needs=["pk"])
 def health_job_list(request, pk):
     device = get_object_or_404(Device, pk=pk)
+    if not device.can_view(request.user):
+        raise PermissionDenied()
 
     health_data = AllJobsView(request)
     health_table = AllJobsTable(
@@ -1123,6 +1134,32 @@ def health_job_list(request, pk):
     )
 
 
+class InPlaceTokenUpdater:
+    def __init__(self, tokens: dict[str, str]):
+        self.tokens = tokens
+
+    def update_headers(self, data: list[dict[str, Any]] | dict[str, Any]) -> None:
+        if isinstance(data, list):
+            for item in data:
+                self.update_headers(item)
+        elif isinstance(data, dict):
+            # 'UrlRepoAction' adds 'url' as an alias to 'repository' for
+            # reusing the 'HttpDownloadAction'.
+            if "url" in data or (data.get("from") == "url" and "repository" in data):
+                if headers := data.get("headers"):
+                    if isinstance(headers, dict):
+                        for key, token_value in headers.items():
+                            if token_value in self.tokens:
+                                headers[key] = self.tokens[token_value]
+            for val in data.values():
+                self.update_headers(val)
+
+    def update_secrets(self, secrets: dict[str, str]) -> None:
+        for k, v in secrets.items():
+            if v in self.tokens.keys():
+                secrets[k] = self.tokens[v]
+
+
 @require_http_methods(["GET", "POST"])
 @csrf_exempt
 def internal_v1_jobs(request, pk):
@@ -1140,36 +1177,20 @@ def internal_v1_jobs(request, pk):
     if request.method == "GET":
         job_def = yaml_safe_load(job.definition)
 
-        tokens = {
+        if tokens := {
             x["name"]: x["token"]
             for x in RemoteArtifactsAuth.objects.filter(user=job.submitter).values(
                 "name", "token"
             )
-        }
+        }:
+            updater = InPlaceTokenUpdater(tokens)
+            if actions := job_def.get("actions"):
+                for action in actions:
+                    if "deploy" in action or "test" in action:
+                        updater.update_headers(action)
 
-        def update_token(headers_dict):
-            for key in headers_dict["headers"]:
-                token_name = headers_dict["headers"][key]
-                if token_name in tokens.keys():
-                    headers_dict["headers"][key] = tokens[token_name]
-
-        if "actions" in job_def:
-            for action in job_def["actions"]:
-                for k, v in action.items():
-                    if k == "deploy":
-                        for a, b in v.items():
-                            if isinstance(b, dict):
-                                if "url" in b and "headers" in b:
-                                    update_token(b)
-                                for i, j in b.items():
-                                    if isinstance(j, dict):
-                                        if "url" in j and "headers" in j:
-                                            update_token(j)
-
-        if "secrets" in job_def:
-            for k, v in job_def["secrets"].items():
-                if v in tokens.keys():
-                    job_def["secrets"][k] = tokens[v]
+            if secrets := job_def.get("secrets"):
+                updater.update_secrets(secrets)
 
         job_ctx = job_def.get("context", {})
 
@@ -1316,7 +1337,7 @@ def internal_v1_jobs_logs(request, pk):
     #       of lines that where actually parsed !!
     test_cases = []
     line_count = 0
-    for line, string in zip(yaml_safe_load(lines), lines.split("\n")):
+    for line_dict, line_string in zip(yaml_safe_load(lines), lines.splitlines(True)):
         # skip lines that where already saved to disk
         duplicated = False
         if line_skip > 0:
@@ -1324,28 +1345,33 @@ def internal_v1_jobs_logs(request, pk):
             line_skip -= 1
         else:
             # Handle lava-event
-            if line["lvl"] == "event":
+            if line_dict["lvl"] == "event":
                 send_event(
-                    ".event", "lavaserver", {"message": line["msg"], "job": job.id}
+                    ".event", "lavaserver", {"message": line_dict["msg"], "job": job.id}
                 )
-                line["lvl"] = "debug"
-                string = "- " + dump(line)
+                line_dict["lvl"] = "debug"
+                line_string = f"- {dump(line_dict)}\n"
+
+            # Fix lines that are missing the newline.
+            # Currently the lava-run submitted logs are missing the final newline.
+            if not line_string.endswith("\n"):
+                line_string += "\n"
 
             # Save the log line
-            logs_instance.write(job, (string + "\n").encode("utf-8"), output, index)
+            logs_instance.write(job, line_string.encode("utf-8"), output, index)
 
         # handle test case results
-        if line["lvl"] == "results":
+        if line_dict["lvl"] == "results":
             starttc = endtc = None
             with contextlib.suppress(KeyError):
-                starttc = line["msg"]["starttc"]
-                del line["msg"]["starttc"]
+                starttc = line_dict["msg"]["starttc"]
+                del line_dict["msg"]["starttc"]
             with contextlib.suppress(KeyError):
-                endtc = line["msg"]["endtc"]
-                del line["msg"]["endtc"]
-            meta_filename = create_metadata_store(line["msg"], job)
+                endtc = line_dict["msg"]["endtc"]
+                del line_dict["msg"]["endtc"]
+            meta_filename = create_metadata_store(line_dict["msg"], job)
             new_test_case = map_scanned_results(
-                results=line["msg"],
+                results=line_dict["msg"],
                 job=job,
                 starttc=starttc,
                 endtc=endtc,
@@ -1412,8 +1438,9 @@ def internal_v1_workers(request, pk=None):
         fields = ["version"]
         worker.version = version
         if version_mismatch and not settings.ALLOW_VERSION_MISMATCH:
-            # If the version does not match, go offline
-            fields.extend(worker.go_state_offline())
+            # If the version does not match and worker is online, go offline
+            if worker.state == Worker.STATE_ONLINE:
+                fields.extend(worker.go_state_offline())
         else:
             # Set last_ping
             worker.last_ping = timezone.now()
@@ -1588,6 +1615,7 @@ def job_list(request):
 
 
 @BreadCrumb("Errors", parent=job_list)
+@login_required
 def job_errors(request):
     data = JobErrorsView(request, model=TestCase, table_class=JobErrorsTable)
     ptable = JobErrorsTable(data.get_table_data(), prefix="job_errors_")
@@ -1691,8 +1719,6 @@ def job_submit(request):
 def job_detail(request, pk):
     job = TestJob.get_restricted_job(pk, request.user)
 
-    pipeline = description_data(job).get("pipeline", {})
-
     # Validate the job definition
     validation_errors = ""
     try:
@@ -1720,7 +1746,6 @@ def job_detail(request, pk):
         "available_content_types": json_dumps(
             QueryCondition.get_similar_job_content_types()
         ),
-        "pipeline_data": pipeline,
         "job_tags": job.tags.all(),
         "size_limit": job.size_limit,
         "validation_errors": validation_errors,
@@ -1749,9 +1774,15 @@ def job_detail(request, pk):
         if lava_job_obj.result == TestCase.RESULT_FAIL:
             lava_job_result = lava_job_obj.action_metadata
 
+    try:
+        json_log_data = json_script(log_data if log_data else [], "logs-initial")
+    except Exception:
+        log_data = None
+        json_log_data = json_script([], "logs-initial")
+
     data.update(
         {
-            "log_data": log_data if log_data else [],
+            "log_data": json_log_data,
             "invalid_log_data": log_data is None,
             "lava_job_result": lava_job_result,
         }
@@ -1961,6 +1992,7 @@ def job_status(request, pk):
         "duration": "<i>...</i>",
         "job_state": job.get_state_display(),
         "failure_comment": job.failure_comment,
+        "job_health": job.health,
         "started": "<i>...</i>",
         "subjobs": [],
     }
@@ -2369,9 +2401,8 @@ def device_detail(request, pk):
     device_log_data = DeviceLogView(
         device, request, model=LogEntry, table_class=DeviceLogEntryTable
     )
-    device_log_ptable = DeviceLogEntryTable(
-        device_log_data.get_table_data(), prefix="device_log_"
-    )
+    device_logs = device_log_data.get_table_data()
+    device_log_ptable = DeviceLogEntryTable(device_logs, prefix="device_log_")
     request_config(request, paginate={"per_page": device_log_ptable.length}).configure(
         device_log_ptable
     )
@@ -2383,11 +2414,16 @@ def device_detail(request, pk):
         mismatch = True
 
     device_can_change = device.can_change(request.user)
+    latest_device_log = device_logs.first()
+    device_health_reason = (
+        mark_safe(latest_device_log.get_change_message()) if latest_device_log else None
+    )
     return render(
         request,
         "lava_scheduler_app/device.html",
         {
             "device": device,
+            "device_health_reason": device_health_reason,
             "recent_job_table": recent_ptable,
             "device_log_table": device_log_ptable,
             "can_change": device_can_change,
@@ -2520,6 +2556,8 @@ def device_health(request, pk):
 @BreadCrumb("{pk}", parent=workers, needs=["pk"])
 def worker_detail(request, pk):
     worker = get_object_or_404(Worker, pk=pk)
+    if not worker.can_view(request.user):
+        raise PermissionDenied()
     data = DeviceTableView(request)
     ptable = NoWorkerDeviceTable(
         data.get_table_data()
