@@ -1,26 +1,12 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2015-2018 Linaro Limited
 #
 # Author: Senthil Kumaran S <senthil.kumaran@linaro.org>
 #
-# This file is part of LAVA.
-#
-# LAVA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License version 3
-# as published by the Free Software Foundation
-#
-# LAVA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public License
-# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import ldap
 from django.contrib.auth.models import User
 from django.core.management.base import BaseCommand, CommandError
-from django.template.defaultfilters import truncatechars
 
 from lava_scheduler_app.utils import get_ldap_user_properties
 
@@ -58,16 +44,8 @@ class Command(BaseCommand):
         except User.DoesNotExist:
             user = User.objects.create(username=username)
             user.email = user_properties.get("mail", "")
-            # Grab max_length and truncate first and last name.
-            # For some users, the command fail as first or last name is too long.
-            first_name_max_length = User._meta.get_field("first_name").max_length
-            last_name_max_length = User._meta.get_field("last_name").max_length
-            user.last_name = truncatechars(
-                user_properties.get("sn", ""), last_name_max_length
-            )
-            user.first_name = truncatechars(
-                user_properties.get("given_name", ""), first_name_max_length
-            )
+            user.last_name = user_properties.get("sn", "")
+            user.first_name = user_properties.get("given_name", "")
             superuser_msg = ""
             if options["superuser"]:
                 user.is_staff = True
