@@ -6,11 +6,8 @@
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
-# pylint:disable=wrong-import-position
-
-import warnings
-
-warnings.filterwarnings("ignore", category=UserWarning, module="environ")
+# pylint: disable=wildcard-import
+# pylint: disable=unused-wildcard-import
 
 import base64
 import contextlib
@@ -37,7 +34,6 @@ from lava_server.settings.config_file import ConfigFile
 # character, it will try to use the rest of the key as a variable name, and
 # expose it on an error message.
 env = environ.Env()
-environ.Env.read_env()
 
 if os.environ.get("DATABASE_URL"):
     DATABASES = {"default": env.db()}
@@ -110,6 +106,16 @@ if "LAVA_JSON_SETTINGS" in os.environ:
         print(exc)
         raise Exception(f"Unable to load LAVA_JSON_SETTINGS") from exc
 
+if "DATABASES" in locals() and DATABASES.get("default"):
+    if db_conn_max_age_str := globals().get("DB_CONN_MAX_AGE"):
+        with contextlib.suppress(ValueError):
+            DATABASES["default"]["CONN_MAX_AGE"] = int(db_conn_max_age_str)
+    if db_conn_health_check_str := globals().get("DB_CONN_HEALTH_CHECKS"):
+        if db_conn_health_check_str in ["True", "true", "1", "yes", "on"]:
+            DATABASES["default"]["CONN_HEALTH_CHECKS"] = True
+        elif db_conn_health_check_str in ["False", "false", "0", "no", "off"]:
+            DATABASES["default"]["CONN_HEALTH_CHECKS"] = False
+
 # Update settings with custom values
 for k, v in update(globals()).items():
     globals()[k] = v
