@@ -6,13 +6,15 @@
 
 from pathlib import Path
 from unittest.mock import patch
-from urllib.parse import urlparse
+from urllib.parse import quote_plus, urlparse
 
+import pytest
 import requests
 
 from lava_common.constants import HTTP_DOWNLOAD_CHUNK_SIZE
 from lava_common.exceptions import InfrastructureError, JobError
 from lava_dispatcher.actions.deploy.download import (
+    HTTP_CODE_OK,
     CopyToLxcAction,
     DownloaderAction,
     DownloadHandler,
@@ -459,7 +461,7 @@ class TestDowload(LavaDispatcherTestCase):
                 pass
 
         class DummyResponseOK:
-            status_code = requests.codes.OK
+            status_code = HTTP_CODE_OK
             headers = {"content-length": "4212"}
 
             def close(self):
@@ -612,7 +614,7 @@ class TestDowload(LavaDispatcherTestCase):
         # Working
         class DummyResponse:
             # pylint: disable=no-self-argument
-            status_code = requests.codes.OK
+            status_code = HTTP_CODE_OK
             headers = {"content-length": "4212"}
 
             def iter_content(self_, size):
@@ -702,9 +704,7 @@ class TestDowload(LavaDispatcherTestCase):
                 },
                 "label": "dtb",
                 "size": 10,
-                "md5sum": "fc5e038d38a57032085441e7fe7010b0",
                 "sha256sum": "936a185caaa266bb9cbe981e9e05cb78cd732b0b3280eb944412bb6f8f8f07af",
-                "sha512sum": "1594244d52f2d8c12b142bb61f47bc2eaf503d6d9ca8480cae9fcf112f66e4967dc5e8fa98285e36db8af1b8ffa8b84cb15e0fbcf836c3deb803c13f37659a60",
             },
         )
         self.assertEqual(
@@ -715,9 +715,7 @@ class TestDowload(LavaDispatcherTestCase):
                         "dtb": {
                             "decompressed": False,
                             "file": "%s/dtb/dtb" % str(tmp_dir_path),
-                            "md5": "fc5e038d38a57032085441e7fe7010b0",
                             "sha256": "936a185caaa266bb9cbe981e9e05cb78cd732b0b3280eb944412bb6f8f8f07af",
-                            "sha512": "1594244d52f2d8c12b142bb61f47bc2eaf503d6d9ca8480cae9fcf112f66e4967dc5e8fa98285e36db8af1b8ffa8b84cb15e0fbcf836c3deb803c13f37659a60",
                         },
                         "file": {"dtb": "%s/dtb/dtb" % str(tmp_dir_path)},
                     }
@@ -767,9 +765,7 @@ class TestDowload(LavaDispatcherTestCase):
                 },
                 "label": "rootfs",
                 "size": 68,
-                "md5sum": "0107d527acf9b8de628b7b4d103c89d1",
                 "sha256sum": "3275a39be7b717d548b66f3c8f23d940603a63b0f13d84a596d979a7f66feb2c",
-                "sha512sum": "d0850c3e0c45bdf74995907a04f69806a070d79a4f0b2dd82d6b96adafdbfd85ce6c1daaff916ff089bdf9b04eba7805041c49afecdbeabca69fef802e60de35",
             },
         )
 
@@ -781,9 +777,7 @@ class TestDowload(LavaDispatcherTestCase):
                         "rootfs": {
                             "decompressed": True,
                             "file": "%s/rootfs/rootfs" % str(tmp_dir_path),
-                            "md5": "0107d527acf9b8de628b7b4d103c89d1",
                             "sha256": "3275a39be7b717d548b66f3c8f23d940603a63b0f13d84a596d979a7f66feb2c",
-                            "sha512": "d0850c3e0c45bdf74995907a04f69806a070d79a4f0b2dd82d6b96adafdbfd85ce6c1daaff916ff089bdf9b04eba7805041c49afecdbeabca69fef802e60de35",
                         },
                         "file": {"rootfs": "%s/rootfs/rootfs" % str(tmp_dir_path)},
                     }
@@ -794,13 +788,11 @@ class TestDowload(LavaDispatcherTestCase):
     def test_predownloaded_job_validation(self):
         factory = Factory()
         factory.validate_job_strict = True
-        job = factory.create_job(
-            "kvm01.jinja2", "sample_jobs/qemu-download-postprocess.yaml"
-        )
+        job = factory.create_job("kvm01", "sample_jobs/qemu-download-postprocess.yaml")
         job.validate()
 
-    def test_predownloaded(self):
-        params = {
+    def test_predownloaded_uniquify_false(self):
+        parameters = {
             "to": "tmpfs",
             "rootfs": {"url": "downloads://rootfs.xz"},
             "namespace": "common",
@@ -808,9 +800,14 @@ class TestDowload(LavaDispatcherTestCase):
         job = self.create_simple_job()
         destdir = job.mkdtemp("some-other-action")
         action = PreDownloadedAction(
-            job, "rootfs", urlparse("downloads://rootfs.xz"), destdir, params
+            job,
+            "rootfs",
+            urlparse("downloads://rootfs.xz"),
+            destdir,
+            uniquify=False,
+            params=parameters["rootfs"],
         )
-        action.parameters = params
+        action.parameters = parameters
 
         filename = Path(action.job.tmp_dir) / "downloads/common/rootfs.xz"
         filename.parent.mkdir(parents=True)
@@ -826,12 +823,174 @@ class TestDowload(LavaDispatcherTestCase):
         self.assertEqual(mapped_path, (destdir + "/rootfs.xz"))
         self.assertTrue(Path(mapped_path).exists())
 
+    def test_predownloaded_validate_set_kernel_type(self):
+        parameters = {
+            "to": "tftp",
+            "kernel": {"url": "downloads://zimage", "type": "zimage"},
+            "namespace": "common",
+        }
+
+        job = self.create_simple_job()
+        destdir = job.mkdtemp("some-other-action")
+        action = PreDownloadedAction(
+            job,
+            "kernel",
+            urlparse("downloads://zimage"),
+            destdir,
+            uniquify=True,
+            params=parameters["kernel"],
+        )
+        action.parameters = parameters
+        action.validate()
+
+        self.assertEqual(
+            action.get_namespace_data(
+                action="download-action", label="type", key="kernel"
+            ),
+            "zimage",
+        )
+
+    def test_predownloaded_tftp_file_path(self):
+        parameters = {
+            "to": "tftp",
+            "kernel": {"url": "downloads://zimage", "type": "zimage"},
+            "namespace": "common",
+        }
+
+        job = self.create_simple_job()
+        destdir = job.mkdtemp("some-other-action")
+
+        action = PreDownloadedAction(
+            job,
+            "kernel",
+            urlparse("downloads://zimage"),
+            destdir,
+            uniquify=True,
+            params=parameters["kernel"],
+        )
+        action.parameters = parameters
+        action.data = {}
+        action.parameters = {"namespace": "common"}
+        filename = Path(action.job.tmp_dir) / "downloads/common/zimage"
+        filename.parent.mkdir(parents=True)
+        filename.touch()
+        suffix = f"{job.job_id}/tftp-deploy-uid"
+        action.set_namespace_data(
+            action="tftp-deploy", label="tftp", key="suffix", value=suffix
+        )
+
+        action.run(None, 4242)
+
+        self.assertEqual(
+            action.get_namespace_data(
+                action="download-action", label="file", key="kernel"
+            ),
+            f"{suffix}/kernel/zimage",
+        )
+
+    def test_predownloaded_tftp_file_path_uniquify_false(self):
+        parameters = {
+            "to": "tftp",
+            "kernel": {"url": "downloads://zimage", "type": "zimage"},
+            "namespace": "common",
+        }
+
+        job = self.create_simple_job()
+        destdir = job.mkdtemp("some-other-action")
+
+        action = PreDownloadedAction(
+            job,
+            "kernel",
+            urlparse("downloads://zimage"),
+            destdir,
+            uniquify=False,
+            params=parameters["kernel"],
+        )
+        action.parameters = parameters
+        action.data = {}
+        action.parameters = {"namespace": "common"}
+        filename = Path(action.job.tmp_dir) / "downloads/common/zimage"
+        filename.parent.mkdir(parents=True)
+        filename.touch()
+        suffix = f"{job.job_id}/tftp-deploy-uid"
+        action.set_namespace_data(
+            action="tftp-deploy", label="tftp", key="suffix", value=suffix
+        )
+
+        action.run(None, 4242)
+
+        self.assertEqual(
+            action.get_namespace_data(
+                action="download-action", label="file", key="kernel"
+            ),
+            f"{suffix}/zimage",
+        )
+
+    def test_predownloaded_tftp_file_path_prefix_not_found(self):
+        parameters = {
+            "to": "tftp",
+            "kernel": {"url": "downloads://zimage", "type": "zimage"},
+            "namespace": "common",
+        }
+
+        job = self.create_simple_job()
+        destdir = job.mkdtemp("some-other-action")
+
+        action = PreDownloadedAction(
+            job,
+            "kernel",
+            urlparse("downloads://zimage"),
+            destdir,
+            uniquify=True,
+            params=parameters["kernel"],
+        )
+        action.parameters = parameters
+        action.data = {}
+        action.parameters = {"namespace": "common"}
+        filename = Path(action.job.tmp_dir) / "downloads/common/zimage"
+        filename.parent.mkdir(parents=True)
+        filename.touch()
+
+        with self.assertRaises(JobError):
+            action.run(None, 4242)
+
+    def test_predownloaded(self):
+        params = {
+            "to": "tmpfs",
+            "rootfs": {"url": "downloads://rootfs.xz"},
+            "namespace": "common",
+        }
+        job = self.create_simple_job()
+        destdir = job.mkdtemp("some-other-action")
+        action = PreDownloadedAction(
+            job, "rootfs", urlparse("downloads://rootfs.xz"), destdir, params=params
+        )
+        action.parameters = params
+
+        filename = Path(action.job.tmp_dir) / "downloads/common/rootfs.xz"
+        filename.parent.mkdir(parents=True)
+        filename.touch()
+
+        action.data = {}
+        action.parameters = {"namespace": "common"}
+        action.validate()
+        action.run(None, 4242)
+        mapped_path = action.get_namespace_data(
+            action="download-action", label="rootfs", key="file"
+        )
+        self.assertEqual(mapped_path, (destdir + "/rootfs/rootfs.xz"))
+        self.assertTrue(Path(mapped_path).exists())
+
     def test_predownloaded_subdirectory(self):
         params = {"to": "tmpfs", "rootfs": {"url": "downloads://subdir/rootfs.xz"}}
         job = self.create_simple_job()
         destdir = job.mkdtemp("some-other-action")
         action = PreDownloadedAction(
-            job, "rootfs", urlparse("downloads://subdir/rootfs.xz"), destdir, params
+            job,
+            "rootfs",
+            urlparse("downloads://subdir/rootfs.xz"),
+            destdir,
+            params=params,
         )
         action.parameters = params
 
@@ -846,7 +1005,7 @@ class TestDowload(LavaDispatcherTestCase):
         mapped_path = action.get_namespace_data(
             action="download-action", label="rootfs", key="file"
         )
-        self.assertEqual(mapped_path, (destdir + "/subdir/rootfs.xz"))
+        self.assertEqual(mapped_path, (destdir + "/rootfs/subdir/rootfs.xz"))
         self.assertTrue(Path(mapped_path).exists())
 
     def test_predownloaded_missing_file(self):
@@ -867,9 +1026,7 @@ class TestDowload(LavaDispatcherTestCase):
     def test_address_place_holder(self):
         factory = Factory()
         factory.validate_job_strict = True
-        job = factory.create_job(
-            "kvm03.jinja2", "sample_jobs/qemu-download-postprocess.yaml"
-        )
+        job = factory.create_job("kvm03", "sample_jobs/qemu-download-postprocess.yaml")
         action = DownloadHandler(
             job, "key", "/path/to/save", urlparse("http://example.com/resource.img")
         )
@@ -889,3 +1046,103 @@ class TestDowload(LavaDispatcherTestCase):
         action.params = action.parameters["images"]["key"]
         action.validate()
         assert action.params["url"] == "http://foobar/resource.img"
+
+
+class TestHttpCache:
+    @pytest.mark.parametrize(
+        "dispatcher_params,expected_url",
+        [
+            # No cache: http_url_format_string not set
+            ({}, "artifact_url"),
+            # With cache: only http_url_format_string set
+            (
+                {"http_url_format_string": "http://kisscache/api/v1/fetch/?url=%s"},
+                "kisscache_url",
+            ),
+            # Include rules: should use cache
+            (
+                {
+                    "http_url_format_string": "http://kisscache/api/v1/fetch/?url=%s",
+                    "http_cache_include_rules": ["http://foobar", "http://example"],
+                },
+                "kisscache_url",
+            ),
+            # Include rules, but not matching: should NOT use cache
+            (
+                {
+                    "http_url_format_string": "http://kisscache/api/v1/fetch/?url=%s",
+                    "http_cache_include_rules": [
+                        "http://foobar",
+                        "http://foo",
+                        "http://bar",
+                    ],
+                },
+                "artifact_url",
+            ),
+            # Exclude rules: should NOT use cache
+            (
+                {
+                    "http_url_format_string": "http://kisscache/api/v1/fetch/?url=%s",
+                    "http_cache_exclude_rules": ["http://foobar", "http://example"],
+                },
+                "artifact_url",
+            ),
+            # Exclude rules, but not matching: should use cache
+            (
+                {
+                    "http_url_format_string": "http://kisscache/api/v1/fetch/?url=%s",
+                    "http_cache_exclude_rules": [
+                        "http://foobar",
+                        "http://foo",
+                        "http://bar",
+                    ],
+                },
+                "kisscache_url",
+            ),
+            # Both include and exclude rules, exclude matches: should NOT use cache
+            (
+                {
+                    "http_url_format_string": "http://kisscache/api/v1/fetch/?url=%s",
+                    "http_cache_include_rules": ["http://example.com"],
+                    "http_cache_exclude_rules": ["http://example.com"],
+                },
+                "artifact_url",
+            ),
+            # Both include and exclude rules, include matches but exclude does not: should use cache
+            (
+                {
+                    "http_url_format_string": "http://kisscache/api/v1/fetch/?url=%s",
+                    "http_cache_include_rules": ["http://example.com"],
+                    "http_cache_exclude_rules": ["http://foobar"],
+                },
+                "kisscache_url",
+            ),
+            # Both include and exclude rules, neither matches: should NOT use cache
+            (
+                {
+                    "http_url_format_string": "http://kisscache/api/v1/fetch/?url=%s",
+                    "http_cache_include_rules": ["http://foobar"],
+                    "http_cache_exclude_rules": ["http://barfoo"],
+                },
+                "artifact_url",
+            ),
+        ],
+    )
+    def test_http_cache(self, dispatcher_params, expected_url):
+        factory = Factory()
+        job = factory.create_job("kvm03", "sample_jobs/qemu-download-postprocess.yaml")
+        kisscache_url = "http://kisscache/api/v1/fetch/?url=%s"
+        artifact_url = "http://example.com/resource.img"
+        if dispatcher_params:
+            job.parameters["dispatcher"] = dispatcher_params
+        action = HttpDownloadAction(job, "key", "/path/to/save", urlparse(artifact_url))
+        action.parameters = {
+            "images": {"key": {"url": artifact_url}},
+            "namespace": "common",
+        }
+        action.params = action.parameters["images"]["key"]
+        action.validate()
+        if expected_url == "artifact_url":
+            assert action.url == urlparse(artifact_url)
+        else:
+            assert action.url == urlparse(kisscache_url % quote_plus(artifact_url))
