@@ -4,10 +4,11 @@
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
-from unittest.mock import MagicMock, patch
+from unittest.mock import MagicMock, call, patch
 
 from lava_common.exceptions import ConnectionClosedError, TestError
-from lava_dispatcher.actions.test.shell import TestShell, TestShellAction
+from lava_dispatcher.actions.test.shell import TestShellAction
+from lava_dispatcher.actions.test_strategy import TestShell
 
 from ...test_basic import LavaDispatcherTestCase
 
@@ -128,7 +129,9 @@ class TestTestShell(LavaDispatcherTestCase):
             action="repo-action", label="repo-action", key="uuid-list", value=["UUID"]
         )
 
-        data = ("ENDRUN", "0_DEFINITION UUID")
+        test_suite = "0_DEFINITION"
+        action.reports = {f"{test_suite}": {"results": {}, "ran": False}}
+        data = ("ENDRUN", f"{test_suite} UUID")
         with self.assertLogs(action.logger, "DEBUG") as action_logs, patch(
             "time.monotonic", monotonic
         ):
@@ -136,15 +139,15 @@ class TestTestShell(LavaDispatcherTestCase):
         self.assertEqual(
             [(r.levelname, r.message) for r in action_logs.records],
             [
-                ("DEBUG", "Received signal: <ENDRUN> 0_DEFINITION UUID"),
+                ("DEBUG", f"Received signal: <ENDRUN> {test_suite} UUID"),
                 ("INFO", "Ending use of test pattern."),
-                ("INFO", "Ending test lava.0_DEFINITION (UUID), duration 1.00"),
+                ("INFO", f"Ending test lava.{test_suite} (UUID), duration 1.00"),
             ],
         )
         action.logger.results.assert_called_once_with(
             {
                 "definition": "lava",
-                "case": "0_DEFINITION",
+                "case": test_suite,
                 "uuid": "UUID",
                 "repository": None,
                 "path": None,
@@ -152,9 +155,10 @@ class TestTestShell(LavaDispatcherTestCase):
                 "result": "pass",
                 "revision": "unspecified",
                 "namespace": "common",
-            },
+            }
         )
         self.assertIsNone(action.current_run)
+        self.assertTrue(action.reports[test_suite]["ran"])
 
         # "signal.ENDRUN exception"
         action = TestShellAction(job)
@@ -163,6 +167,80 @@ class TestTestShell(LavaDispatcherTestCase):
         with self.assertRaises(TestError):
             self.assertIs(action.check_patterns("signal", MockConnection(data)), True)
 
+    def test_signal_end_run_handle_expected(self):
+        job = self.create_simple_job()
+        action = TestShellAction(job)
+        expected_tcs = ["tc1", "tc2", "tc3", "tc4"]
+        action.report = {
+            "tc1": "pass",
+            "tc2": "fail",
+            "tc5": "pass",
+            "tc6": {"set": "set1", "result": "fail"},
+        }
+
+        action.logger.results = MagicMock()
+        action.parameters = {"namespace": "common"}
+        action.data = {}
+        action.set_namespace_data(
+            action="test",
+            label="UUID",
+            key="testdef_expected",
+            value=expected_tcs,
+        )
+
+        # The expected test cases list handler is triggered on "signal.ENDRUN".
+        test_suite = "0_DEFINITION"
+        action.reports = {f"{test_suite}": {"results": {}, "ran": False}}
+        params = [test_suite, "UUID"]
+        with self.assertLogs(action.logger, "DEBUG") as action_logs:
+            self.assertIs(action.signal_end_run(params), None)
+
+        # Test that missing test case are sorted and reported as 'fail'.
+        action.logger.results.assert_has_calls(
+            [
+                call(
+                    {
+                        "definition": test_suite,
+                        "case": "tc3",
+                        "result": "fail",
+                        "level": None,
+                        "extra": {
+                            "reason": "missing expected test cases are reported as 'fail' by LAVA."
+                        },
+                    }
+                ),
+                call(
+                    {
+                        "definition": test_suite,
+                        "case": "tc4",
+                        "result": "fail",
+                        "level": None,
+                        "extra": {
+                            "reason": "missing expected test cases are reported as 'fail' by LAVA."
+                        },
+                    }
+                ),
+            ],
+            any_order=False,
+        )
+
+        # Test that warning logs appear for both missing and unexpected test cases.
+        actual_logs = [(r.levelname, r.message) for r in action_logs.records]
+        self.assertIn(
+            ("WARNING", "Reporting missing expected test cases as 'fail' ..."),
+            actual_logs,
+        )
+        self.assertIn(("WARNING", "Unexpected test result: tc5: pass"), actual_logs)
+        self.assertIn(
+            (
+                "WARNING",
+                "Unexpected test result: tc6: {'set': 'set1', 'result': 'fail'}",
+            ),
+            actual_logs,
+        )
+
+        self.assertTrue(action.reports[test_suite]["ran"])
+
     def test_signal_start_end_tc(self):
         job = self.create_simple_job()
 
@@ -325,7 +403,7 @@ class TestTestShell(LavaDispatcherTestCase):
                 (
                     "DEBUG",
                     "Received signal: <TESTCASE> RESULT=pass TEST_CASE_ID=case_id MEASUREMENT=1234",
-                ),
+                )
             ],
         )
         action.logger.marker.assert_called_once_with(
@@ -337,7 +415,7 @@ class TestTestShell(LavaDispatcherTestCase):
                 "case": "case_id",
                 "result": "pass",
                 "measurement": 1234.0,
-            },
+            }
         )
 
         # "signal.TESTCASE with measurement and unit"
@@ -355,7 +433,7 @@ class TestTestShell(LavaDispatcherTestCase):
                 (
                     "DEBUG",
                     "Received signal: <TESTCASE> RESULT=pass TEST_CASE_ID=case_id MEASUREMENT=1234 UNITS=s",
-                ),
+                )
             ],
         )
         action.logger.marker.assert_called_once_with(
@@ -420,7 +498,7 @@ class TestTestShell(LavaDispatcherTestCase):
                 (
                     "DEBUG",
                     "Received signal: <TESTREFERENCE> case-id pass http://example.com",
-                ),
+                )
             ],
         )
         action.logger.results.assert_called_once_with(
@@ -431,3 +509,126 @@ class TestTestShell(LavaDispatcherTestCase):
                 "reference": "http://example.com",
             }
         )
+
+    def test_cleanup_no_run(self):
+        job = self.create_simple_job()
+        action = TestShellAction(job)
+        action.parameters = {"namespace": "common"}
+        testdefs = [
+            {
+                "name": "test1",
+                "from": "git",
+                "expected": ["tc1", "tc2"],
+            },
+            {
+                "name": "test2",
+                "from": "git",
+                "expected": ["tc3", "tc4"],
+            },
+        ]
+        action.parameters = {
+            "definitions": testdefs,
+            "stage": 0,
+        }
+        action.set_namespace_data(
+            action="test-definition",
+            label="test-definition",
+            key="test_list",
+            value=[testdefs],
+        )
+        # No report for test '1_test2'
+        action.reports = {
+            "0_test1": {"results": {"tc1": "pass", "tc2": "pass"}, "ran": True},
+        }
+
+        with patch.object(action, "handle_expected") as mock_handle_expected:
+            action.cleanup(None, None)
+            self.assertEqual(action.report, {})
+            mock_handle_expected.assert_called_once_with(["tc3", "tc4"], "1_test2")
+
+    def test_cleanup_incomplete_run(self):
+        job = self.create_simple_job()
+        action = TestShellAction(job)
+        action.parameters = {"namespace": "common"}
+        testdefs = [
+            {
+                "name": "test1",
+                "from": "git",
+                "expected": ["tc1", "tc2"],
+            },
+            {
+                "name": "test2",
+                "from": "git",
+                "expected": ["tc3", "tc4"],
+            },
+        ]
+        action.parameters = {
+            "definitions": testdefs,
+            "stage": 0,
+        }
+        action.set_namespace_data(
+            action="test-definition",
+            label="test-definition",
+            key="test_list",
+            value=[testdefs],
+        )
+        action.reports = {
+            "0_test1": {"results": {"tc1": "pass", "tc2": "pass"}, "ran": True},
+            # Test fails to finish and only one of two results saved.
+            "1_test2": {"results": {"tc3": "pass"}, "ran": False},
+        }
+
+        with patch.object(action, "handle_expected") as mock_handle_expected:
+            action.cleanup(None, None)
+            self.assertEqual(action.report, {"tc3": "pass"})
+            mock_handle_expected.assert_called_once_with(["tc3", "tc4"], "1_test2")
+
+    def test_cleanup_multi_stages(self):
+        job = self.create_simple_job()
+        action = TestShellAction(job)
+        action.parameters = {"namespace": "common"}
+        testdefs = [
+            {
+                "name": "test3",
+                "from": "git",
+                "expected": ["tc5", "tc6"],
+            },
+            {
+                "name": "test4",
+                "from": "git",
+                "expected": ["tc7", "tc8"],
+            },
+        ]
+        action.parameters = {
+            "definitions": testdefs,
+            "stage": 1,
+        }
+        action.set_namespace_data(
+            action="test-definition",
+            label="test-definition",
+            key="test_list",
+            value=[
+                [
+                    {
+                        "name": "test1",
+                        "from": "git",
+                        "expected": ["tc1", "tc2"],
+                    },
+                    {
+                        "name": "test2",
+                        "from": "git",
+                        "expected": ["tc3", "tc4"],
+                    },
+                ],
+                testdefs,
+            ],
+        )
+        # No run.
+        action.reports = {}
+
+        with patch.object(action, "handle_expected") as mock_handle_expected:
+            action.cleanup(None, None)
+
+            self.assertEqual(mock_handle_expected.call_count, 2)
+            mock_handle_expected.assert_any_call(["tc5", "tc6"], "2_test3")
+            mock_handle_expected.assert_any_call(["tc7", "tc8"], "3_test4")
