@@ -36,10 +36,10 @@ class ConnectionFactory(Factory):
     """
 
     def create_ssh_job(self, filename):
-        return self.create_job("ssh-host-01.jinja2", filename, validate=False)
+        return self.create_job("ssh-host-01", filename, validate=False)
 
     def create_bbb_job(self, filename):
-        return self.create_job("bbb-02.jinja2", filename, validate=False)
+        return self.create_job("bbb-02", filename, validate=False)
 
 
 class TestConnection(LavaDispatcherTestCase):
@@ -327,9 +327,7 @@ class TestConnection(LavaDispatcherTestCase):
         )
         if "protocols" in scp_overlay.parameters:
             for params in scp_overlay.parameters["protocols"][MultinodeProtocol.name]:
-                (replacement_key, placeholder) = [
-                    (key, value) for key, value in params["message"].items()
-                ][0]
+                replacement_key, placeholder = next(iter(params["message"].items()))
                 self.assertEqual(data[replacement_key], "172.16.200.165")
                 self.assertEqual(placeholder, "$ipaddr")
         environment = scp_overlay.get_namespace_data(
@@ -449,8 +447,8 @@ class TestTimeouts(LavaDispatcherTestCase):
             test_shell.connection_timeout.duration, 240
         )  # job specifies 4 minutes
         self.assertEqual(
-            test_shell.timeout.duration, 300
-        )  # job (test action block) specifies 5 minutes
+            test_shell.timeout.duration, 420
+        )  # job (test action block) specifies 5 minutes with 3 times retry.
         self.assertEqual(deploy.timeout.duration, 120)  # job specifies 2 minutes
         self.assertEqual(deploy.connection_timeout.duration, Timeout.default_duration())
         self.assertNotEqual(
@@ -470,7 +468,7 @@ class TestTimeouts(LavaDispatcherTestCase):
         with open(y_file) as uboot_ramdisk:
             data = yaml_safe_load(uboot_ramdisk)
         data["timeouts"]["connection"] = {"seconds": 20}
-        job = self.factory.create_custom_job("bbb-01.jinja2", data)
+        job = self.factory.create_custom_job("bbb-01", data)
         for action in job.pipeline.actions:
             if action.pipeline:
                 for check_action in action.pipeline.actions:
@@ -483,7 +481,7 @@ class TestTimeouts(LavaDispatcherTestCase):
                         self.assertEqual(check_action.connection_timeout.duration, 20)
 
         test_shell = job.pipeline.find_action(TestShellAction)
-        self.assertEqual(test_shell.timeout.duration, 300)
+        self.assertEqual(test_shell.timeout.duration, 420)
         auto = job.pipeline.find_action(AutoLoginAction)
         self.assertEqual(auto.connection_timeout.duration / 60, 12)
 
@@ -504,7 +502,7 @@ class TestTimeouts(LavaDispatcherTestCase):
         data["timeouts"]["connections"]["uboot-commands"] = {}
         data["timeouts"]["connections"]["uboot-commands"]["seconds"] = 45
         self.assertEqual(connection_timeout, 240)
-        job = self.factory.create_custom_job("bbb-01.jinja2", data)
+        job = self.factory.create_custom_job("bbb-01", data)
         retry = job.pipeline.find_action(UBootCommandsAction)
         self.assertEqual(
             retry.timeout.duration, 90
@@ -524,7 +522,7 @@ class TestDisconnect(LavaDispatcherTestCase):
     def test_handled_disconnect(self):
         factory = ConnectionFactory()
         job = factory.create_job(
-            "mps2plus-01.jinja2", "sample_jobs/mps2plus.yaml", validate=False
+            "mps2plus-01", "sample_jobs/mps2plus.yaml", validate=False
         )
         job.validate()
         self.assertTrue(
@@ -534,16 +532,16 @@ class TestDisconnect(LavaDispatcherTestCase):
     def test_disconnect_with_plain_connection_command(self):
         factory = ConnectionFactory()
         job = factory.create_job(
-            "mps2plus-02.jinja2", "sample_jobs/mps2plus.yaml", validate=False
+            "mps2plus-02", "sample_jobs/mps2plus.yaml", validate=False
         )
-        # mps2plus-02.jinja2 does not use tags for it's connection
+        # mps2plus-02 does not use tags for it's connection
         with self.assertRaises(JobError):
             job.validate()
 
     def test_unhandled_disconnect(self):
         factory = ConnectionFactory()
         job = factory.create_job(
-            "mps2plus-03.jinja2", "sample_jobs/mps2plus.yaml", validate=False
+            "mps2plus-03", "sample_jobs/mps2plus.yaml", validate=False
         )
         try:
             job.validate()
@@ -551,7 +549,7 @@ class TestDisconnect(LavaDispatcherTestCase):
             pass
         except InfrastructureError:
             raise self.skipTest("Cannot validate if minicom is not on PATH")
-        # mps2plus-03.jinja2 does use a tag, but minicom cannot be disconnected from currently
+        # mps2plus-03 does use a tag, but minicom cannot be disconnected from currently
         # Want to check the exception message gives some useful information
         # on how to amend the device dictionary.
         with self.assertRaises(JobError) as exc:
