@@ -3,6 +3,7 @@
 # Author: Neil Williams <neil.williams@linaro.org>
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import glob
 import os
@@ -11,12 +12,12 @@ import shutil
 import stat
 import tempfile
 import unittest
-from unittest.mock import patch
+from unittest.mock import call, patch
 
 import pexpect
 
 from lava_common.exceptions import InfrastructureError
-from lava_common.yaml import yaml_safe_dump, yaml_safe_load
+from lava_common.yaml import yaml_safe_load
 from lava_dispatcher.actions.deploy.apply_overlay import ApplyOverlayTftp
 from lava_dispatcher.actions.deploy.download import DownloaderAction
 from lava_dispatcher.actions.deploy.fastboot import FastbootAction
@@ -24,14 +25,13 @@ from lava_dispatcher.actions.deploy.lxc import LxcAction
 from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.actions.deploy.testdef import (
     GitRepoAction,
+    RepoAction,
     TestDefinitionAction,
     TestInstallAction,
     TestOverlayAction,
     TestRunnerAction,
 )
 from lava_dispatcher.actions.test.shell import PatternFixup
-from lava_dispatcher.device import NewDevice
-from lava_dispatcher.parser import JobParser
 from lava_dispatcher.power import FinalizeAction
 from tests.lava_dispatcher.test_basic import Factory, LavaDispatcherTestCase
 from tests.lava_dispatcher.test_uboot import UBootFactory
@@ -58,7 +58,7 @@ class TestDefinitionHandlers(LavaDispatcherTestCase):
     def setUp(self):
         super().setUp()
         self.factory = Factory()
-        self.job = self.factory.create_job("qemu01.jinja2", "sample_jobs/kvm.yaml")
+        self.job = self.factory.create_job("qemu01", "sample_jobs/kvm.yaml")
         with open(
             os.path.join(os.path.dirname(__file__), "testdefs", "params.yaml")
         ) as params:
@@ -69,7 +69,7 @@ class TestDefinitionHandlers(LavaDispatcherTestCase):
         action = self.job.pipeline.actions[0]
         overlay = action.pipeline.actions[0]
         testdef = overlay.pipeline.actions[2]
-        self.assertEqual(len(overlay.pipeline.actions), 5)
+        self.assertEqual(len(overlay.pipeline.actions), 4)
         self.assertIsInstance(testdef, TestDefinitionAction)
         testdef.validate()
         self.assertEqual(
@@ -110,17 +110,9 @@ class TestDefinitionHandlers(LavaDispatcherTestCase):
         testdef = self.job.pipeline.find_action(TestDefinitionAction)
         testdef.validate()
         self.assertEqual([], testdef.errors)
-        (rendered, _) = self.factory.create_device("kvm01.jinja2")
-        device = NewDevice(yaml_safe_load(rendered))
-        kvm_yaml = os.path.join(os.path.dirname(__file__), "sample_jobs/kvm.yaml")
-        parser = JobParser()
-        with open(kvm_yaml) as sample_job_data:
-            content = yaml_safe_load(sample_job_data)
-        data = [block["test"] for block in content["actions"] if "test" in block][0]
-        definitions = [block for block in data["definitions"] if "path" in block][0]
-        definitions["name"] = "smoke tests"
-        job = parser.parse(yaml_safe_dump(content), device, 4212, None, "")
+        job = self.factory.create_job("kvm01", "sample_jobs/kvm.yaml")
         testdef = job.pipeline.find_action(TestDefinitionAction)
+        testdef.test_list[0][0]["name"] = "smoke tests"  # Introduce match error
         testdef.validate()
         self.assertNotEqual([], testdef.errors)
         self.assertIn(
@@ -183,7 +175,7 @@ class TestDefinitionHandlers(LavaDispatcherTestCase):
         )
 
     def test_overlay_override(self):
-        job = self.factory.create_job("qemu01.jinja2", "sample_jobs/kvm-context.yaml")
+        job = self.factory.create_job("qemu01", "sample_jobs/kvm-context.yaml")
         overlay = job.pipeline.find_action(OverlayAction)
         self.assertEqual(
             "/sysroot/lava-%s", overlay.get_constant("lava_test_results_dir", "posix")
@@ -251,7 +243,6 @@ class TestDefinitionParams(LavaDispatcherTestCase):
                 "###test parameters from job submission###\n",
                 "VARIABLE_NAME_1='eth2'\n",
                 "VARIABLE_NAME_2='wlan0'\n",
-                "######\n",
             },
         )
         testdef = {
@@ -268,7 +259,6 @@ class TestDefinitionParams(LavaDispatcherTestCase):
                 "###test parameters from job submission###\n",
                 "VARIABLE_NAME_1='eth2'\n",
                 "VARIABLE_NAME_2='wlan0'\n",
-                "######\n",
             },
         )
 
@@ -378,10 +368,7 @@ class TestDefinitions(LavaDispatcherTestCase):
             "lava_dispatcher.actions.deploy.tftp.which",
             return_value="/usr/bin/in.tftpd",
         ), patch(
-            "lava_dispatcher.actions.deploy.overlay.which",
-            return_value="/usr/bin/rpcinfo",
-        ), patch(
-            "lava_dispatcher.actions.deploy.overlay.rpcinfo_nfs",
+            "lava_dispatcher.actions.deploy.apply_overlay.rpcinfo_nfs",
             return_value=None,
         ):
             self.job.validate()
@@ -411,10 +398,11 @@ class TestDefinitions(LavaDispatcherTestCase):
                 label="test-runscript-overlay",
                 key="testdef_levels",
             ),
-            {"1.3.2.4.4": "0_smoke-tests", "1.3.2.4.8": "1_singlenode-advanced"},
+            {"1.1.4.4": "0_smoke-tests", "1.1.4.8": "1_singlenode-advanced"},
         )
         self.assertEqual(
-            {repo.uuid for repo in git_repos}, {"4999_1.3.2.4.1", "4999_1.3.2.4.5"}
+            {repo.uuid for repo in git_repos},
+            {f"{self.job.job_id}_1.1.4.1", f"{self.job.job_id}_1.1.4.5"},
         )
         self.assertEqual(
             set(
@@ -444,12 +432,12 @@ class TestDefinitions(LavaDispatcherTestCase):
         self.assertEqual(
             self.job.context["test"],
             {
-                "4999_1.3.2.4.5": {
+                f"{self.job.job_id}_1.1.4.1": {
                     "testdef_pattern": {
                         "pattern": "(?P<test_case_id>.*-*):\\s+(?P<result>(pass|fail))"
                     }
                 },
-                "4999_1.3.2.4.1": {
+                f"{self.job.job_id}_1.1.4.5": {
                     "testdef_pattern": {
                         "pattern": "(?P<test_case_id>.*-*):\\s+(?P<result>(pass|fail))"
                     }
@@ -466,7 +454,7 @@ class TestDefinitions(LavaDispatcherTestCase):
         self.assertIsNotNone(uuid_list)
         for key, value in enumerate(testdef_index):
             if start_run == "%s_%s" % (key, value):
-                self.assertEqual("4999_1.3.2.4.1", uuid_list[key])
+                self.assertEqual(f"{self.job.job_id}_1.1.4.1", uuid_list[key])
                 self.assertEqual(
                     self.job.context["test"][uuid_list[key]]["testdef_pattern"][
                         "pattern"
@@ -510,7 +498,7 @@ test3a: skip
     )
     def test_deployment_data(self):
         job = self.factory.create_job(
-            "hi960-hikey-01.jinja2", "sample_jobs/hikey960-oe-aep.yaml"
+            "hi960-hikey-01", "sample_jobs/hikey960-oe-aep.yaml"
         )
         job.validate()
         description_ref = self.pipeline_reference("hikey960-oe-aep.yaml", job=job)
@@ -545,3 +533,302 @@ test3a: skip
         self.assertEqual(
             "oe", fastboot_installscript.parameters["deployment_data"]["distro"]
         )
+
+
+class TestRunScript(LavaDispatcherTestCase):
+    def setUp(self):
+        self.job = self.create_simple_job()
+
+        self.action = TestRunnerAction(self.job)
+        self.action.section = "test"
+        # Set by TestDefinitionAction to: JobID_RepoActionPipelineLevel
+        self.action.test_uuid = "1_1.1.4.5"
+        # TestDefinitionAction populate the below pipeline for each testdef:
+        # UrlRepoAction->TestOverlayAction->TestInstallAction->TestRunnerAction
+        self.action.level = "1.1.4.8"
+
+        self.action.parameters = {
+            "name": "test1",
+            "path": "test1.yaml",
+            "from": "git",
+            "repository": "http://example.com/repo.git",
+            # populated by TestDefinitionAction.
+            "test_name": "1_test1",
+        }
+
+        self.testdef = {
+            "parameters": {"key1": "value1"},
+            "run": {"steps": ["step1", "step2"]},
+        }
+
+    def test_validate(self):
+        with patch(
+            "lava_dispatcher.actions.deploy.testdef.TestRunnerAction.get_namespace_data",
+            side_effect=[["test0", "test1"], None],
+        ):
+            with patch(
+                "lava_dispatcher.actions.deploy.testdef.TestRunnerAction.set_namespace_data"
+            ) as mock_set_testdef_levels:
+                self.action.validate()
+                mock_set_testdef_levels.assert_called_once_with(
+                    action="test-runscript-overlay",
+                    key="testdef_levels",
+                    label="test-runscript-overlay",
+                    value={"1.1.4.8": "1_test1"},
+                )
+                self.assertTrue(self.action.valid)
+
+    def test_validate_unique_names(self):
+        with patch(
+            "lava_dispatcher.actions.deploy.testdef.TestRunnerAction.get_namespace_data",
+            side_effect=[["test1", "test1"], None],
+        ):
+            with patch(
+                "lava_dispatcher.actions.deploy.testdef.TestRunnerAction.set_namespace_data"
+            ):
+                self.action.validate()
+                self.assertFalse(self.action.valid)
+                self.assertIn(
+                    "Test definition names need to be unique.", self.action.errors
+                )
+
+    def test_validate_update_testdef_levels(self):
+        with patch(
+            "lava_dispatcher.actions.deploy.testdef.TestRunnerAction.get_namespace_data",
+            side_effect=[["test0", "test1"], {"1.1.4.4": "0_test0"}],
+        ):
+            with patch(
+                "lava_dispatcher.actions.deploy.testdef.TestRunnerAction.set_namespace_data"
+            ) as mock_set_testdef_levels:
+                self.action.validate()
+                mock_set_testdef_levels.assert_called_once_with(
+                    action="test-runscript-overlay",
+                    key="testdef_levels",
+                    label="test-runscript-overlay",
+                    value={"1.1.4.4": "0_test0", "1.1.4.8": "1_test1"},
+                )
+                self.assertTrue(self.action.valid)
+
+    @patch(
+        "lava_dispatcher.actions.deploy.testdef.TestRunnerAction.get_namespace_data",
+        side_effect=[
+            "/overlay/runner/path",
+            {"1.1.4.4": "0_test0", "1.1.4.8": "1_test1"},
+            "/lava-1/1/tests/1_test1",
+        ],
+    )
+    @patch("lava_dispatcher.actions.deploy.testdef.TestOverlayAction.run")
+    def test_run(
+        self,
+        mock_super_run,
+        mock_get_ns_data,
+    ):
+        with patch("builtins.open", unittest.mock.mock_open()) as mock_file:
+            with patch(
+                "lava_dispatcher.actions.deploy.testdef.yaml_safe_load",
+                return_value=self.testdef,
+            ):
+                self.action.run(None, None)
+
+                mock_file().write.assert_has_calls(
+                    [
+                        call("###default parameters from test definition###\n"),
+                        call("key1='value1'\n"),
+                        call("######\n"),
+                        call("###test parameters from job submission###\n"),
+                        call("######\n"),
+                        call("set -e\n"),
+                        call("set -x\n"),
+                        call("export TESTRUN_ID=1_test1\n"),
+                        call("cd /lava-1/1/tests/1_test1\n"),
+                        call('UUID="$(cat uuid)"\n'),
+                        call("set +x\n"),
+                        call('echo "<LAVA_SIGNAL_STARTRUN $TESTRUN_ID $UUID>"\n'),
+                        call("set -x\n"),
+                        call("step1\n"),
+                        call("step2\n"),
+                        call("set +x\n"),
+                        call('echo "<LAVA_SIGNAL_ENDRUN $TESTRUN_ID $UUID>"\n'),
+                    ]
+                )
+
+                self.assertEqual(self.action.results["uuid"], "1_1.1.4.5")
+                self.assertEqual(self.action.results["name"], "test1")
+                self.assertEqual(self.action.results["path"], "test1.yaml")
+                self.assertEqual(self.action.results["from"], "git")
+                self.assertEqual(
+                    self.action.results["repository"],
+                    "http://example.com/repo.git",
+                )
+
+    @patch(
+        "lava_dispatcher.actions.deploy.testdef.TestRunnerAction.get_namespace_data",
+        side_effect=[
+            "/overlay/runner/path",
+            {"1.1.4.4": "0_test0", "1.1.4.8": "1_test1"},
+            "/lava-1/1/tests/1_test1",
+        ],
+    )
+    @patch("lava_dispatcher.actions.deploy.testdef.TestOverlayAction.run")
+    def test_run_kmsg_signal(
+        self,
+        mock_super_run,
+        mock_get_ns_data,
+    ):
+        self.action.parameters["lava-signal"] = "kmsg"
+
+        with patch("builtins.open", unittest.mock.mock_open()) as mock_file:
+            with patch(
+                "lava_dispatcher.actions.deploy.testdef.yaml_safe_load",
+                return_value=self.testdef,
+            ):
+                self.action.run(None, None)
+
+                mock_file().write.assert_has_calls(
+                    [
+                        call("###default parameters from test definition###\n"),
+                        call("key1='value1'\n"),
+                        call("######\n"),
+                        call("###test parameters from job submission###\n"),
+                        call("######\n"),
+                        call("set -e\n"),
+                        call("set -x\n"),
+                        call("export TESTRUN_ID=1_test1\n"),
+                        call("cd /lava-1/1/tests/1_test1\n"),
+                        call('UUID="$(cat uuid)"\n'),
+                        call("set +x\n"),
+                        call("export KMSG=true\n"),
+                        call(
+                            'echo "<0><LAVA_SIGNAL_STARTRUN $TESTRUN_ID $UUID>" > /dev/kmsg\n'
+                        ),
+                        call("set -x\n"),
+                        call("step1\n"),
+                        call("step2\n"),
+                        call("set +x\n"),
+                        call("unset KMSG\n"),
+                        call(
+                            'echo "<0><LAVA_SIGNAL_ENDRUN $TESTRUN_ID $UUID>" > /dev/kmsg\n'
+                        ),
+                    ]
+                )
+
+    @patch(
+        "lava_dispatcher.actions.deploy.testdef.TestRunnerAction.get_namespace_data",
+        side_effect=[
+            "/overlay/runner/path",
+            {"1.1.4.4": "0_test0", "1.1.4.8": "1_test1"},
+            "/lava-1/1/tests/1_test1",
+        ],
+    )
+    @patch("lava_dispatcher.actions.deploy.testdef.TestOverlayAction.run")
+    def test_run_character_delay(
+        self,
+        mock_super_run,
+        mock_get_ns_data,
+    ):
+        self.action.parameters["needs_character_delay"] = True
+        self.job.device["character_delays"] = {"test": "10"}
+
+        with patch("builtins.open", unittest.mock.mock_open()) as mock_file:
+            with patch(
+                "lava_dispatcher.actions.deploy.testdef.yaml_safe_load",
+                return_value=self.testdef,
+            ):
+                self.action.run(None, None)
+
+                mock_file().write.assert_has_calls(
+                    [
+                        call("###default parameters from test definition###\n"),
+                        call("key1='value1'\n"),
+                        call("######\n"),
+                        call("###test parameters from job submission###\n"),
+                        call("######\n"),
+                        call("set -e\n"),
+                        call("set -x\n"),
+                        call("export TESTRUN_ID=1_test1\n"),
+                        call("cd /lava-1/1/tests/1_test1\n"),
+                        call('UUID="$(cat uuid)"\n'),
+                        call("set +x\n"),
+                        call("export CHARACTER_DELAY=0.01\n"),
+                        call("sleep 0.01\n"),
+                        call('echo "<LAVA_SIGNAL_STARTRUN $TESTRUN_ID $UUID>"\n'),
+                        call("set -x\n"),
+                        call("step1\n"),
+                        call("step2\n"),
+                        call("set +x\n"),
+                        call("sleep 0.01\n"),
+                        call('echo "<LAVA_SIGNAL_ENDRUN $TESTRUN_ID $UUID>"\n'),
+                    ]
+                )
+
+
+class TestStoreTestdefExpectedList(LavaDispatcherTestCase):
+    def setUp(self):
+        self.job = self.create_simple_job()
+        self.action = RepoAction(self.job)
+        self.action.uuid = "UUID"
+        self.action.parameters = {
+            "namespace": "common",
+            "repository": "https://gitlab.com/lava/functional-tests.git",
+            "from": "git",
+            "path": "posix/smoke-tests-basic.yaml",
+            "name": "smoke-tests",
+        }
+        self.testdef: dict[str, dict | list] = {
+            "metadata": {
+                "format": "Lava-Test Test Definition 1.0",
+                "name": "test-expected-test-case-list",
+            }
+        }
+
+    def tes_no_expected_list(self):
+        data = self.action.get_namespace_data(
+            action="test", label="UUID", key="testdef_expected"
+        )
+
+        self.assertIsNone(data)
+
+    def test_empty_expected_list(self):
+        self.testdef["expected"] = []
+
+        self.action.store_testdef(self.testdef, "git")
+
+        data = self.action.get_namespace_data(
+            action="test", label="UUID", key="testdef_expected"
+        )
+
+        self.assertIsNone(data)
+
+    def test_expected_list_from_testdef(self):
+        self.testdef["expected"] = ["tc1", "tc2", "tc3"]
+
+        self.action.store_testdef(self.testdef, "git")
+
+        data = self.action.get_namespace_data(
+            action="test", label="UUID", key="testdef_expected"
+        )
+
+        self.assertEqual(data, self.testdef["expected"])
+
+    def test_expected_list_from_jobdef(self):
+        self.action.parameters["expected"] = ["tc2", "tc3", "tc4"]
+
+        self.action.store_testdef(self.testdef, "git")
+
+        data = self.action.get_namespace_data(
+            action="test", label="UUID", key="testdef_expected"
+        )
+
+        self.assertEqual(data, self.action.parameters["expected"])
+
+    def test_jobdef_overwrites_testdef_expected_list(self):
+        self.testdef["expected"] = ["tc1", "tc2", "tc3"]
+        self.action.parameters["expected"] = ["tc2", "tc3", "tc4"]
+
+        self.action.store_testdef(self.testdef, "git")
+
+        data = self.action.get_namespace_data(
+            action="test", label="UUID", key="testdef_expected"
+        )
+
+        self.assertEqual(data, self.action.parameters["expected"])
