@@ -5,7 +5,7 @@
 # SPDX-License-Identifier: GPL-2.0-or-later
 
 
-from unittest.mock import patch
+from unittest.mock import MagicMock, patch
 
 import pexpect
 
@@ -19,31 +19,19 @@ from lava_dispatcher.actions.test.interactive import TestInteractiveAction
 from tests.lava_dispatcher.test_basic import Factory, LavaDispatcherTestCase
 
 
-class InteractiveFactory(Factory):
-    def create_interactive_job(self, device, filename):
-        return self.create_job(device, filename)
-
-
 class TestInteractive(LavaDispatcherTestCase):
     def test_pipeline(self):
-        factory = InteractiveFactory()
-        job = factory.create_interactive_job(
-            "b2260-01.jinja2", "sample_jobs/b2260-interactive.yaml"
-        )
+        factory = Factory()
+        job = factory.create_job("b2260-01", "sample_jobs/b2260-interactive.yaml")
         job.validate()
         description_ref = LavaDispatcherTestCase.pipeline_reference(
             "b2260-interactive.yaml", job=job
         )
-        self.assertEqual(
-            description_ref,
-            job.pipeline.describe(),
-        )
+        self.assertEqual(description_ref, job.pipeline.describe())
 
     def test_bbb(self):
-        factory = InteractiveFactory()
-        job = factory.create_interactive_job(
-            "bbb-01.jinja2", "sample_jobs/bbb-uboot-interactive.yaml"
-        )
+        factory = Factory()
+        job = factory.create_job("bbb-01", "sample_jobs/bbb-uboot-interactive.yaml")
         job.validate()
         description_ref = LavaDispatcherTestCase.pipeline_reference(
             "bbb-uboot-interactive.yaml", job=job
@@ -54,10 +42,8 @@ class TestInteractive(LavaDispatcherTestCase):
         )
 
     def test_stages(self):
-        factory = InteractiveFactory()
-        job = factory.create_interactive_job(
-            "docker-01.jinja2", "sample_jobs/docker-interactive.yaml"
-        )
+        factory = Factory()
+        job = factory.create_job("docker-01", "sample_jobs/docker-interactive.yaml")
         with patch(
             "lava_dispatcher.actions.deploy.docker.which",
             return_value="/usr/bin/docker",
@@ -67,10 +53,7 @@ class TestInteractive(LavaDispatcherTestCase):
         description_ref = LavaDispatcherTestCase.pipeline_reference(
             "docker-interactive.yaml", job=job
         )
-        self.assertEqual(
-            description_ref,
-            job.pipeline.describe(),
-        )
+        self.assertEqual(description_ref, job.pipeline.describe())
         self.assertEqual(
             job.pipeline.actions[3].pipeline.actions[0].parameters["stage"],
             0,
@@ -332,6 +315,18 @@ class TestInteractiveScript(LavaDispatcherTestCase):
         self.assertEqual(action.logger.data, [])
         self.assertEqual(test_connection.data, [])
 
+        self.assertEqual(
+            action.report,
+            {
+                "wait prompt": "pass",
+                "network.help": "pass",
+                "network.dhcp": "pass",
+                "network.host": "fail",
+                "network.dig": "fail",
+                "network.ping": "fail",
+            },
+        )
+
     @patch("time.monotonic", Timing())
     def test_run_script_echo_discard(self):
         action = TestInteractiveAction(self.create_simple_job())
@@ -459,10 +454,7 @@ class TestInteractiveScript(LavaDispatcherTestCase):
         test_connection = Connection(conn_data)
         substitutions = {}
         action.run_script(test_connection, script, substitutions)
-        self.assertEqual(
-            substitutions,
-            {"{val}": "bar"},
-        )
+        self.assertEqual(substitutions, {"{val}": "bar"})
         self.assertEqual(action.logger.data, [])
         self.assertEqual(test_connection.data, [])
 
@@ -706,10 +698,8 @@ class TestInteractiveScript(LavaDispatcherTestCase):
         self.assertEqual(test_connection.data, [])
 
     def test_namespace_connection(self):
-        factory = InteractiveFactory()
-        job = factory.create_interactive_job(
-            "b2260-01.jinja2", "sample_jobs/b2260-interactive.yaml"
-        )
+        factory = Factory()
+        job = factory.create_job("b2260-01", "sample_jobs/b2260-interactive.yaml")
         action = job.pipeline.find_action(TestInteractiveAction)
         action.logger = Logger(
             [
@@ -739,10 +729,8 @@ class TestInteractiveScript(LavaDispatcherTestCase):
         self.assertEqual(new_connection, data_connection)
 
     def test_connection_namespace_connection(self):
-        factory = InteractiveFactory()
-        job = factory.create_interactive_job(
-            "b2260-01.jinja2", "sample_jobs/b2260-interactive.yaml"
-        )
+        factory = Factory()
+        job = factory.create_job("b2260-01", "sample_jobs/b2260-interactive.yaml")
         action = job.pipeline.find_action(TestInteractiveAction)
         action.parameters["connection-namespace"] = "foobar"
         action.logger = Logger(
@@ -774,3 +762,164 @@ class TestInteractiveScript(LavaDispatcherTestCase):
 
         new_connection = action.run(previous_connection, 10)
         self.assertEqual(new_connection, data_connection)
+
+
+class TestInteractiveReport(LavaDispatcherTestCase):
+    @patch("time.monotonic", Timing())
+    def test_run_with_expected(self):
+        action = TestInteractiveAction(self.create_simple_job())
+        action.parameters = {
+            "stage": 0,
+            "interactive": [
+                {
+                    "name": "network",
+                    "prompts": ["=> "],
+                    "script": [],
+                    "expected": ["tc1", "tc2"],
+                }
+            ],
+        }
+        mock_connection = MagicMock()
+        mock_connection.prompt_str = None
+
+        with patch.object(
+            action,
+            "get_namespace_data",
+            side_effect=[
+                mock_connection,
+                {"{SERVER_IP}": "serverip", "{JOB_ID}": "123"},
+            ],
+        ):
+            with patch.object(action, "run_script") as mock_run_script:
+                with patch.object(action, "handle_expected") as mock_handle_expected:
+                    with patch.object(action, "handle_summary") as mock_handle_summary:
+                        with patch.object(action, "logger"):
+                            action.run(mock_connection, None)
+
+                            self.assertIn("0_network", action.reports)
+                            self.assertIs(
+                                action.reports["0_network"]["results"],
+                                action.report,
+                            )
+                            mock_run_script.assert_called_once()
+                            mock_handle_expected.assert_called_once_with(
+                                ["tc1", "tc2"], "0_network"
+                            )
+                            self.assertTrue(action.reports["0_network"]["ran"])
+                            mock_handle_summary.assert_called_once_with("0_network")
+
+    @patch("time.monotonic", Timing())
+    def test_run_without_expected(self):
+        action = TestInteractiveAction(self.create_simple_job())
+        action.parameters = {
+            "stage": 0,
+            "interactive": [
+                {
+                    "name": "network",
+                    "prompts": ["=> "],
+                    "script": [],
+                }
+            ],
+        }
+        mock_connection = MagicMock()
+        mock_connection.prompt_str = None
+
+        with patch.object(
+            action,
+            "get_namespace_data",
+            side_effect=[
+                mock_connection,
+                {"{SERVER_IP}": "serverip", "{JOB_ID}": "123"},
+            ],
+        ):
+            with patch.object(action, "run_script") as mock_run_script:
+                with patch.object(action, "handle_expected") as mock_handle_expected:
+                    with patch.object(action, "handle_summary") as mock_handle_summary:
+                        with patch.object(action, "logger"):
+                            action.run(mock_connection, None)
+
+                            self.assertIn("0_network", action.reports)
+                            self.assertIs(
+                                action.reports["0_network"]["results"],
+                                action.report,
+                            )
+                            mock_run_script.assert_called_once()
+                            mock_handle_expected.assert_not_called()
+                            self.assertTrue(action.reports["0_network"]["ran"])
+                            mock_handle_summary.assert_called_once_with("0_network")
+
+
+class TestInteractiveCleanup(LavaDispatcherTestCase):
+    def setUp(self):
+        self.job = self.create_simple_job()
+        self.action = TestInteractiveAction(self.job)
+
+    def test_cleanup_no_run(self):
+        scripts = [
+            {"name": "script1", "expected": ["tc1", "tc2"]},
+            {"name": "script2", "expected": ["tc3", "tc4"]},
+        ]
+        self.action.parameters = {"interactive": scripts, "stage": 0}
+        # No report for script2
+        self.action.reports = {
+            "0_script1": {"results": {"tc1": "pass", "tc2": "pass"}, "ran": True},
+        }
+
+        with patch.object(self.action, "handle_expected") as mock_handle_expected:
+            self.action.cleanup(None, None)
+            self.assertEqual(self.action.report, {})
+            mock_handle_expected.assert_called_once_with(["tc3", "tc4"], "0_script2")
+
+    def test_cleanup_incomplete_run(self):
+        job = self.create_simple_job()
+        action = TestInteractiveAction(job)
+        scripts = [
+            {"name": "script1", "expected": ["tc1", "tc2"]},
+            {"name": "script2", "expected": ["tc3", "tc4"]},
+        ]
+        action.parameters = {"interactive": scripts, "stage": 0}
+        action.reports = {
+            "0_script1": {"results": {"tc1": "pass", "tc2": "pass"}, "ran": True},
+            # Script fails to finish and only one of two results saved.
+            "0_script2": {"results": {"tc3": "pass"}, "ran": False},
+        }
+
+        with patch.object(action, "handle_expected") as mock_handle_expected:
+            action.cleanup(None, None)
+            self.assertEqual(action.report, {"tc3": "pass"})
+            mock_handle_expected.assert_called_once_with(["tc3", "tc4"], "0_script2")
+
+    def test_cleanup_no_expected(self):
+        job = self.create_simple_job()
+        action = TestInteractiveAction(job)
+        scripts = [
+            {"name": "script1"},
+            {"name": "script2", "expected": ["tc3", "tc4"]},
+        ]
+        action.parameters = {"interactive": scripts, "stage": 0}
+        action.reports = {
+            "0_script1": {"results": {}, "ran": False},
+            "0_script2": {"results": {"tc3": "pass"}, "ran": False},
+        }
+
+        with patch.object(action, "handle_expected") as mock_handle_expected:
+            action.cleanup(None, None)
+            # script1 has no expected, so handle_expected should not be called for it
+            mock_handle_expected.assert_called_once_with(["tc3", "tc4"], "0_script2")
+
+    def test_cleanup_all_ran(self):
+        job = self.create_simple_job()
+        action = TestInteractiveAction(job)
+        scripts = [
+            {"name": "script1", "expected": ["tc1", "tc2"]},
+            {"name": "script2", "expected": ["tc3", "tc4"]},
+        ]
+        action.parameters = {"interactive": scripts, "stage": 0}
+        action.reports = {
+            "0_script1": {"results": {"tc1": "pass", "tc2": "pass"}, "ran": True},
+            "0_script2": {"results": {"tc3": "pass", "tc4": "pass"}, "ran": True},
+        }
+
+        with patch.object(action, "handle_expected") as mock_handle_expected:
+            action.cleanup(None, None)
+            mock_handle_expected.assert_not_called()
