@@ -4,16 +4,18 @@
 #         Remi Duraffort <remi.duraffort@linaro.org>
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import glob
 import os
 import unittest
+from typing import TYPE_CHECKING
 from unittest.mock import patch
 
 from pexpect import EOF as pexpect_eof
 
 from lava_common.exceptions import InfrastructureError, JobError
-from lava_common.yaml import yaml_safe_dump, yaml_safe_load
+from lava_common.yaml import yaml_safe_load
 from lava_dispatcher.action import Action, Pipeline
 from lava_dispatcher.actions.boot import AutoLoginAction, LoginAction
 from lava_dispatcher.actions.boot.qemu import BootQEMUImageAction, CallQemuAction
@@ -21,15 +23,16 @@ from lava_dispatcher.actions.deploy.download import DownloaderAction
 from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.actions.deploy.testdef import InlineRepoAction
 from lava_dispatcher.connections.serial import QemuSession
-from lava_dispatcher.device import NewDevice
-from lava_dispatcher.parser import JobParser
 from lava_dispatcher.utils.filesystem import mkdtemp
 from lava_dispatcher.utils.messages import LinuxKernelMessages
 from tests.lava_dispatcher.test_basic import Factory, LavaDispatcherTestCase
 from tests.lava_dispatcher.test_defs import allow_missing_path, check_missing_path
-from tests.lava_dispatcher.test_messages import create_shell_session_cat_file
+from tests.lava_dispatcher.test_messages import TestMessagesBase
 from tests.utils import DummyLogger, infrastructure_error
 
+if TYPE_CHECKING:
+    from lava_dispatcher.shell import ShellSession
+
 
 class TestKVMSimulation(LavaDispatcherTestCase):
     def test_kvm_simulation(self):
@@ -124,9 +127,7 @@ class TestKVMBasicDeploy(LavaDispatcherTestCase):
             "arch": "amd64",
             "no_kvm": True,
         }  # override to allow unit tests on all types of systems
-        self.job = self.factory.create_job(
-            "qemu01.jinja2", "sample_jobs/kvm.yaml", job_ctx
-        )
+        self.job = self.factory.create_job("qemu01", "sample_jobs/kvm.yaml", job_ctx)
 
     def test_deploy_job(self):
         self.assertEqual(self.job.pipeline.job, self.job)
@@ -137,12 +138,6 @@ class TestKVMBasicDeploy(LavaDispatcherTestCase):
         description_ref = self.pipeline_reference("kvm.yaml", job=self.job)
         self.assertEqual(description_ref, self.job.pipeline.describe())
 
-        overlay = self.job.pipeline.find_action(OverlayAction)
-        self.assertIn(
-            "persistent-nfs-overlay",
-            [action.name for action in overlay.pipeline.actions],
-        )
-
     def test_validate(self):
         try:
             allow_missing_path(
@@ -155,7 +150,7 @@ class TestKVMBasicDeploy(LavaDispatcherTestCase):
 
     def test_available_architectures(self):
         job_ctx = {"arch": "unknown", "no_kvm": True}
-        job = self.factory.create_job("qemu01.jinja2", "sample_jobs/kvm.yaml", job_ctx)
+        job = self.factory.create_job("qemu01", "sample_jobs/kvm.yaml", job_ctx)
         self.assertIsNotNone(job.device["available_architectures"])
         self.assertEqual(job.parameters["context"]["arch"], "unknown")
         self.assertRaises(JobError, job.pipeline.validate_actions)
@@ -163,7 +158,8 @@ class TestKVMBasicDeploy(LavaDispatcherTestCase):
     def test_overlay(self):
         overlay = self.job.pipeline.find_action(OverlayAction)
         self.assertIsNotNone(overlay)
-        # these tests require that lava-dispatcher itself is installed, not just running tests from a git clone
+        # these tests require that lava-dispatcher itself is installed,
+        # not just running tests from a git clone
         self.assertTrue(os.path.exists(overlay.lava_test_dir))
         self.assertIsNot(overlay.lava_test_dir, "/")
         self.assertNotIn("lava_multi_node_test_dir", dir(overlay))
@@ -276,17 +272,6 @@ class TestKVMDeployOverlays(LavaDispatcherTestCase):
         self.assertEqual(description_ref, self.job.pipeline.describe())
 
 
-def prepare_test_connection(failure=False):
-    if failure:
-        logfile = os.path.join(os.path.dirname(__file__), "kernel-login-error.txt")
-    else:
-        logfile = os.path.join(os.path.dirname(__file__), "kernel-1.txt")
-    if not os.path.exists(logfile):
-        raise OSError("Missing test support file.")
-    message_list = LinuxKernelMessages.get_init_prompts()
-    return create_shell_session_cat_file(logfile, message_list)
-
-
 class TestKVMInlineTestDeploy(LavaDispatcherTestCase):
     def setUp(self):
         super().setUp()
@@ -304,35 +289,26 @@ class TestKVMInlineTestDeploy(LavaDispatcherTestCase):
             self.assertEqual([], action.errors)
 
     def test_extra_options(self):
-        (rendered, _) = self.factory.create_device("kvm01.jinja2")
-        device = NewDevice(yaml_safe_load(rendered))
-        kvm_yaml = os.path.join(
-            os.path.dirname(__file__), "sample_jobs/kvm-inline.yaml"
+        def override_qemu_extra(device_dict: dict[str, object]) -> None:
+            device_dict["actions"]["boot"]["methods"]["qemu"]["parameters"]["extra"] = [
+                "-smp",
+                1,
+                "-global",
+                "virtio-blk-device.scsi=off",
+                "device virtio-scsi-device,id=scsi",
+                '--append "console=ttyAMA0 root=/dev/vda rw"',
+            ]
+
+        job = self.factory.create_job(
+            "kvm01",
+            "sample_jobs/kvm-inline.yaml",
+            device_dict_preprocessor=override_qemu_extra,
         )
-        with open(kvm_yaml) as sample_job_data:
-            job_data = yaml_safe_load(sample_job_data)
-        device["actions"]["boot"]["methods"]["qemu"]["parameters"][
-            "extra"
-        ] = yaml_safe_load(
-            """
-                  - -smp
-                  - 1
-                  - -global
-                  - virtio-blk-device.scsi=off
-                  - -device virtio-scsi-device,id=scsi
-                  - --append "console=ttyAMA0 root=/dev/vda rw"
-                  """
-        )
-        self.assertIsInstance(
-            device["actions"]["boot"]["methods"]["qemu"]["parameters"]["extra"][1], int
-        )
-        parser = JobParser()
-        job = parser.parse(yaml_safe_dump(job_data), device, 4212, None, "")
-        job.logger = DummyLogger()
         job.validate()
         qemu = job.pipeline.find_action(CallQemuAction)
         self.assertIsInstance(qemu.sub_command, list)
-        [self.assertIsInstance(item, str) for item in qemu.base_sub_command]
+        for item in qemu.base_sub_command:
+            self.assertIsInstance(item, str)
         self.assertIn("virtio-blk-device.scsi=off", qemu.base_sub_command)
         self.assertIn("1", qemu.base_sub_command)
         self.assertNotIn(1, qemu.base_sub_command)
@@ -413,7 +389,17 @@ class TestKvmConnection(LavaDispatcherTestCase):
             self.assertEqual(call_qemu.session_class, QemuSession)
 
 
-class TestAutoLogin(LavaDispatcherTestCase):
+class TestAutoLogin(TestMessagesBase):
+    def prepare_test_connection(self, failure: bool = False) -> ShellSession:
+        if failure:
+            logfile = os.path.join(os.path.dirname(__file__), "kernel-login-error.txt")
+        else:
+            logfile = os.path.join(os.path.dirname(__file__), "kernel-1.txt")
+        if not os.path.exists(logfile):
+            raise OSError("Missing test support file.")
+        message_list = LinuxKernelMessages.get_init_prompts()
+        return self.create_shell_session_cat_file(logfile, message_list)
+
     def setUp(self):
         super().setUp()
         factory = Factory()
@@ -436,7 +422,7 @@ class TestAutoLogin(LavaDispatcherTestCase):
         autologinaction.populate(autologinaction.parameters)
 
         # initialise the first Connection object, a command line shell
-        shell_connection = prepare_test_connection()
+        shell_connection = self.prepare_test_connection()
         autologinaction.set_namespace_data(
             action="deploy-device-env",
             label="environment",
@@ -516,7 +502,7 @@ class TestAutoLogin(LavaDispatcherTestCase):
         autologinaction.validate()
 
         # initialise the first Connection object, a command line shell
-        shell_connection = prepare_test_connection()
+        shell_connection = self.prepare_test_connection()
 
         # Test the AutoLoginAction directly
         with autologinaction.timeout(None, None) as max_end_time:
@@ -547,7 +533,7 @@ class TestAutoLogin(LavaDispatcherTestCase):
         autologinaction.validate()
 
         # initialise the first Connection object, a command line shell
-        shell_connection = prepare_test_connection()
+        shell_connection = self.prepare_test_connection()
 
         # Test the AutoLoginAction directly
         with autologinaction.timeout(None, None) as max_end_time:
@@ -571,7 +557,7 @@ class TestAutoLogin(LavaDispatcherTestCase):
         loginaction.parameters = autologinaction.parameters
 
         # initialise the first Connection object, a command line shell
-        shell_connection = prepare_test_connection(True)
+        shell_connection = self.prepare_test_connection(True)
 
         # Test the AutoLoginAction directly
         with self.assertRaisesRegex(
@@ -659,7 +645,7 @@ class TestQemuNFS(LavaDispatcherTestCase):
     def setUp(self):
         super().setUp()
         factory = Factory()
-        self.job = factory.create_job("kvm02.jinja2", "sample_jobs/qemu-nfs.yaml")
+        self.job = factory.create_job("kvm02", "sample_jobs/qemu-nfs.yaml")
         self.job.logger = DummyLogger()
 
     @unittest.skipIf(
