@@ -8,7 +8,7 @@ from __future__ import annotations
 import os
 import time
 from shlex import quote as shlex_quote
-from unittest.mock import MagicMock
+from unittest.mock import MagicMock, patch
 
 from pexpect import EOF as pexpect_eof
 
@@ -20,23 +20,35 @@ from lava_dispatcher.utils.messages import LinuxKernelMessages
 from tests.lava_dispatcher.test_basic import LavaDispatcherTestCase
 
 
-def create_shell_session_cat_file(
-    file_to_cat: str, pexpect_patterns: list[str]
-) -> ShellSession:
-    shell_command = ShellCommand(
-        f"cat {shlex_quote(file_to_cat)}",
-        Timeout("test-cat-command", None),
-        logger=MagicMock(),
-    )
+class TestMessagesBase(LavaDispatcherTestCase):
+    def create_shell_session_cat_file(
+        self,
+        file_to_cat: str,
+        pexpect_patterns: list[str],
+        infinite_stream: bool = False,
+    ) -> ShellSession:
+        cmd_str = f"cat {shlex_quote(file_to_cat)}"
 
-    shell_session = ShellSession(MagicMock(), shell_command)
-    pexpect_patterns.append(pexpect_eof)
-    shell_session.prompt_str = pexpect_patterns
+        if infinite_stream:
+            cmd_str += " -"
+        else:
+            pexpect_patterns.append(pexpect_eof)
 
-    return shell_session
+        shell_command = ShellCommand(
+            cmd_str,
+            Timeout("test-cat-command", None),
+            logger=MagicMock(),
+        )
+        # Force cleanup otherwise race conditions might happen
+        self.addCleanup(shell_command.close, force=True)
+
+        shell_session = ShellSession(shell_command)
+        shell_session.prompt_str = pexpect_patterns
+
+        return shell_session
 
 
-class TestBootMessages(LavaDispatcherTestCase):
+class TestBootMessages(TestMessagesBase):
     def setUp(self):
         super().setUp()
         self.max_end_time = time.monotonic() + 30
@@ -50,7 +62,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
         self.assertIsNotNone(message_list)
-        connection = create_shell_session_cat_file(logfile, message_list)
+        connection = self.create_shell_session_cat_file(logfile, message_list)
         with self.assertRaises(JobError):
             LinuxKernelMessages.parse_failures(
                 connection,
@@ -63,7 +75,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         logfile = os.path.join(os.path.dirname(__file__), "kernel-1.txt")
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
-        connection = create_shell_session_cat_file(logfile, message_list)
+        connection = self.create_shell_session_cat_file(logfile, message_list)
         results = LinuxKernelMessages.parse_failures(
             connection,
             action=Action(self.create_job_mock()),
@@ -77,7 +89,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
         self.assertIsNotNone(message_list)
-        connection = create_shell_session_cat_file(logfile, message_list)
+        connection = self.create_shell_session_cat_file(logfile, message_list)
         results = LinuxKernelMessages.parse_failures(
             connection,
             action=Action(self.create_job_mock()),
@@ -108,7 +120,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
         self.assertIsNotNone(message_list)
-        connection = create_shell_session_cat_file(logfile, message_list)
+        connection = self.create_shell_session_cat_file(logfile, message_list)
         with self.assertRaises(JobError):
             LinuxKernelMessages.parse_failures(
                 connection,
@@ -122,7 +134,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
         self.assertIsNotNone(message_list)
-        create_shell_session_cat_file(logfile, message_list)
+        self.create_shell_session_cat_file(logfile, message_list)
         self.assertRaises(InfrastructureError)
 
     def test_kernel_kasan(self):
@@ -130,7 +142,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
         self.assertIsNotNone(message_list)
-        connection = create_shell_session_cat_file(logfile, message_list)
+        connection = self.create_shell_session_cat_file(logfile, message_list)
         results = LinuxKernelMessages.parse_failures(
             connection,
             action=Action(self.create_job_mock()),
@@ -155,7 +167,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
         self.assertIsNotNone(message_list)
-        connection = create_shell_session_cat_file(logfile, message_list)
+        connection = self.create_shell_session_cat_file(logfile, message_list)
         results = LinuxKernelMessages.parse_failures(
             connection,
             action=Action(self.create_job_mock()),
@@ -180,7 +192,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
         self.assertIsNotNone(message_list)
-        connection = create_shell_session_cat_file(logfile, message_list)
+        connection = self.create_shell_session_cat_file(logfile, message_list)
         results = LinuxKernelMessages.parse_failures(
             connection,
             action=Action(self.create_job_mock()),
@@ -197,7 +209,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
         self.assertIsNotNone(message_list)
-        connection = create_shell_session_cat_file(logfile, message_list)
+        connection = self.create_shell_session_cat_file(logfile, message_list)
         results = LinuxKernelMessages.parse_failures(
             connection,
             action=Action(self.create_job_mock()),
@@ -222,7 +234,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
         self.assertIsNotNone(message_list)
-        connection = create_shell_session_cat_file(logfile, message_list)
+        connection = self.create_shell_session_cat_file(logfile, message_list)
         results = LinuxKernelMessages.parse_failures(
             connection,
             action=Action(self.create_job_mock()),
@@ -263,7 +275,7 @@ class TestBootMessages(LavaDispatcherTestCase):
         self.assertTrue(os.path.exists(logfile))
         message_list = LinuxKernelMessages.get_init_prompts()
         self.assertIsNotNone(message_list)
-        connection = create_shell_session_cat_file(logfile, message_list)
+        connection = self.create_shell_session_cat_file(logfile, message_list)
         results = LinuxKernelMessages.parse_failures(
             connection,
             action=Action(self.create_job_mock()),
@@ -271,3 +283,38 @@ class TestBootMessages(LavaDispatcherTestCase):
             fail_msg="",
         )
         self.assertEqual(len(results), 134)
+
+    def test_kernel_panic_and_reset_overlapped(self):
+        logfile = os.path.join(
+            os.path.dirname(__file__), "kernel-panic-and-reset-overlapped.txt"
+        )
+        self.assertTrue(os.path.exists(logfile))
+        message_list = LinuxKernelMessages.get_init_prompts()
+        self.assertIsNotNone(message_list)
+        connection = self.create_shell_session_cat_file(
+            logfile, message_list, infinite_stream=True
+        )
+        action = Action(self.create_job_mock())
+
+        start_time = time.monotonic()
+        end_time = start_time + 10
+
+        def monotonic_timeline():
+            yield start_time  # start = time.monotonic()
+            yield start_time + 0.1  # max_end_time - time.monotonic()
+            yield start_time + 0.1  # connection.wait(max_end_time)
+
+            yield end_time  # sub_index = connection.wait()
+            raise ValueError("Timeline misconfigured")
+
+        timeline_iter = monotonic_timeline()
+
+        with self.assertRaisesRegex(JobError, "^Kernel panic - not syncing$"), patch(
+            "time.monotonic", side_effect=lambda: next(timeline_iter)
+        ):
+            LinuxKernelMessages.parse_failures(
+                connection,
+                action=action,
+                max_end_time=end_time,
+                fail_msg="",
+            )
