@@ -8,8 +8,8 @@ from tests.lava_dispatcher.test_basic import Factory, LavaDispatcherTestCase
 
 
 class N1sdpFactory(Factory):
-    def create_n1sdp_job(self, filename):  # pylint: disable=no-self-use
-        return self.create_job("n1sdp-01.jinja2", filename)
+    def create_n1sdp_job(self, filename):
+        return self.create_job("n1sdp-01", filename)
 
 
 class TestN1sdp(LavaDispatcherTestCase):
