@@ -3,11 +3,13 @@
 # Author: Milosz Wasilewski <milosz.wasilewski@linaro.org>
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
+from __future__ import annotations
 
 import csv
 import json
 import xml.etree.ElementTree as ET
 from datetime import timedelta
+from typing import TYPE_CHECKING
 
 import pytest
 from django.conf import settings
@@ -16,6 +18,7 @@ from django.contrib.auth.models import Group, User
 from django.http import FileResponse, HttpResponse
 from django.urls import reverse
 from django.utils import timezone
+from rest_framework import status
 from rest_framework.test import APIClient
 
 from lava_common.version import __version__
@@ -28,6 +31,7 @@ from lava_scheduler_app.models import (
     DeviceType,
     GroupDevicePermission,
     GroupDeviceTypePermission,
+    RemoteArtifactsAuth,
     Tag,
     TestJob,
     Worker,
@@ -35,6 +39,9 @@ from lava_scheduler_app.models import (
 from lava_server.files import File
 from linaro_django_xmlrpc.models import AuthToken
 
+if TYPE_CHECKING:
+    from typing import Any
+
 EXAMPLE_JOB = """
 job_name: test
 visibility: public
@@ -117,6 +124,7 @@ class TestRestApi:
         # create second token to check whether authentication still works
         AuthToken.objects.create(user=self.user, secret="userkey2")  # nosec - unittest
 
+        self.anonymousclient = APIClient()
         self.userclient = APIClient()
         self.userclient.credentials(HTTP_AUTHORIZATION="Token " + self.usertoken.secret)
         self.userclient_no_token = APIClient()
@@ -250,7 +258,15 @@ class TestRestApi:
         self.tag1 = Tag.objects.create(name="tag1", description="description1")
         self.tag2 = Tag.objects.create(name="tag2", description="description2")
 
-    def hit(self, client, url):
+        # create user1 remote artifact tokens
+        RemoteArtifactsAuth.objects.create(
+            user=self.user, name="token1", token="value1"
+        )
+        RemoteArtifactsAuth.objects.create(
+            user=self.user, name="token2", token="value2"
+        )
+
+    def hit(self, client: APIClient, url: str) -> dict[str, Any] | str:
         response = client.get(url)
         assert response.status_code == 200  # nosec - unit test support
 
@@ -501,10 +517,7 @@ class TestRestApi:
             f"{reverse('api-root', args=(self.version,))}"
             f"jobs/{self.public_testjob1.id}/csv/"
         )
-        data = self.hit(
-            self.userclient,
-            csv_url,
-        )
+        data = self.hit(self.userclient, csv_url)
         csv_data = csv.reader(data.splitlines())
         assert list(csv_data)[1][0] == str(
             self.public_testjob1.id
@@ -660,6 +673,15 @@ ok 2 bar
         msg = json.loads(response.content)
         assert msg["message"] == "Job valid."
 
+    def test_testjob_validate_invalid(self):
+        response = self.userclient.post(
+            reverse("api-root", args=[self.version]) + "jobs/validate/",
+            {"definition": "job_name:"},
+        )
+        assert response.status_code == 200  # nosec - unit test support
+        msg = json.loads(response.content)
+        assert msg["message"] == "Job invalid: expected str @ ['job_name']"
+
     def test_testjob_validate_testdef(self):
         "Test validating valid test definition."
         testdef = """
@@ -710,7 +732,7 @@ ok 2 bar
         msg = json.loads(response.content)
         assert (
             msg["message"]
-            == "Test defnition invalid: required key not provided @ data['metadata']['name']"
+            == "Test definition invalid: required key not provided @ data['metadata']['name']"
         )
 
     def test_testjob_validate_testdef_wrong_key_type(self):
@@ -729,7 +751,7 @@ ok 2 bar
         msg = json.loads(response.content)
         assert (
             msg["message"]
-            == "Test defnition invalid: expected dict for dictionary value @ data['parameters']"
+            == "Test definition invalid: expected dict for dictionary value @ data['parameters']"
         )
 
     def test_testjobs_filters(self):
@@ -1602,6 +1624,14 @@ ok 2 bar
         msg = json.loads(response.content)
         assert msg["message"] == "Job cancel signal sent."  # nosec - unit test support
 
+    def test_cancel_404(self):
+        response = self.adminclient.get(
+            reverse("api-root", args=[self.version]) + "jobs/52022/cancel/"
+        )
+        assert response.status_code == 404
+        msg = json.loads(response.content)
+        assert msg["detail"] == "Not found."
+
     def test_batch_cancel(self, mocker):
         mocker.patch("lava_scheduler_app.models.TestJob.cancel")
         response = self.adminclient.post(
@@ -1876,15 +1906,93 @@ ok 2 bar
         )
         assert ret.status_code == 200
 
+    def test_remote_artifact_tokens_list_unauthorized(self, db):
+        ret = self.anonymousclient.get(
+            reverse("api-root", args=(self.version,)) + "remote-artifact-tokens/"
+        )
+        assert ret.status_code == status.HTTP_403_FORBIDDEN
+
+    def test_remote_artifact_tokens_list(self, db):
+        data = self.hit(
+            self.userclient,
+            reverse("api-root", args=(self.version,)) + "remote-artifact-tokens/",
+        )
+        assert isinstance(data, dict)
+        assert data["results"] == [
+            {"name": "token1", "token": "value1"},
+            {"name": "token2", "token": "value2"},
+        ]
+
+    def test_remote_artifact_tokens_retrieve_unauthorized(self, db):
+        ret = self.anonymousclient.get(
+            reverse("api-root", args=(self.version,)) + "remote-artifact-tokens/token1/"
+        )
+        assert ret.status_code == status.HTTP_403_FORBIDDEN
+
+    def test_remote_artifact_tokens_retrieve(self, db):
+        data = self.hit(
+            self.userclient,
+            reverse("api-root", args=(self.version,))
+            + "remote-artifact-tokens/token1/",
+        )
+        assert data == {"name": "token1", "token": "value1"}
+
+    def test_remote_artifact_tokens_create_unauthorized(self, db):
+        ret = self.anonymousclient.post(
+            reverse("api-root", args=[self.version]) + "remote-artifact-tokens/",
+            {"name": "token3", "token": "value3"},
+        )
+        assert ret.status_code == status.HTTP_403_FORBIDDEN
+
+    def test_remote_artifact_tokens_create(self, db):
+        ret = self.userclient.post(
+            reverse("api-root", args=[self.version]) + "remote-artifact-tokens/",
+            {"name": "token3", "token": "value3"},
+        )
+        assert ret.status_code == status.HTTP_201_CREATED
+
+    def test_remote_artifact_tokens_update_unauthorized(self, db):
+        ret = self.anonymousclient.put(
+            reverse("api-root", args=[self.version]) + "remote-artifact-tokens/token1/",
+            {"name": "token1", "token": "value11"},
+        )
+        assert ret.status_code == status.HTTP_403_FORBIDDEN
+
+    def test_remote_artifact_tokens_update_value(self, db):
+        ret = self.userclient.put(
+            reverse("api-root", args=[self.version]) + "remote-artifact-tokens/token1/",
+            {"name": "token1", "token": "value11"},
+        )
+        assert ret.status_code == status.HTTP_200_OK
+
+    def test_remote_artifact_tokens_update_name(self, db):
+        ret = self.userclient.put(
+            reverse("api-root", args=[self.version]) + "remote-artifact-tokens/token1/",
+            {"name": "token11", "token": "value1"},
+        )
+        assert ret.status_code == status.HTTP_200_OK
+
+    def test_remote_artifact_tokens_delete_unauthorized(self, db):
+        ret = self.anonymousclient.delete(
+            reverse("api-root", args=[self.version]) + "remote-artifact-tokens/token1/"
+        )
+        assert ret.status_code == status.HTTP_403_FORBIDDEN
+
+    def test_remote_artifact_tokens_delete(self, db):
+        ret = self.userclient.delete(
+            reverse("api-root", args=[self.version]) + "remote-artifact-tokens/token1/"
+        )
+        assert ret.status_code == status.HTTP_204_NO_CONTENT
+
 
 def test_serializers_partial():
     # make sure TestJob, Worker and Device serializers are always instantiated
     # with partial argument set to True
     worker_serializer = serializers.WorkerSerializer()
-    assert worker_serializer.partial == True
+    assert worker_serializer.partial is True
 
     testjob_serializer = serializers.TestJobSerializer()
-    assert testjob_serializer.partial == True
+    assert testjob_serializer.partial is True
 
     device_serializer = serializers.DeviceSerializer()
-    assert device_serializer.partial == True
+    assert device_serializer.partial is True
