@@ -10,6 +10,7 @@ import xmlrpc.client
 from io import BytesIO as StringIO
 
 import pytest
+from django.contrib.admin.models import LogEntry
 from django.contrib.auth.models import Group, Permission, User
 from django.test.client import Client
 from django.utils import timezone
@@ -23,6 +24,7 @@ from lava_scheduler_app.models import (
     DeviceType,
     GroupDevicePermission,
     GroupDeviceTypePermission,
+    RemoteArtifactsAuth,
     Tag,
     TestJob,
     Worker,
@@ -263,6 +265,50 @@ class TestSchedulerAPI(TestCaseWithFactory):
         self.assertEqual(f.faultString, "Invalid health 'Completed'")
 
 
+@pytest.mark.usefixtures("setup")
+class TestNotification(unittest.TestCase):
+    @pytest.mark.django_db
+    def test_valid_user(self):
+        good_submission = """
+notify:
+  recipients:
+  - to:
+     method: email
+     user: admin
+  criteria:
+    status: complete
+        """
+        validate_yaml(yaml_safe_load(good_submission))
+
+    @pytest.mark.django_db
+    def test_invalid_user(self):
+        bad_submission = """
+notify:
+  recipients:
+  - to:
+     method: email
+     user: foo
+  criteria:
+    status: complete
+        """
+        self.assertRaises(
+            SubmissionException, validate_yaml, yaml_safe_load(bad_submission)
+        )
+
+    @pytest.mark.django_db
+    def test_device_owner(self):
+        good_submission = """
+notify:
+  recipients:
+  - to:
+     method: email
+     user: "{LAVA_DEVICE_OWNER}"
+  criteria:
+    status: complete
+        """
+        validate_yaml(yaml_safe_load(good_submission))
+
+
 class TestVoluptuous(unittest.TestCase):
     def test_breakage_detection(self):
         bad_submission = """
@@ -748,11 +794,11 @@ def test_devices_add(setup):
 
     Worker.objects.create(hostname="worker01")
     server("admin", "admin").scheduler.devices.add("black01", "black", "worker01")
-    Device.objects.count() == 1
-    Device.objects.all()[0].hostname == "black01"
-    Device.objects.all()[0].device_type.name == "black"
-    Device.objects.all()[0].worker_host.hostname == "worker01"
-    Device.objects.all()[0].get_health_display() == "Unknown"
+    assert Device.objects.count() == 1
+    assert Device.objects.all()[0].hostname == "black01"
+    assert Device.objects.all()[0].device_type.name == "black"
+    assert Device.objects.all()[0].worker_host.hostname == "worker01"
+    assert Device.objects.all()[0].get_health_display() == "Unknown"
 
     # 2. test description, health
     server("admin", "admin").scheduler.devices.add(
@@ -766,12 +812,17 @@ def test_devices_add(setup):
         "second device",
     )
 
-    Device.objects.count() == 2
+    assert Device.objects.count() == 2
     # Should be ordered as black01, black02
-    Device.objects.all().order_by("hostname")[1].hostname == "black02"
-    Device.objects.all().order_by("hostname")[1].device_type.name == "black"
-    Device.objects.all().order_by("hostname")[1].worker_host.hostname == "worker01"
-    Device.objects.all().order_by("hostname")[1].get_health_display() == "Maintenance"
+    assert Device.objects.all().order_by("hostname")[1].hostname == "black02"
+    assert Device.objects.all().order_by("hostname")[1].device_type.name == "black"
+    assert (
+        Device.objects.all().order_by("hostname")[1].worker_host.hostname == "worker01"
+    )
+    assert (
+        Device.objects.all().order_by("hostname")[1].get_health_display()
+        == "Maintenance"
+    )
 
     # 3. wrong health
     with pytest.raises(xmlrpc.client.Fault) as exc:
@@ -1193,6 +1244,19 @@ def test_devices_update(setup):
     device.refresh_from_db()
     assert device.description == "hello"  # nosec
 
+    # 7. update health reason escaped
+    reason = "<script>"
+    server("admin", "admin").scheduler.devices.update(
+        "black01", None, None, None, None, "GOOD", None, None, reason
+    )
+    device.refresh_from_db()
+    assert device.health == Device.HEALTH_GOOD
+    log_entry = LogEntry.objects.filter(object_id=device.hostname).first()
+    assert log_entry is not None
+    msg = log_entry.get_change_message()
+    assert reason not in msg
+    assert "&lt;script&gt;" in msg
+
 
 @pytest.mark.django_db
 def test_devices_tags_add(setup):
@@ -1665,7 +1729,7 @@ def test_device_types_update(setup):
         "qemu", None, None, None, None, None, None
     )
     dt = DeviceType.objects.get(name="qemu")
-    assert dt.description == None  # nosec
+    assert dt.description is None  # nosec
 
     server("admin", "admin").scheduler.device_types.update(
         "qemu", "emulated", True, None, 12, "jobs", True
@@ -2512,3 +2576,99 @@ def test_workers_update(setup):
         server("admin", "admin").scheduler.workers.update("example.com", None, "wrong")
     assert exc.value.faultCode == 400  # nosec
     assert exc.value.faultString == "Invalid health: wrong"  # nosec
+
+
+@pytest.mark.django_db
+class TestRemoteArtifactTokens:
+    @pytest.fixture
+    def user(self):
+        return User.objects.create_user(username="user", password="user")
+
+    @pytest.fixture
+    def tokens(self, user):
+        RemoteArtifactsAuth.objects.create(user=user, name="n1", token="v1")
+        RemoteArtifactsAuth.objects.create(user=user, name="n2", token="v2")
+
+    def test_list_requires_auth(self):
+        with pytest.raises(xmlrpc.client.Fault) as exc:
+            server().scheduler.remote_artifact_tokens.list()
+        assert exc.value.faultCode == 401
+        assert (
+            exc.value.faultString
+            == "Authentication with user and token required for this API."
+        )
+
+    def test_list_empty(self, user):
+        assert server("user", "user").scheduler.remote_artifact_tokens.list() == []
+
+    def test_list(self, tokens):
+        assert server("user", "user").scheduler.remote_artifact_tokens.list() == [
+            {"name": "n1", "token": "v1"},
+            {"name": "n2", "token": "v2"},
+        ]
+
+    def test_show_requires_auth(self):
+        with pytest.raises(xmlrpc.client.Fault) as exc:
+            server().scheduler.remote_artifact_tokens.show("n1")
+        assert exc.value.faultCode == 401
+        assert (
+            exc.value.faultString
+            == "Authentication with user and token required for this API."
+        )
+
+    def test_show_not_found(self, setup):
+        with pytest.raises(xmlrpc.client.Fault) as exc:
+            server("user", "user").scheduler.remote_artifact_tokens.show("n1")
+        assert exc.value.faultCode == 404
+        assert exc.value.faultString == "Token 'n1' was not found."
+
+    def test_show(self, tokens):
+        srv = server("user", "user")
+        assert srv.scheduler.remote_artifact_tokens.show("n1") == "v1"
+        assert srv.scheduler.remote_artifact_tokens.show("n2") == "v2"
+
+    def test_add_requires_auth(self):
+        with pytest.raises(xmlrpc.client.Fault) as exc:
+            server().scheduler.remote_artifact_tokens.add("n1", "v1")
+        assert exc.value.faultCode == 401
+        assert (
+            exc.value.faultString
+            == "Authentication with user and token required for this API."
+        )
+
+    def test_add(self, setup):
+        server("user", "user").scheduler.remote_artifact_tokens.add("n1", "v1")
+
+        user = User.objects.get(username="user")
+        token = RemoteArtifactsAuth.objects.filter(user=user).get(name="n1")
+        assert token.name == "n1"
+        assert token.token == "v1"
+
+    def test_add_exists(self, tokens):
+        with pytest.raises(xmlrpc.client.Fault) as exc:
+            server("user", "user").scheduler.remote_artifact_tokens.add("n1", "v1")
+        assert exc.value.faultCode == 400
+        assert exc.value.faultString == "Bad request: token already exists?"
+
+    def test_delete_requires_auth(self):
+        with pytest.raises(xmlrpc.client.Fault) as exc:
+            server().scheduler.remote_artifact_tokens.delete("n1")
+        assert (
+            exc.value.faultString
+            == "Authentication with user and token required for this API."
+        )
+
+    def test_delete_not_found(self, setup):
+        with pytest.raises(xmlrpc.client.Fault) as exc:
+            server("user", "user").scheduler.remote_artifact_tokens.delete("n1")
+        assert exc.value.faultCode == 404
+        assert exc.value.faultString == "Token 'n1' was not found."
+
+    def test_delete(self, tokens):
+        srv = server("user", "user")
+        srv.scheduler.remote_artifact_tokens.delete("n1")
+        srv.scheduler.remote_artifact_tokens.delete("n2")
+
+        user = User.objects.get(username="user")
+        assert not RemoteArtifactsAuth.objects.filter(user=user, name="n1").exists()
+        assert not RemoteArtifactsAuth.objects.filter(user=user, name="n2").exists()
