@@ -1,6 +1,12 @@
+# Copyright (C) 2019 Linaro
+#
+# Author: Antonio Terceiro <antonio.terceiro@linaro.org>
+#
+# SPDX-License-Identifier: GPL-2.0-or-later
+
 import os
 
-from lava_common.compat import yaml_safe_load
+from lava_common.yaml import yaml_safe_load
 from tests.lava_scheduler_app.test_pipeline import YamlFactory
 from tests.lava_scheduler_app.test_submission import TestCaseWithFactory
 
@@ -13,7 +19,7 @@ class YamlMenuFactory(YamlFactory):
         sample_job_file = os.path.join(
             os.path.dirname(__file__), "sample_jobs", "mustang-menu-ramdisk.yaml"
         )
-        with open(sample_job_file, "r") as test_support:
+        with open(sample_job_file) as test_support:
             data = yaml_safe_load(test_support)
         data.update(kw)
         return data
