@@ -225,7 +225,7 @@ class PipelineDeviceTags(TestCaseWithFactory):
             ),
             user,
         )
-        self.assertEqual({tag for tag in job.tags.all()}, set(tag_list))
+        self.assertEqual(set(job.tags.all()), set(tag_list))
 
 
 class TestPipelineSubmit(TestCaseWithFactory):
@@ -635,7 +635,7 @@ class TestYamlMultinode(TestCaseWithFactory):
 
     def test_multinode_tags(self):
         Tag.objects.all().delete()
-        self.factory.ensure_tag("tap"),
+        self.factory.ensure_tag("tap")
         self.factory.ensure_tag("virtio")
         with open(
             os.path.join(
@@ -678,7 +678,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         with open(
             os.path.join(
                 os.path.dirname(__file__), "sample_jobs", "bbb-qemu-multinode.yaml"
-            ),
+            )
         ) as f:
             submission = yaml_safe_load(f)
 
@@ -721,22 +721,22 @@ class TestYamlMultinode(TestCaseWithFactory):
                 "mirror": "http://ftp.us.debian.org/debian/",
             }
         }
-        for role, _ in jobs.items():
-            if role == "server":
-                self.assertNotIn("lava-lxc", jobs[role][0]["protocols"])
-            elif role == "client":
-                self.assertIn("lava-lxc", jobs[role][0]["protocols"])
+        for role_name, role_dicts in jobs.items():
+            if role_name == "server":
+                self.assertNotIn("lava-lxc", role_dicts[0]["protocols"])
+            elif role_name == "client":
+                self.assertIn("lava-lxc", role_dicts[0]["protocols"])
                 self.assertEqual(
-                    jobs[role][0]["protocols"]["lava-lxc"], protocol_data["lava-lxc"]
+                    role_dicts[0]["protocols"]["lava-lxc"], protocol_data["lava-lxc"]
                 )
             else:
-                self.fail("Unrecognised role: %s" % role)
+                self.fail("Unrecognised role: %s" % role_name)
 
     def test_multinode_hikey(self):
         with open(
             os.path.join(
                 os.path.dirname(__file__), "sample_jobs", "hikey_multinode.yaml"
-            ),
+            )
         ) as f:
             submission = yaml_safe_load(f)
         target_group = "arbitrary-group-id"  # for unit tests only
@@ -761,21 +761,21 @@ class TestYamlMultinode(TestCaseWithFactory):
                 "template": "debian",
             }
         }
-        for role, _ in jobs.items():
-            if role == "server":
-                self.assertIn("lava-lxc", jobs[role][0]["protocols"])
+        for role_name, role_dicts in jobs.items():
+            if role_name == "server":
+                self.assertIn("lava-lxc", role_dicts[0]["protocols"])
                 self.assertEqual(
-                    jobs[role][0]["protocols"]["lava-lxc"],
+                    role_dicts[0]["protocols"]["lava-lxc"],
                     server_protocol_data["lava-lxc"],
                 )
-            elif role == "client":
-                self.assertIn("lava-lxc", jobs[role][0]["protocols"])
+            elif role_name == "client":
+                self.assertIn("lava-lxc", role_dicts[0]["protocols"])
                 self.assertEqual(
-                    jobs[role][0]["protocols"]["lava-lxc"],
+                    role_dicts[0]["protocols"]["lava-lxc"],
                     client_protocol_data["lava-lxc"],
                 )
             else:
-                self.fail("Unrecognised role: %s" % role)
+                self.fail("Unrecognised role: %s" % role_name)
 
     def test_multinode_nexus4(self):
         with open(
@@ -795,9 +795,9 @@ class TestYamlMultinode(TestCaseWithFactory):
                 "distribution": "debian",
             }
         }
-        for role, _ in jobs.items():
-            if role == "device":
-                for job in jobs[role]:
+        for role_name, role_dicts in jobs.items():
+            if role_name == "device":
+                for job in role_dicts:
                     self.assertIn("lava-lxc", job["protocols"])
                     self.assertEqual(
                         job["protocols"]["lava-lxc"], device_protocol_data["lava-lxc"]
@@ -805,7 +805,7 @@ class TestYamlMultinode(TestCaseWithFactory):
                     self.assertIn("reboot_to_fastboot", job)
                     self.assertEqual(job["reboot_to_fastboot"], False)
             else:
-                self.fail("Unrecognised role: %s" % role)
+                self.fail("Unrecognised role: %s" % role_name)
 
     def test_multinode_protocols(self):
         user = self.factory.make_user()
@@ -1057,9 +1057,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         self.factory.make_device(device_type, "fakeqemu3")
         self.factory.make_device(device_type, "fakeqemu4")
         with open(
-            os.path.join(
-                os.path.dirname(__file__), "sample_jobs", "kvm-multinode.yaml"
-            ),
+            os.path.join(os.path.dirname(__file__), "sample_jobs", "kvm-multinode.yaml")
         ) as f:
             submission = yaml_safe_load(f)
         role_list = submission["protocols"][MultinodeProtocol.name]["roles"]
@@ -1225,7 +1223,6 @@ class VlanInterfaces(TestCaseWithFactory):
         self.assertIn("10M", client_job["protocols"]["lava-vland"]["vlan_one"]["tags"])
         self.assertIn("vlan_two", server_job["protocols"]["lava-vland"])
         self.assertIn("100M", server_job["protocols"]["lava-vland"]["vlan_two"]["tags"])
-        client_job["protocols"]["lava-vland"]["vlan_one"]
         bbb_01 = Device.objects.get(hostname="bbb-01")
         client_config = bbb_01.load_configuration()
         self.assertIn("eth0", client_config["parameters"]["interfaces"])
