@@ -8,6 +8,7 @@ from json import loads as json_loads
 from pathlib import Path
 
 import pytest
+from django.contrib.admin.models import LogEntry
 from django.contrib.auth.models import Group, User
 from django.contrib.contenttypes.models import ContentType
 from django.http import Http404
@@ -28,6 +29,7 @@ from lava_scheduler_app.models import (
     Worker,
 )
 from lava_scheduler_app.views import (
+    InPlaceTokenUpdater,
     device_report_data,
     job_report_data,
     type_report_data,
@@ -207,7 +209,7 @@ def test_devices(client, setup):
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/alldevices.html"  # nosec
     assert len(ret.context["devices_table"].data) == 4  # nosec
-    hostnames = set([x.hostname for x in ret.context["devices_table"].data])
+    hostnames = {x.hostname for x in ret.context["devices_table"].data}
     assert hostnames == set(
         ["qemu_:')-,;~", "qemu01", "juno-uboot-01", "juno-uefi-01"]
     )  # nosec
@@ -219,7 +221,7 @@ def test_devices_active(client, setup):
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/activedevices.html"  # nosec
     assert len(ret.context["active_devices_table"].data) == 4  # nosec
-    hostnames = set([x.hostname for x in ret.context["active_devices_table"].data])
+    hostnames = {x.hostname for x in ret.context["active_devices_table"].data}
     assert hostnames == set(
         ["qemu_:')-,;~", "qemu01", "juno-uboot-01", "juno-uefi-01"]
     )  # nosec
@@ -312,7 +314,7 @@ def test_devices_maintenance(client, setup):
         ret.templates[0].name == "lava_scheduler_app/maintenance_devices.html"
     )
     assert len(ret.context["maintenance_devices_table"].data) == 2  # nosec
-    hostnames = set([x.hostname for x in ret.context["maintenance_devices_table"].data])
+    hostnames = {x.hostname for x in ret.context["maintenance_devices_table"].data}
     assert hostnames == set(["qemu_:')-,;~", "qemu01"])  # nosec
 
 
@@ -789,6 +791,7 @@ def test_device_detail(client, setup):
 
 @pytest.mark.django_db
 def test_failure_report(client, setup):
+    assert client.login(username="tester", password="tester") is True
     ret = client.get(reverse("lava.scheduler.failure_report"))
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/failure_report.html"  # nosec
@@ -802,6 +805,14 @@ def test_failure_report(client, setup):
     assert ret.context["device"] == "juno-uboot-01"  # nosec
 
 
+@pytest.mark.django_db
+def test_failure_reports_anonymous(client, setup):
+    url = reverse("lava.scheduler.failure_report")
+    ret = client.get(url)
+    assert ret.status_code == 302
+    assert ret.url == f'{reverse("login")}?next={url}'
+
+
 @pytest.mark.django_db
 def test_health_job_list(client, setup):
     ret = client.get(reverse("lava.scheduler.labhealth.detail", args=["qemu01"]))
@@ -934,12 +945,21 @@ def test_jobs_my_error(client, setup):
 
 @pytest.mark.django_db
 def test_job_errors(client, setup):
+    assert client.login(username="tester", password="tester") is True
     ret = client.get(reverse("lava.scheduler.job.errors"))
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/job_errors.html"  # nosec
     assert len(ret.context["job_errors_table"].data) == 0  # nosec
 
 
+@pytest.mark.django_db
+def test_job_errors_anonymous(client, setup):
+    url = reverse("lava.scheduler.job.errors")
+    ret = client.get(url)
+    assert ret.status_code == 302
+    assert ret.url == f'{reverse("login")}?next={url}'
+
+
 @pytest.mark.django_db
 def test_job_detail(client, setup):
     ret = client.get(
@@ -950,7 +970,10 @@ def test_job_detail(client, setup):
     )
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/job.html"  # nosec
-    assert ret.context["log_data"] == []  # nosec
+    assert (
+        ret.context["log_data"]
+        == '<script id="logs-initial" type="application/json">[]</script>'
+    )  # nosec
 
 
 @pytest.mark.django_db
@@ -998,19 +1021,19 @@ def test_job_submit(client, setup):
     ret = client.get(reverse("lava.scheduler.job.submit"))
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/job_submit.html"  # nosec
-    assert ret.context["is_authorized"] == False  # nosec
+    assert ret.context["is_authorized"] is False  # nosec
     # Anonymous user POST
     ret = client.post(reverse("lava.scheduler.job.submit"), {"definition-input": ""})
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/job_submit.html"  # nosec
-    assert ret.context["is_authorized"] == False  # nosec
+    assert ret.context["is_authorized"] is False  # nosec
 
     # Logged-user GET
     assert client.login(username="tester", password="tester") is True  # nosec
     ret = client.get(reverse("lava.scheduler.job.submit"))
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/job_submit.html"  # nosec
-    assert ret.context["is_authorized"] == True  # nosec
+    assert ret.context["is_authorized"] is True  # nosec
 
     # Logged-user POST as JSON
     ret = client.post(
@@ -1065,7 +1088,7 @@ def test_lab_health(client, setup):
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/labhealth.html"  # nosec
     assert len(ret.context["device_health_table"].data) == 4  # nosec
-    hostnames = set([x["hostname"] for x in ret.context["device_health_table"].data])
+    hostnames = {x["hostname"] for x in ret.context["device_health_table"].data}
     assert hostnames == set(
         ["qemu_:')-,;~", "qemu01", "juno-uboot-01", "juno-uefi-01"]
     )  # nosec
@@ -1073,6 +1096,7 @@ def test_lab_health(client, setup):
 
 @pytest.mark.django_db
 def test_reports(client, setup):
+    assert client.login(username="tester", password="tester") is True
     ret = client.get(reverse("lava.scheduler.reports"))
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/reports.html"  # nosec
@@ -1082,13 +1106,21 @@ def test_reports(client, setup):
     assert len(ret.context["job_day_report"]) == 7  # nosec
 
 
+@pytest.mark.django_db
+def test_reports_anonymous(client, setup):
+    url = reverse("lava.scheduler.reports")
+    ret = client.get(url)
+    assert ret.status_code == 302
+    assert ret.url == f'{reverse("login")}?next={url}'
+
+
 @pytest.mark.django_db
 def test_workers(client, setup):
     ret = client.get(reverse("lava.scheduler.workers"))
     assert ret.status_code == 200  # nosec
     assert ret.templates[0].name == "lava_scheduler_app/allworkers.html"  # nosec
     assert len(ret.context["worker_table"].data) == 4  # nosec
-    workers = set([x.hostname for x in ret.context["worker_table"].data])
+    workers = {x.hostname for x in ret.context["worker_table"].data}
     assert workers == set(
         ["worker_:')-,;~", "example.com", "worker-01", "worker-02"]
     )  # nosec
@@ -1190,6 +1222,25 @@ def test_device_health(client, setup):
     assert device.health == 3  # nosec
 
 
+@pytest.mark.django_db
+def test_device_health_reason_escaped(client, setup):
+    device = Device.objects.get(hostname="qemu01")
+    assert client.login(username="admin", password="admin") is True
+
+    reason = "<script>"
+    ret = client.post(
+        reverse("lava.scheduler.device.health", kwargs={"pk": device.hostname}),
+        {"health": "bad", "reason": reason},
+    )
+    assert ret.status_code == 302
+
+    log_entry = LogEntry.objects.filter(object_id=device.hostname).first()
+    assert log_entry is not None
+    msg = log_entry.get_change_message()
+    assert reason not in msg
+    assert "&lt;script&gt;" in msg
+
+
 @pytest.mark.django_db
 def test_worker_health_no_perm(client, setup):
     worker = Worker.objects.get(hostname="worker-01")
@@ -1215,6 +1266,25 @@ def test_worker_health(client, setup):
     assert worker.health == 1  # nosec
 
 
+def test_worker_health_reason_escapsed(client, setup):
+    worker = Worker.objects.get(hostname="worker-01")
+    assert worker.health == 0
+    assert client.login(username="admin", password="admin") is True
+
+    reason = "<script>"
+    ret = client.post(
+        reverse("lava.scheduler.worker.health", kwargs={"pk": worker.hostname}),
+        {"health": "Maintenance", "reason": reason},
+    )
+    assert ret.status_code == 302
+
+    log_entry = LogEntry.objects.filter(object_id=worker.hostname).first()
+    assert log_entry is not None
+    msg = log_entry.get_change_message()
+    assert reason not in msg
+    assert "&lt;script&gt;" in msg
+
+
 @pytest.mark.django_db
 def test_username_list_json_no_auth(client, setup):
     ret = client.get(reverse("lava.scheduler.username_list_json"), {"term": "t"})
@@ -1371,10 +1441,10 @@ def test_internal_v1_jobs_logs(client, setup, mocker):
     LOGS = """- {"dt": "2023-06-01T05:24:00.060423", "lvl": "info", "msg": "lava-dispatcher, installed at version: 2023.02"}
 - {"dt": "2023-06-01T05:24:00.060872", "lvl": "info", "msg": "start: 0 validate"}
 - {"dt": "2023-06-01T05:24:00.061082", "lvl": "info", "msg": "Start time: 2023-06-01 05:24:00.061063+00:00 (UTC)"}
-- {"dt": "2023-06-01T05:24:00.061300", "lvl": "debug", "msg": "Validating that http://example.com/artefacts/debug/bl1.bin exists"}
-- {"dt": "2023-06-01T05:24:00.435983", "lvl": "debug", "msg": "Validating that http://example.com/artefacts/debug/el3_payload.bin exists"}
-- {"dt": "2023-06-01T05:24:00.448713", "lvl": "debug", "msg": "Validating that http://example.com/artefacts/debug/fip.bin exists"}
-- {"dt": "2023-06-01T05:24:00.460928", "lvl": "debug", "msg": "Validating that http://example.com/artefacts/debug/ns_bl1u.bin exists"}
+- {"dt": "2023-06-01T05:24:00.061300", "lvl": "debug", "msg": "Validating that http://example.com/artifacts/debug/bl1.bin exists"}
+- {"dt": "2023-06-01T05:24:00.435983", "lvl": "debug", "msg": "Validating that http://example.com/artifacts/debug/el3_payload.bin exists"}
+- {"dt": "2023-06-01T05:24:00.448713", "lvl": "debug", "msg": "Validating that http://example.com/artifacts/debug/fip.bin exists"}
+- {"dt": "2023-06-01T05:24:00.460928", "lvl": "debug", "msg": "Validating that http://example.com/artifacts/debug/ns_bl1u.bin exists"}
 - {"dt": "2023-06-01T05:24:00.472695", "lvl": "info", "msg": "validate duration: 0.41"}
 - {"dt": "2023-06-01T05:24:00.472852", "lvl": "results", "msg": {"case": "validate", "definition": "lava", "result": "pass"}}
 - {"dt": "2023-06-01T05:24:00.473085", "lvl": "info", "msg": "start: 1 fvp-deploy (timeout 00:05:00) [common]"}
@@ -1476,3 +1546,130 @@ def test_internal_v1_jobs_logs(client, setup, mocker):
     )
     assert ret.status_code == 413
     assert ret.content.decode("utf-8") == REQUEST_DATA_TOO_BIG_MSG
+
+
+class TestInPlaceTokenUpdater:
+    @pytest.fixture
+    def actions(self):
+        return yaml_safe_load(
+            """
+- deploy:
+    to: tftp
+    kernel:
+        url: https://example.com/Image.gz
+        headers:
+            Authorization: token_name
+    dtb:
+        url: https://example.com/device.dtb
+        headers:
+            Authorization: real_token
+- deploy:
+    to: downloads
+    images:
+        nfsrootfs:
+            url: https://example.com/rootfs.ext4.xz
+            headers:
+                Authorization: token_name
+            compression: xz
+            format: tar
+            overlays:
+                modules:
+                    url: https://example.com/modules.tar.xz
+                    path: "/usr/"
+                    format: tar
+                    compression: xz
+                    headers:
+                        Authorization: token_name
+- test:
+    definitions:
+    - name: test
+      compression: xz
+      from: url
+      path: test.yaml
+      repository: https://example.com/testdef.tar.xz
+      headers:
+        Authorization: token_name
+            """
+        )
+
+    @pytest.fixture
+    def expected_actions(self):
+        return yaml_safe_load(
+            """
+- deploy:
+    to: tftp
+    kernel:
+        url: https://example.com/Image.gz
+        headers:
+            Authorization: token_value
+    dtb:
+        url: https://example.com/device.dtb
+        headers:
+            Authorization: real_token
+- deploy:
+    to: downloads
+    images:
+        nfsrootfs:
+            url: https://example.com/rootfs.ext4.xz
+            headers:
+                Authorization: token_value
+            compression: xz
+            format: tar
+            overlays:
+                modules:
+                    url: https://example.com/modules.tar.xz
+                    path: "/usr/"
+                    format: tar
+                    compression: xz
+                    headers:
+                        Authorization: token_value
+- test:
+    definitions:
+    - name: test
+      compression: xz
+      from: url
+      path: test.yaml
+      repository: https://example.com/testdef.tar.xz
+      headers:
+        Authorization: token_value
+            """
+        )
+
+    @pytest.fixture
+    def secrets(self):
+        return yaml_safe_load(
+            """
+secrets:
+    secret1: secret1_name
+    secret2: real_secret
+            """
+        )
+
+    @pytest.fixture
+    def expected_secrets(self):
+        return yaml_safe_load(
+            """
+secrets:
+    secret1: secret1_value
+    secret2: real_secret
+            """
+        )
+
+    @pytest.fixture
+    def tokens(self):
+        return {
+            "token_name": "token_value",
+            "secret1_name": "secret1_value",
+        }
+
+    def test_update_headers(self, tokens, actions, expected_actions):
+        updater = InPlaceTokenUpdater(tokens)
+        for action in actions:
+            if "deploy" in action or "test" in action:
+                updater.update_headers(action)
+        assert actions == expected_actions
+
+    def test_update_secrets(self, tokens, secrets, expected_secrets):
+        updater = InPlaceTokenUpdater(tokens)
+        updater.update_secrets(secrets["secrets"])
+        assert secrets == expected_secrets
